@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -29,8 +37,7 @@
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #include <gst/tag/tag.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "rb-debug.h"
 #include "rb-marshal.h"
@@ -41,6 +48,7 @@
 #include "rb-player-gst-tee.h"
 /*#include "rb-player-gst-data-tee.h"*/
 #include "rb-player-gst.h"
+#include "rb-player-gst-helper.h"
 
 static void rb_player_init (RBPlayerIface *iface);
 static void rb_player_gst_filter_init (RBPlayerGstFilterIface *iface);
@@ -90,6 +98,14 @@ enum
 	PROP_PLAYBIN
 };
 
+enum
+{
+	MISSING_PLUGINS,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 struct _RBPlayerGstPrivate
 {
 	char *uri;
@@ -97,6 +113,7 @@ struct _RBPlayerGstPrivate
 	GDestroyNotify stream_data_destroy;
 
 	GstElement *playbin;
+	GstElement *volume_handler;
 
 	gboolean can_signal_direct_error;
 	GError *error;
@@ -160,6 +177,17 @@ rb_player_gst_class_init (RBPlayerGstClass *klass)
 							      "playbin element",
 							      GST_TYPE_ELEMENT,
 							      G_PARAM_READABLE));
+	
+	signals[MISSING_PLUGINS] =
+		g_signal_new ("missing-plugins",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      0,	/* no point handling this internally */
+			      NULL, NULL,
+			      rb_marshal_VOID__POINTER_POINTER_POINTER,
+			      G_TYPE_NONE,
+			      3,
+			      G_TYPE_POINTER, G_TYPE_STRV, G_TYPE_STRV);
 
 	g_type_class_add_private (klass, sizeof (RBPlayerGstPrivate));
 }
@@ -236,6 +264,10 @@ rb_player_gst_finalize (GObject *object)
 
 		rb_player_gst_gst_free_playbin (mp);
 	}
+	if (mp->priv->volume_handler) {
+		g_object_unref (mp->priv->volume_handler);
+		mp->priv->volume_handler = NULL;
+	}
 
 	if (mp->priv->waiting_tees) {
 		g_list_foreach (mp->priv->waiting_tees, (GFunc)gst_object_sink, NULL);
@@ -372,6 +404,7 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayerGst *player)
 		rb_debug ("Could not transform tag value type %s into %s",
 			  g_type_name (G_VALUE_TYPE (val)),
 			  g_type_name (G_VALUE_TYPE (newval)));
+		g_free (newval);
 		return;
 	}
 
@@ -389,6 +422,38 @@ process_tag (const GstTagList *list, const gchar *tag, RBPlayerGst *player)
 	g_hash_table_insert (player->priv->idle_info_ids, GUINT_TO_POINTER (signal->id), NULL);
 }
 
+static void
+rb_player_gst_handle_missing_plugin_message (RBPlayerGst *player, GstMessage *message)
+{
+	char **details;
+	char **descriptions;
+	char *detail;
+	char *description;
+	int count;
+
+	rb_debug ("got missing-plugin message from %s: %s",
+		  GST_OBJECT_NAME (GST_MESSAGE_SRC (message)),
+		  gst_missing_plugin_message_get_installer_detail (message));
+
+	/* probably need to wait to collect any subsequent missing-plugin
+	 * messages, but I think we'd need to wait for state changes to do
+	 * that.  for now, we can only handle a single message.
+	 */
+	count = 1;
+
+	details = g_new0 (char *, count + 1);
+	descriptions = g_new0 (char *, count + 1);
+
+	detail = gst_missing_plugin_message_get_installer_detail (message);
+	description = gst_missing_plugin_message_get_description (message);
+	details[0] = g_strdup (detail);
+	descriptions[0] = g_strdup (description);
+
+	g_signal_emit (player, signals[MISSING_PLUGINS], 0, player->priv->stream_data, details, descriptions);
+	g_strfreev (details);
+	g_strfreev (descriptions);
+}
+
 static gboolean
 rb_player_gst_bus_cb (GstBus * bus, GstMessage * message, RBPlayerGst *mp)
 {
@@ -504,6 +569,12 @@ rb_player_gst_bus_cb (GstBus * bus, GstMessage * message, RBPlayerGst *mp)
 		g_value_set_string (signal->info, gst_structure_get_name (structure));
 		g_idle_add ((GSourceFunc) emit_signal_idle, signal);
 	}
+	case GST_MESSAGE_ELEMENT: {
+		if (gst_is_missing_plugin_message (message)) {
+			rb_player_gst_handle_missing_plugin_message (mp, message);
+		}
+		break;
+	}
 	default:
 		break;
 	}
@@ -534,17 +605,13 @@ rb_player_gst_construct (RBPlayerGst *mp, GError **error)
 	/* Use gconfaudiosink for audio if there's no audio sink yet */
 	g_object_get (G_OBJECT (mp->priv->playbin), "audio-sink", &sink, NULL);
 	if (sink == NULL) {
-		sink = gst_element_factory_make ("gconfaudiosink", "audiosink");
+		sink = rb_player_gst_try_audio_sink ("gconfaudiosink", "audiosink");
 		if (sink == NULL) {
 			/* fall back to autoaudiosink */
-			sink = gst_element_factory_make ("autoaudiosink", "audiosink");
+			sink = rb_player_gst_try_audio_sink ("autoaudiosink", "audiosink");
 		}
 
 		if (sink != NULL) {
-			/* set the profile property on the gconfaudiosink to "music and movies" */
-			if (g_object_class_find_property (G_OBJECT_GET_CLASS (sink), "profile"))
-				g_object_set (G_OBJECT (sink), "profile", 1, NULL);
-
 			g_object_set (G_OBJECT (mp->priv->playbin), "audio-sink", sink, NULL);
 		}
 	} else {
@@ -630,7 +697,7 @@ missing_element:
 		g_set_error (error,
 			     RB_PLAYER_ERROR,
 			     RB_PLAYER_ERROR_GENERAL,
-			     err);
+			     "%s", err);
 		g_free (err);
 		rb_player_gst_gst_free_playbin (mp);
 		return FALSE;
@@ -662,6 +729,7 @@ rb_player_gst_sync_pipeline (RBPlayerGst *mp)
 			return FALSE;
 		}
 	}
+
 	/* FIXME: Set up a timeout to watch if the pipeline doesn't
 	 * go to PAUSED/PLAYING within some time (5 secs maybe?)
 	 */
@@ -694,6 +762,7 @@ end_gstreamer_operation (RBPlayerGst *mp, gboolean op_failed, GError **error)
 			     RB_PLAYER_ERROR_GENERAL,
 			     _("Unknown playback error"));
 	}
+
 }
 
 static void
@@ -872,6 +941,12 @@ rb_player_gst_close (RBPlayer *player, const char *uri, GError **error)
 	begin_gstreamer_operation (mp);
 	ret = gst_element_set_state (mp->priv->playbin, GST_STATE_READY) == GST_STATE_CHANGE_SUCCESS;
 	end_gstreamer_operation (mp, !ret, error);
+
+	if (mp->priv->volume_handler) {
+		g_object_unref (mp->priv->volume_handler);
+		mp->priv->volume_handler = NULL;
+	}
+
 	return ret;
 }
 
@@ -934,6 +1009,47 @@ rb_player_gst_playing (RBPlayer *player)
 	return mp->priv->playing;
 }
 
+static gboolean
+emit_volume_changed_idle (RBPlayerGst *mp)
+{
+	_rb_player_emit_volume_changed (RB_PLAYER (mp), mp->priv->cur_volume);
+	return FALSE;
+}
+
+static void
+stream_volume_changed (GObject *element, GParamSpec *pspec, RBPlayerGst *mp)
+{
+	gdouble v;
+	g_object_get (element, "volume", &v, NULL);
+	mp->priv->cur_volume = v;
+
+	g_idle_add ((GSourceFunc) emit_volume_changed_idle, mp);
+}
+
+static void
+find_volume_handler (RBPlayerGst *mp)
+{
+	/* look for a 'volume' property provided by the sink */
+	if (mp->priv->volume_handler == NULL && mp->priv->playbin != NULL) {
+		GstElement *sink;
+
+		g_object_get (mp->priv->playbin, "audio-sink", &sink, NULL);
+		if (sink != NULL) {
+			mp->priv->volume_handler = rb_player_gst_find_element_with_property (sink, "volume");
+			g_object_unref (sink);
+		}
+
+		if (mp->priv->volume_handler == NULL) {
+			mp->priv->volume_handler = g_object_ref (mp->priv->playbin);
+		}
+
+		g_signal_connect_object (mp->priv->volume_handler,
+					 "notify::volume",
+					 G_CALLBACK (stream_volume_changed),
+					 mp, 0);
+	}
+}
+
 static void
 rb_player_gst_set_replaygain (RBPlayer *player,
 			      const char *uri,
@@ -970,11 +1086,12 @@ rb_player_gst_set_replaygain (RBPlayer *player,
 
 	rb_debug ("Scale : %f New volume : %f", scale, mp->priv->cur_volume * scale);
 
-	if (mp->priv->playbin != NULL) {
+	find_volume_handler (mp);
+	if (mp->priv->volume_handler != NULL) {
 		GParamSpec *volume_pspec;
 		GValue val = {0,};
 
-		volume_pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (mp->priv->playbin),
+		volume_pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (mp->priv->volume_handler),
 							     "volume");
 		g_value_init (&val, G_TYPE_DOUBLE);
 
@@ -982,7 +1099,7 @@ rb_player_gst_set_replaygain (RBPlayer *player,
 		if (g_param_value_validate (volume_pspec, &val))
 			rb_debug ("replay gain too high, reducing value to %f", g_value_get_double (&val));
 
-		g_object_set_property (G_OBJECT (mp->priv->playbin), "volume", &val);
+		g_object_set_property (G_OBJECT (mp->priv->volume_handler), "volume", &val);
 		g_value_unset (&val);
 	}
 }
@@ -994,11 +1111,9 @@ rb_player_gst_set_volume (RBPlayer *player,
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 	g_return_if_fail (volume >= 0.0 && volume <= 1.0);
 
-	if (mp->priv->playbin != NULL) {
-		g_object_set (G_OBJECT (mp->priv->playbin),
-			      "volume",
-			      volume,
-			      NULL);
+	find_volume_handler (mp);
+	if (mp->priv->volume_handler != NULL) {
+		g_object_set (mp->priv->volume_handler, "volume", volume, NULL);
 	}
 
 	mp->priv->cur_volume = volume;
