@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Authors: Ramiro Estrugo <ramiro@eazel.com>
 */
@@ -25,7 +25,7 @@
 #ifndef EEL_GCONF_EXTENSIONS_H
 #define EEL_GCONF_EXTENSIONS_H
 
-#include <glib/gerror.h>
+#include <glib.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 
