@@ -423,7 +423,50 @@ rb_marshal_OBJECT__OBJECT (GClosure     *closure,
   g_value_take_object (return_value, v_return);
 }
 
-/* STRING:STRING (./rb-marshal.list:9) */
+/* OBJECT:VOID (./rb-marshal.list:9) */
+extern void rb_marshal_OBJECT__VOID (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data);
+void
+rb_marshal_OBJECT__VOID (GClosure     *closure,
+                         GValue       *return_value G_GNUC_UNUSED,
+                         guint         n_param_values,
+                         const GValue *param_values,
+                         gpointer      invocation_hint G_GNUC_UNUSED,
+                         gpointer      marshal_data)
+{
+  typedef GObject* (*GMarshalFunc_OBJECT__VOID) (gpointer     data1,
+                                                 gpointer     data2);
+  register GMarshalFunc_OBJECT__VOID callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  GObject* v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 1);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_OBJECT__VOID) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       data2);
+
+  g_value_take_object (return_value, v_return);
+}
+
+/* STRING:STRING (./rb-marshal.list:10) */
 extern void rb_marshal_STRING__STRING (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
@@ -468,7 +511,7 @@ rb_marshal_STRING__STRING (GClosure     *closure,
   g_value_take_string (return_value, v_return);
 }
 
-/* VOID:BOOLEAN,BOOLEAN (./rb-marshal.list:10) */
+/* VOID:BOOLEAN,BOOLEAN (./rb-marshal.list:11) */
 extern void rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -511,7 +554,7 @@ rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* BOXED:BOXED (./rb-marshal.list:11) */
+/* BOXED:BOXED (./rb-marshal.list:12) */
 extern void rb_marshal_BOXED__BOXED (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
@@ -556,7 +599,52 @@ rb_marshal_BOXED__BOXED (GClosure     *closure,
   g_value_take_boxed (return_value, v_return);
 }
 
-/* VOID:BOXED,BOXED (./rb-marshal.list:12) */
+/* BOXED:OBJECT (./rb-marshal.list:13) */
+extern void rb_marshal_BOXED__OBJECT (GClosure     *closure,
+                                      GValue       *return_value,
+                                      guint         n_param_values,
+                                      const GValue *param_values,
+                                      gpointer      invocation_hint,
+                                      gpointer      marshal_data);
+void
+rb_marshal_BOXED__OBJECT (GClosure     *closure,
+                          GValue       *return_value G_GNUC_UNUSED,
+                          guint         n_param_values,
+                          const GValue *param_values,
+                          gpointer      invocation_hint G_GNUC_UNUSED,
+                          gpointer      marshal_data)
+{
+  typedef gpointer (*GMarshalFunc_BOXED__OBJECT) (gpointer     data1,
+                                                  gpointer     arg_1,
+                                                  gpointer     data2);
+  register GMarshalFunc_BOXED__OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gpointer v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOXED__OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_object (param_values + 1),
+                       data2);
+
+  g_value_take_boxed (return_value, v_return);
+}
+
+/* VOID:BOXED,BOXED (./rb-marshal.list:14) */
 extern void rb_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -599,7 +687,7 @@ rb_marshal_VOID__BOXED_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:13) */
+/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:15) */
 extern void rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -646,7 +734,7 @@ rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,OBJECT (./rb-marshal.list:14) */
+/* VOID:BOXED,OBJECT (./rb-marshal.list:16) */
 extern void rb_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -689,7 +777,7 @@ rb_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,POINTER (./rb-marshal.list:15) */
+/* VOID:BOXED,POINTER (./rb-marshal.list:17) */
 extern void rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -732,7 +820,7 @@ rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,STRING,BOXED (./rb-marshal.list:16) */
+/* VOID:BOXED,STRING,BOXED (./rb-marshal.list:18) */
 extern void rb_marshal_VOID__BOXED_STRING_BOXED (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -777,7 +865,7 @@ rb_marshal_VOID__BOXED_STRING_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,ULONG (./rb-marshal.list:17) */
+/* VOID:BOXED,ULONG (./rb-marshal.list:19) */
 extern void rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -820,7 +908,7 @@ rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:DOUBLE,LONG (./rb-marshal.list:18) */
+/* VOID:DOUBLE,LONG (./rb-marshal.list:20) */
 extern void rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -863,7 +951,7 @@ rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT64 (./rb-marshal.list:19) */
+/* VOID:UINT64 (./rb-marshal.list:21) */
 extern void rb_marshal_VOID__UINT64 (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
@@ -904,7 +992,7 @@ rb_marshal_VOID__UINT64 (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT (./rb-marshal.list:20) */
+/* VOID:INT,INT (./rb-marshal.list:22) */
 extern void rb_marshal_VOID__INT_INT (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
@@ -947,7 +1035,7 @@ rb_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,DOUBLE (./rb-marshal.list:21) */
+/* VOID:INT,INT,DOUBLE (./rb-marshal.list:23) */
 extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -992,7 +1080,7 @@ rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT (./rb-marshal.list:22) */
+/* VOID:OBJECT,INT,INT (./rb-marshal.list:24) */
 extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1037,7 +1125,7 @@ rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:23) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:25) */
 extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -1088,7 +1176,7 @@ rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:24) */
+/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:26) */
 extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -1133,7 +1221,7 @@ rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT (./rb-marshal.list:25) */
+/* VOID:POINTER,INT (./rb-marshal.list:27) */
 extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -1176,7 +1264,7 @@ rb_marshal_VOID__POINTER_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT,POINTER (./rb-marshal.list:26) */
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:28) */
 extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -1221,7 +1309,7 @@ rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,LONG,LONG (./rb-marshal.list:27) */
+/* VOID:POINTER,LONG,LONG (./rb-marshal.list:29) */
 extern void rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -1266,7 +1354,7 @@ rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER (./rb-marshal.list:28) */
+/* VOID:POINTER,POINTER (./rb-marshal.list:30) */
 extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -1309,7 +1397,7 @@ rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER,POINTER (./rb-marshal.list:29) */
+/* VOID:POINTER,POINTER,POINTER (./rb-marshal.list:31) */
 extern void rb_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -1354,7 +1442,7 @@ rb_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,UINT (./rb-marshal.list:30) */
+/* VOID:POINTER,UINT (./rb-marshal.list:32) */
 extern void rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -1397,7 +1485,7 @@ rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,ULONG (./rb-marshal.list:31) */
+/* VOID:POINTER,ULONG (./rb-marshal.list:33) */
 extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1440,7 +1528,7 @@ rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,DOUBLE (./rb-marshal.list:32) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:34) */
 extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1483,7 +1571,7 @@ rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./rb-marshal.list:33) */
+/* VOID:STRING,INT (./rb-marshal.list:35) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -1526,7 +1614,7 @@ rb_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,POINTER (./rb-marshal.list:34) */
+/* VOID:STRING,POINTER (./rb-marshal.list:36) */
 extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1569,7 +1657,7 @@ rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING (./rb-marshal.list:35) */
+/* VOID:STRING,STRING (./rb-marshal.list:37) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1612,7 +1700,7 @@ rb_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:36) */
+/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:38) */
 extern void rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -1657,7 +1745,7 @@ rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:37) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:39) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -1702,7 +1790,7 @@ rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:38) */
+/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:40) */
 extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -1751,7 +1839,7 @@ rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:ULONG,FLOAT (./rb-marshal.list:39) */
+/* VOID:ULONG,FLOAT (./rb-marshal.list:41) */
 extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -1794,7 +1882,7 @@ rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:40) */
+/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:42) */
 extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1837,7 +1925,7 @@ rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:41) */
+/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:43) */
 extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
