@@ -20,11 +20,7 @@
 #include "config.h"
 
 #include <string.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtktreednd.h>
+#include <gtk/gtk.h>
 #include "rb-tree-dnd.h"
 
 #include "rb-debug.h"
@@ -478,7 +474,7 @@ scroll_row_timeout (gpointer data)
 	g_return_val_if_fail(priv_data != NULL, TRUE);
 
 	gdk_window_get_pointer (gtk_tree_view_get_bin_window (tree_view), &x, &y, NULL);
-	gtk_tree_view_widget_to_tree_coords (tree_view, x, y, &x, &y);
+	gtk_tree_view_convert_widget_to_bin_window_coords (tree_view, x, y, &x, &y);
 	gtk_tree_view_get_visible_rect (tree_view, &visible_rect);
 
 	/* see if we are near the edge. */
@@ -749,7 +745,7 @@ rb_tree_dnd_drag_motion_cb (GtkWidget        *widget,
 			priv_data->previous_dest_path = path;
 			if (priv_data->select_on_drag_timeout == 0) {
 				rb_debug("Setting up a new select on drag timeout");
-				priv_data->select_on_drag_timeout = g_timeout_add (2000, select_on_drag_timeout, tree_view);
+				priv_data->select_on_drag_timeout = g_timeout_add_seconds (2, select_on_drag_timeout, tree_view);
 			}
 		} else {
 			gtk_tree_path_free (path);
