@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -26,10 +34,14 @@
 #include <stdlib.h>
 
 #include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <glib/gi18n.h>
 #include <gobject/gvaluecollector.h>
+#include <gio/gio.h>
+
+#ifdef HAVE_HAL
+#include <libhal.h>
+#include <dbus/dbus.h>
+#endif
 
 #include "rb-util.h"
 #include "rb-debug.h"
@@ -267,7 +279,6 @@ totem_pixbuf_mirror (GdkPixbuf *pixbuf)
 GtkWidget *
 rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size)
 {
-
 	if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_LTR) {
 		return gtk_image_new_from_stock (stock_id, size);
 	} else {
@@ -287,14 +298,14 @@ rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size)
 		
 		
 		mirror = gdk_pixbuf_copy (pixbuf);
-		gdk_pixbuf_unref (pixbuf);
+		g_object_unref (pixbuf);
 
 		if (!mirror)
 			return NULL;
 
 		totem_pixbuf_mirror (mirror);
 		gtk_image_set_from_pixbuf (GTK_IMAGE (image), mirror);
-		gdk_pixbuf_unref (mirror);
+		g_object_unref (mirror);
 
 		return image;
 	}
@@ -311,89 +322,11 @@ rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path)
 	if (menu == NULL) {
 		g_warning ("Couldn't get menu widget for %s", path);
 	} else {
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, 
-			gtk_get_current_event_time ());
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, 
+				gtk_get_current_event_time ());
 	}
 }
 
-static GList *
-get_mount_points (void)
-{
-	GnomeVFSVolumeMonitor *monitor;
-	GList *volumes;
-	GList *it;
-	GList *mount_points = NULL;
-
-	monitor = gnome_vfs_get_volume_monitor ();
-	/* FIXME: should also get the list of connected drivers (network
-	 * shares I assume)
-	 */
-	volumes = gnome_vfs_volume_monitor_get_mounted_volumes (monitor);
-
-	for (it = volumes; it != NULL; it = it->next) {
-		gchar *uri;
-		GnomeVFSVolume *volume;
-
-		volume = GNOME_VFS_VOLUME (it->data);
-		uri = gnome_vfs_volume_get_activation_uri (volume);
-		g_assert (uri != NULL);
-		mount_points = g_list_prepend (mount_points, uri);
-	}
-
-	g_list_foreach (volumes, (GFunc)gnome_vfs_volume_ref, NULL);
-	g_list_free (volumes);
-
-	return mount_points;
-}
-
-
-gchar *
-rb_uri_get_mount_point (const char *uri)
-{
-	GList *mount_points = get_mount_points ();
-	GList *it;
-	gchar *mount_point = NULL;
-
-	for (it = mount_points; it != NULL; it = it->next) {
-		if (g_str_has_prefix (uri, it->data)) {
-			if ((mount_point == NULL) || (strlen (mount_point) < strlen (it->data))) {
-				g_free (mount_point);
-				mount_point = g_strdup (it->data);
-			}
-		}
-	}
-	g_list_foreach (mount_points, (GFunc)g_free, NULL);
-	g_list_free (mount_points);
-
-	return mount_point;
-}
-
-gboolean
-rb_uri_is_mounted (const char *uri)
-{
-	GList *mount_points = get_mount_points ();
-	GList *it;
-	gboolean found = FALSE;
-
-	if ((uri == NULL) || (*uri == '\0')) {
-		return TRUE;
-	}
-
-	for (it = mount_points; it != NULL; it = it->next) {
-		if (strcmp (it->data, uri) == 0) {
-			found = TRUE;
-			break;
-		}
-	}
-	g_list_foreach (mount_points, (GFunc)g_free, NULL);
-	g_list_free (mount_points);
-
-/*	if (found == FALSE) {
-		g_print ("%s not mounted\n", uri);
-		}*/
-
-	return found;
-}
 
 gboolean
 rb_is_main_thread (void)
@@ -460,7 +393,7 @@ rb_threads_init (void)
 	rb_debug ("GMutex %s recursive", mutex_recurses ? "is" : "isn't");
 
 	/* purge useless thread-pool threads occasionally */
-	g_timeout_add (30 * 1000, purge_useless_threads, NULL);
+	g_timeout_add_seconds (30, purge_useless_threads, NULL);
 }
 
 gchar **
@@ -877,13 +810,13 @@ rb_uri_list_parse (const char *uri_list)
 	return g_list_reverse (result);
 }
 
-const gchar*
-rb_mime_get_friendly_name (const gchar *mime_type)
+char*
+rb_mime_get_friendly_name (const char *mime_type)
 {
-	const gchar *name = NULL;
+	gchar *name = NULL;
 	
 	if (name == NULL && mime_type)
-		name = gnome_vfs_mime_get_description (mime_type);
+		name = g_content_type_get_description (mime_type);
 	if (name == NULL)
 		name = _("Unknown");
 
@@ -908,6 +841,45 @@ rb_signal_accumulator_object_handled (GSignalInvocationHint *hint,
 	return FALSE;
 }
 
+gboolean
+rb_signal_accumulator_value_array (GSignalInvocationHint *hint,
+				   GValue *return_accu,
+				   const GValue *handler_return,
+				   gpointer bleh)
+{
+	GValueArray *a;
+	GValueArray *b;
+	int i;
+
+	if (handler_return == NULL)
+		return TRUE;
+
+	a = NULL;
+	if (G_VALUE_HOLDS_BOXED (return_accu)) {
+		a = g_value_get_boxed (return_accu);
+		if (a != NULL) {
+			a = g_value_array_copy (a);
+		}
+	}
+
+	if (a == NULL) {
+		a = g_value_array_new (1);
+	}
+
+	if (G_VALUE_HOLDS_BOXED (handler_return)) {
+		b = g_value_get_boxed (handler_return);
+		for (i=0; i < b->n_values; i++) {
+			GValue *z = g_value_array_get_nth (b, i);
+			a = g_value_array_append (a, z);
+		}
+	}
+
+	g_value_unset (return_accu);
+	g_value_init (return_accu, G_TYPE_VALUE_ARRAY);
+	g_value_set_boxed (return_accu, a);
+	return TRUE;
+}
+
 void
 rb_value_array_append_data (GValueArray *array, GType type, ...)
 {
@@ -953,15 +925,14 @@ rb_str_in_strv (const char *needle, char **haystack)
 
 /**
  * rb_set_tree_view_column_fixed_width:
- *
- * Sets a fixed size for a tree view column based on
- * a set of strings to be displayed in the column.
- *
  * @treeview: the #GtkTreeView containing the column
  * @column: the #GtkTreeViewColumn to size
  * @renderer: the #GtkCellRenderer used in the column
  * @strings: a NULL-terminated set of strings to base the size on
  * @padding: a small amount of extra padding for the column
+ *
+ * Sets a fixed size for a tree view column based on
+ * a set of strings to be displayed in the column.
  */
 void
 rb_set_tree_view_column_fixed_width (GtkWidget  *treeview,
@@ -991,3 +962,75 @@ rb_set_tree_view_column_fixed_width (GtkWidget  *treeview,
 	gtk_tree_view_column_set_fixed_width (column, max_width + padding);
 }
 
+/**
+ * rb_scale_pixbuf_to_size:
+ * @pixbuf: the #GdkPixbuf containing the original image
+ * @size: a stock icon size
+ *
+ * Creates a new #GdkPixbuf from the original one, for a target of
+ * size, respecting the aspect ratio of the image.
+ */
+GdkPixbuf *
+rb_scale_pixbuf_to_size (GdkPixbuf *pixbuf, GtkIconSize size)
+{
+	int icon_size;
+	int width, height;
+	int d_width, d_height;
+
+	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
+
+	if (gtk_icon_size_lookup (size, &icon_size, NULL) == FALSE)
+		return NULL;
+
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+
+	if (width > height) {
+		d_width = icon_size;
+		d_height = d_width * height / width;
+	} else {
+		d_height = icon_size;
+		d_width = d_height * width / height;
+	}
+
+	return gdk_pixbuf_scale_simple (pixbuf, d_width, d_height, GDK_INTERP_BILINEAR);
+}
+
+#ifdef HAVE_HAL
+char *
+rb_gvolume_get_udi (GVolume *volume, gpointer ctx)
+{
+	char *udi, *dev, **udis;
+	int num_udis;
+
+	g_return_val_if_fail (volume != NULL, NULL);
+	g_return_val_if_fail (G_IS_VOLUME (volume), NULL);
+	g_return_val_if_fail (ctx != NULL, NULL);
+
+	udi = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+	if (udi != NULL)
+		return udi;
+
+	dev = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	udis = libhal_manager_find_device_string_match ((LibHalContext *) ctx,
+							"block.device", dev,
+							&num_udis, NULL);
+
+	if (udis == NULL || num_udis < 1) {
+		libhal_free_string_array (udis);
+		return NULL;
+	}
+
+	udi = g_strdup (udis[0]);
+	libhal_free_string_array (udis);
+
+	return udi;
+}
+#else
+char *
+rb_gvolume_get_udi (GVolume *volume, gpointer ctx)
+{
+	return NULL;
+}
+#endif /* HAVE_HAL */
+
