@@ -6,6 +6,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -38,6 +46,14 @@
  * we actually send it the request.
  */
 
+/**
+ * SECTION:rb-metadata
+ * @short_description: metadata reader and writer interface
+ *
+ * Provides a simple synchronous interface for metadata extraction and updating.
+ *
+ */
+
 #include <config.h>
 
 #include "rb-metadata.h"
@@ -64,6 +80,7 @@ static void rb_metadata_finalize (GObject *object);
 static gboolean tried_env_address = FALSE;
 static DBusConnection *dbus_connection = NULL;
 static GPid metadata_child = 0;
+static int metadata_stdout = -1;
 static GMainContext *main_context = NULL;
 static GStaticMutex conn_mutex = G_STATIC_MUTEX_INIT;
 
@@ -71,6 +88,8 @@ struct RBMetaDataPrivate
 {
 	char       *uri;
 	char       *mimetype;
+	char      **missing_plugins;
+	char      **plugin_descriptions;
 	GHashTable *metadata;
 };
 
@@ -109,6 +128,11 @@ rb_metadata_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_metadata_parent_class)->finalize (object);
 }
 
+/**
+ * rb_metadata_new:
+ *
+ * Return value: new #RBMetaData instance
+ */
 RBMetaData *
 rb_metadata_new (void)
 {
@@ -135,6 +159,12 @@ kill_metadata_service (void)
 		g_spawn_close_pid (metadata_child);
 		metadata_child = 0;
 	}
+
+	if (metadata_stdout != -1) {
+		rb_debug ("closing metadata child process stdout pipe");
+		close (metadata_stdout);
+		metadata_stdout = -1;
+	}
 }
 
 static gboolean
@@ -202,7 +232,6 @@ start_metadata_service (GError **error)
 	}
 
 	if (dbus_address == NULL) {
-		gint metadata_stdout;
 		GPtrArray *argv;
 		gboolean res;
 		char **debug_args;
@@ -215,7 +244,7 @@ start_metadata_service (GError **error)
 		 * but when --enable-uninstalled-build is specified, we look
 		 * in the directory it's built in.
 		 */
-#ifdef METADATA_UNINSTALLED_DIR
+#ifdef USE_UNINSTALLED_DIRS
 		g_ptr_array_add (argv, METADATA_UNINSTALLED_DIR "/rhythmbox-metadata");
 #else
 		g_ptr_array_add (argv, LIBEXEC_DIR G_DIR_SEPARATOR_S INSTALLED_METADATA_HELPER);
@@ -320,6 +349,17 @@ read_error_from_message (RBMetaData *md, DBusMessageIter *iter, GError **error)
 	g_free (error_message);
 }
 
+/**
+ * rb_metadata_load:
+ * @md: a #RBMetaData
+ * @uri: URI from which to load metadata
+ * @error: returns error information
+ *
+ * Reads metadata information from the specified URI.
+ * Once this has returned successfully (with *error == NULL),
+ * rb_metadata_get, rb_metadata_get_mime, rb_metadata_has_missing_plugins,
+ * and rb_metadata_get_missing_plugins can usefully be called.
+ */
 void
 rb_metadata_load (RBMetaData *md,
 		  const char *uri,
@@ -389,6 +429,35 @@ rb_metadata_load (RBMetaData *md,
 			rb_debug ("couldn't read response message");
 		}
 	}
+	
+	if (*error == NULL) {
+		if (!rb_metadata_dbus_get_strv (&iter, &md->priv->missing_plugins)) {
+			g_set_error (error,
+				     RB_METADATA_ERROR,
+				     RB_METADATA_ERROR_INTERNAL,
+				     _("D-BUS communication error"));
+			rb_debug ("couldn't get missing plugin data from response message");
+		}
+	}
+
+	if (*error == NULL) {
+		if (!rb_metadata_dbus_get_strv (&iter, &md->priv->plugin_descriptions)) {
+			g_set_error (error,
+				     RB_METADATA_ERROR,
+				     RB_METADATA_ERROR_INTERNAL,
+				     _("D-BUS communication error"));
+			rb_debug ("couldn't get missing plugin descriptions from response message");
+		}
+	}
+
+	/* if we're missing some plugins, we'll need to make sure the
+	 * metadata helper rereads the registry before the next load.
+	 * the easiest way to do this is to kill it.
+	 */
+	if (*error == NULL && md->priv->missing_plugins != NULL) {
+		rb_debug ("missing plugins; killing metadata service to force registry reload");
+		kill_metadata_service ();
+	}
 
 	if (*error == NULL) {
 		if (!rb_metadata_dbus_get_boolean (&iter, &ok)) {
@@ -397,23 +466,26 @@ rb_metadata_load (RBMetaData *md,
 				     RB_METADATA_ERROR_INTERNAL,
 				     _("D-BUS communication error"));
 			rb_debug ("couldn't get success flag from response message");
-		} else if (ok) {
-			/* get mime type */
-			if (!rb_metadata_dbus_get_string (&iter, &md->priv->mimetype)) {
-				g_set_error (error,
-					     RB_METADATA_ERROR,
-					     RB_METADATA_ERROR_INTERNAL,
-					     _("D-BUS communication error"));
-			} else {
-				/* get metadata */
-				rb_debug ("got mimetype: %s", md->priv->mimetype);
-				rb_metadata_dbus_read_from_message (md, md->priv->metadata, &iter);
-			}
-		} else {
+		} else if (ok == FALSE) {
 			read_error_from_message (md, &iter, error);
 		}
 	}
 
+	if (ok && *error == NULL) {
+		if (!rb_metadata_dbus_get_string (&iter, &md->priv->mimetype)) {
+			g_set_error (error,
+				     RB_METADATA_ERROR,
+				     RB_METADATA_ERROR_INTERNAL,
+				     _("D-BUS communication error"));
+		} else {
+			rb_debug ("got mimetype: %s", md->priv->mimetype);
+		}
+	}
+
+	if (ok && *error == NULL) {
+		rb_metadata_dbus_read_from_message (md, md->priv->metadata, &iter);
+	}
+
 	if (message)
 		dbus_message_unref (message);
 	if (response)
@@ -424,15 +496,78 @@ rb_metadata_load (RBMetaData *md,
 	g_static_mutex_unlock (&conn_mutex);
 }
 
+/**
+ * rb_metadata_get_mime:
+ * @md: a #RBMetaData
+ *
+ * Returns the type of the file from which metadata was read.
+ * This isn't really a MIME type, but it looks like one.
+ *
+ * Return value: MIME type-ish string
+ */
 const char *
 rb_metadata_get_mime (RBMetaData *md)
 {
 	return md->priv->mimetype;
 }
 
+/**
+ * rb_metadata_has_missing_plugins:
+ * @md: a #RBMetaData
+ *
+ * If the metadata reader could not decode the file it was asked to
+ * because one or more media framework plugins (specifically, for the
+ * existing implementations, GStreamer plugins) required are missing,
+ * this will return TRUE.
+ *
+ * Return value: TRUE if required plugins are missing
+ */
 gboolean
-rb_metadata_get (RBMetaData *md, RBMetaDataField field,
-		 GValue *ret)
+rb_metadata_has_missing_plugins (RBMetaData *md)
+{
+	return (md->priv->missing_plugins != NULL);
+}
+
+/**
+ * rb_metadata_get_missing_plugins:
+ * @md: a #RBMetaData
+ * @missing_plugins: returns machine-readable missing plugin information
+ * @plugin_descriptions: returns human-readable missing plugin descriptions
+ *
+ * This function returns the information used to request automatic
+ * installation of media framework plugins required to decode the target URI.
+ * Use g_strfreev() to free the returned information arrays.
+ *
+ * Return value: TRUE if missing plugin information was returned
+ */
+gboolean
+rb_metadata_get_missing_plugins (RBMetaData *md,
+				 char ***missing_plugins,
+				 char ***plugin_descriptions)
+{
+	if (md->priv->missing_plugins == NULL) {
+		return FALSE;
+	}
+
+	*missing_plugins = g_strdupv (md->priv->missing_plugins);
+	*plugin_descriptions = g_strdupv (md->priv->plugin_descriptions);
+	return TRUE;
+}
+
+
+/**
+ * rb_metadata_get:
+ * @md: a #RBMetaData
+ * @field: the #RBMetaDataField to retrieve
+ * @val: returns the field value
+ *
+ * Retrieves the value of a metadata field extracted from the target URI.
+ * If the target URI contained no value for the field, returns FALSE.
+ *
+ * Return value: TRUE if a value was returned
+ */
+gboolean
+rb_metadata_get (RBMetaData *md, RBMetaDataField field, GValue *ret)
 {
 	GValue *val;
 	if (!md->priv->metadata)
@@ -447,6 +582,17 @@ rb_metadata_get (RBMetaData *md, RBMetaDataField field,
 	return FALSE;
 }
 
+/**
+ * rb_metadata_set:
+ * @md: a #RBMetaData
+ * @field: the #RBMetaDataField to set
+ * @val: the vaule to set
+ *
+ * Sets a metadata field value.  The value is only stored inside the
+ * #RBMetaData object until rb_metadata_save is called.
+ *
+ * Return value: TRUE if the field is valid
+ */
 gboolean
 rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 		 const GValue *val)
@@ -466,6 +612,16 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 	return TRUE;
 }
 
+/**
+ * rb_metadata_can_save:
+ * @md: a #RBMetaData
+ * @mimetype: the MIME type-ish string to check
+ *
+ * Checks if the metadata writer is capable of updating file metadata
+ * for a given media type.
+ *
+ * Return value: TRUE if the file metadata for the given media type can be updated
+ */
 gboolean
 rb_metadata_can_save (RBMetaData *md, const char *mimetype)
 {
@@ -518,6 +674,14 @@ rb_metadata_can_save (RBMetaData *md, const char *mimetype)
 	return can_save;
 }
 
+/**
+ * rb_metadata_save:
+ * @md: a #RBMetaData
+ * @error: returns error information
+ *
+ * Saves all metadata changes made with rb_metadata_set to the
+ * target URI.
+ */
 void
 rb_metadata_save (RBMetaData *md, GError **error)
 {
