@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -28,10 +36,8 @@
 #include <gst/gsttagsetter.h>
 #include <gst/tag/tag.h>
 #include <gst/gsturi.h>
-
-#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
+#include <gio/gio.h>
 #include <gst/pbutils/pbutils.h>
-#endif /* HAVE_GSTREAMER_0_10_MISSING_PLUGINS */
 
 #include "rb-metadata.h"
 #include "rb-debug.h"
@@ -53,7 +59,17 @@ const char * ignore_mime_types[] = {
 	"text/",
 	"application/xml",
 	"application/zip",
-	"application/x-executable"
+	"application/x-executable",
+	"application/x-ms-dos-executable",
+	"application/x-bzip",
+	"application/x-gzip",
+	"application/x-java",
+	"application/x-lha",
+	"application/pdf",
+	"application/postscript",
+	"application/x-rar",
+	"application/msword",
+	"application/octet-stream"
 };
 
 /*
@@ -303,6 +319,47 @@ error:
 	return NULL;
 }
 
+static void
+qt_pad_added_cb (GstElement *demux, GstPad *demuxpad, GstPad *muxpad)
+{
+	if (gst_pad_link (demuxpad, muxpad) != GST_PAD_LINK_OK)
+		rb_debug ("unable to link pad from qtdemux to mp4mux");
+	else
+		rb_debug ("linked pad from qtdemux to mp4mux");
+}
+
+
+static GstElement *
+rb_add_qt_tagger (RBMetaData *md, GstElement *element)
+{
+	GstElement *demux;
+	GstElement *mux;
+	GstPad *muxpad;
+
+	demux = gst_element_factory_make ("qtdemux", NULL);
+	mux = gst_element_factory_make ("mp4mux", NULL);
+	if (demux == NULL || mux == NULL)
+		goto error;
+
+	gst_bin_add_many (GST_BIN (md->priv->pipeline), demux, mux, NULL);
+	if (!gst_element_link (element, demux))
+		goto error;
+
+	muxpad = gst_element_get_request_pad (mux, "audio_%d");
+	g_signal_connect (demux, "pad-added", G_CALLBACK (qt_pad_added_cb), muxpad);
+	
+	gst_tag_setter_merge_tags (GST_TAG_SETTER (mux), md->priv->tags, GST_TAG_MERGE_REPLACE_ALL);
+
+	return mux;
+
+error:
+	if (demux != NULL)
+		g_object_unref (demux);
+	if (mux != NULL)
+		g_object_unref (mux);
+	return NULL;
+}
+
 static void
 add_supported_type (RBMetaData *md,
 		    const char *mime,
@@ -320,7 +377,7 @@ static void
 rb_metadata_init (RBMetaData *md)
 {
 	RBAddTaggerElem tagger;
-	gboolean has_gnomevfssink = FALSE;
+	gboolean has_giosink = FALSE;
 	gboolean has_id3 = FALSE;
 
 	md->priv = RB_METADATA_GET_PRIVATE (md);
@@ -336,10 +393,9 @@ rb_metadata_init (RBMetaData *md)
  	 * only registering types we have plugins for defeats the second
 	 * purpose.
  	 */
-	has_gnomevfssink = (gst_element_factory_find ("gnomevfssrc") != NULL &&
-			    gst_element_factory_find ("gnomevfssink") != NULL);
+	has_giosink = (gst_element_factory_find ("giostreamsink") != NULL);
 	has_id3 = (gst_element_factory_find ("id3v2mux") != NULL);
-	tagger = (has_gnomevfssink && has_id3) ?  rb_add_id3_tagger : NULL;
+	tagger = (has_giosink && has_id3) ?  rb_add_id3_tagger : NULL;
 	add_supported_type (md, "application/x-id3", tagger, "MP3");
 	add_supported_type (md, "audio/mpeg", tagger, "MP3");
 
@@ -350,7 +406,7 @@ rb_metadata_init (RBMetaData *md)
 			      gst_default_registry_check_feature_version ("vorbisparse", 0, 10, 6) &&
 			      gst_default_registry_check_feature_version ("oggmux", 0, 10, 6) &&
 			      gst_default_registry_check_feature_version ("oggdemux", 0, 10, 6));
-		tagger = (has_gnomevfssink && has_vorbis) ?  rb_add_ogg_tagger : NULL;
+		tagger = (has_giosink && has_vorbis) ?  rb_add_ogg_tagger : NULL;
 	}
 	add_supported_type (md, "application/ogg", tagger, "Ogg Vorbis");
 	add_supported_type (md, "audio/x-vorbis", tagger, "Ogg Vorbis");
@@ -359,9 +415,13 @@ rb_metadata_init (RBMetaData *md)
 	add_supported_type (md, "audio/x-wav", NULL, "WAV");
 	add_supported_type (md, "video/x-ms-asf", NULL, "ASF");
 
-	tagger = (has_gnomevfssink && gst_element_factory_find ("flactag")) ?  rb_add_flac_tagger : NULL;
+	tagger = (has_giosink && gst_element_factory_find ("flactag")) ?  rb_add_flac_tagger : NULL;
 	add_supported_type (md, "audio/x-flac", tagger, "FLAC");
 
+	tagger = (has_giosink && gst_element_factory_find ("qtdemux") && gst_element_factory_find ("mp4mux")) ? rb_add_qt_tagger : NULL;
+	add_supported_type (md, "audio/x-m4a", tagger, "M4A");
+	add_supported_type (md, "video/quicktime", tagger, "M4A");			/* hmm. */
+
 }
 
 static void
@@ -469,7 +529,9 @@ rb_metadata_gst_tag_to_field (const char *tag)
 	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_ALBUMARTISTID))
 		return RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID;
 	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_SORTNAME))
-		return RB_METADATA_FIELD_MUSICBRAINZ_ARTISTSORTNAME;
+		return RB_METADATA_FIELD_ARTIST_SORTNAME;
+	else if (!strcmp (tag, GST_TAG_ALBUM_SORTNAME))
+		return RB_METADATA_FIELD_ALBUM_SORTNAME;
 
 #endif
 	else
@@ -540,8 +602,10 @@ rb_metadata_gst_field_to_gst_tag (RBMetaDataField field)
 		return GST_TAG_MUSICBRAINZ_ALBUMID;
 	case RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID:
 		return GST_TAG_MUSICBRAINZ_ALBUMARTISTID;
-	case RB_METADATA_FIELD_MUSICBRAINZ_ARTISTSORTNAME:
+	case RB_METADATA_FIELD_ARTIST_SORTNAME:
 		return GST_TAG_MUSICBRAINZ_SORTNAME;
+	case RB_METADATA_FIELD_ALBUM_SORTNAME:
+		return GST_TAG_ALBUM_SORTNAME;
 #endif
 	default:
 		return NULL;
@@ -576,10 +640,13 @@ static char *
 make_undecodable_error (RBMetaData *md)
 {
 	const char *human_name;
+	char *free_name = NULL;
 
-	human_name= rb_metadata_gst_type_to_name (md, md->priv->type);
-	if (human_name == NULL)
-		human_name = rb_mime_get_friendly_name (md->priv->type);
+	human_name = rb_metadata_gst_type_to_name (md, md->priv->type);
+	if (human_name == NULL) {
+		free_name = rb_mime_get_friendly_name (md->priv->type);
+		human_name = free_name;
+	}
 
 	if (human_name) {
 		return g_strdup_printf (_("The GStreamer plugins to decode \"%s\" files cannot be found"),
@@ -588,6 +655,8 @@ make_undecodable_error (RBMetaData *md)
 		return g_strdup_printf (_("The file contains a stream of type %s, which is not decodable"),
 					md->priv->type);
 	}
+
+	g_free (free_name);
 }
 
 static void
@@ -776,16 +845,6 @@ rb_metadata_gst_unknown_type_cb (GstElement *decodebin, GstPad *pad, GstCaps *ca
 	}
 
 	md->priv->has_non_audio = TRUE;
-#ifndef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
-	/* try to shortcut it a bit */
-	{
-		char *msg;
-
-		msg = make_undecodable_error (md);
-		GST_ELEMENT_ERROR (md->priv->pipeline, STREAM, CODEC_NOT_FOUND, ("%s", msg), (NULL));
-		g_free (msg);
-	}
-#endif
 }
 
 static GstElement *make_pipeline_element (GstElement *pipeline, const char *element, GError **error)
@@ -804,7 +863,6 @@ static GstElement *make_pipeline_element (GstElement *pipeline, const char *elem
 	return elem;
 }
 
-#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
 static void
 rb_metadata_handle_missing_plugin_message (RBMetaData *md, GstMessage *message)
 {
@@ -813,7 +871,6 @@ rb_metadata_handle_missing_plugin_message (RBMetaData *md, GstMessage *message)
 		  gst_missing_plugin_message_get_installer_detail (message));
 	md->priv->missing_plugins = g_slist_prepend (md->priv->missing_plugins, gst_message_ref (message));
 }
-#endif
 
 static gboolean
 rb_metadata_bus_handler (GstBus *bus, GstMessage *message, RBMetaData *md)
@@ -882,11 +939,9 @@ rb_metadata_bus_handler (GstBus *bus, GstMessage *message, RBMetaData *md)
 	}
 	case GST_MESSAGE_ELEMENT:
 	{
-#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
 		if (gst_is_missing_plugin_message (message)) {
 			rb_metadata_handle_missing_plugin_message (md, message);
 		}
-#endif
 		break;
 	}
 	default:
@@ -976,9 +1031,7 @@ rb_metadata_load (RBMetaData *md,
 						    NULL, (GDestroyNotify) free_gvalue);
 
 	/* The main tagfinding pipeline looks like this:
- 	 * gnomevfssrc ! decodebin ! fakesink
-	 * or
-	 * filesrc ! decodebin ! fakesink
+ 	 * <src> ! decodebin ! fakesink
  	 *
  	 * but we can only link the fakesink in when the decodebin
  	 * creates an audio source pad.  we do this in the 'new-decoded-pad'
@@ -1182,11 +1235,7 @@ rb_metadata_load (RBMetaData *md,
 gboolean
 rb_metadata_can_save (RBMetaData *md, const char *mimetype)
 {
-#ifdef ENABLE_TAG_WRITING
 	return rb_metadata_gst_type_to_tag_function (md, mimetype) != NULL;
-#else
-	return FALSE;
-#endif
 }
 
 static void
@@ -1243,13 +1292,13 @@ void
 rb_metadata_save (RBMetaData *md, GError **error)
 {
 	GstElement *pipeline = NULL;
-	GstElement *gnomevfssrc = NULL;
+	GstElement *source = NULL;
         GstElement *retag_end = NULL; /* the last element after retagging subpipeline */
 	const char *plugin_name = NULL;
 	char *tmpname_prefix = NULL;
 	char *tmpname = NULL;
-	GnomeVFSHandle *handle = NULL;
-	GnomeVFSResult result;
+	GOutputStream *stream = NULL;
+	GError *io_error = NULL;
 	RBAddTaggerElem add_tagger_func;
 
 	g_return_if_fail (md->priv->uri != NULL);
@@ -1260,27 +1309,29 @@ rb_metadata_save (RBMetaData *md, GError **error)
 	tmpname_prefix = rb_uri_make_hidden (md->priv->uri);
 	rb_debug ("temporary file name prefix: %s", tmpname_prefix);
 
-	result = rb_uri_mkstemp (tmpname_prefix, &tmpname, &handle);
+	rb_uri_mkstemp (tmpname_prefix, &tmpname, &stream, &io_error);
 	g_free (tmpname_prefix);
-	if (result != GNOME_VFS_OK)
-		goto vfs_error;
+	if (io_error != NULL) {
+		goto gio_error;
+	}
 
 	pipeline = gst_pipeline_new ("pipeline");
 	md->priv->pipeline = pipeline;
 
 	/* Source */
-	plugin_name = "gnomevfssrc";
-	if (!(gnomevfssrc = gst_element_factory_make (plugin_name, plugin_name)))
-		goto missing_plugin;
-	gst_bin_add (GST_BIN (pipeline), gnomevfssrc);
-	g_object_set (G_OBJECT (gnomevfssrc), "location", md->priv->uri, NULL);
+	source = gst_element_make_from_uri (GST_URI_SRC, md->priv->uri, "urisrc");
+	if (source == NULL) {
+		plugin_name = "urisrc";
+		goto missing_plugin;	
+	}
+	gst_bin_add (GST_BIN (pipeline), source);
 
 	/* Sink */
-	plugin_name = "gnomevfssink";
+	plugin_name = "giostreamsink";
 	if (!(md->priv->sink = gst_element_factory_make (plugin_name, plugin_name)))
 		goto missing_plugin;
 
-	g_object_set (G_OBJECT (md->priv->sink), "handle", handle, NULL);
+	g_object_set (G_OBJECT (md->priv->sink), "stream", stream, NULL);
 
 	md->priv->tags = gst_tag_list_new ();
 	g_hash_table_foreach (md->priv->metadata,
@@ -1298,7 +1349,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		goto out_error;
 	}
 
-	retag_end = add_tagger_func (md, gnomevfssrc);
+	retag_end = add_tagger_func (md, source);
 	if (!retag_end) {
 		g_set_error (error,
 			     RB_METADATA_ERROR,
@@ -1327,10 +1378,15 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		g_propagate_error (error, md->priv->error);
 		goto out_error;
 	}
-	if (handle != NULL) {
-		if ((result = gnome_vfs_close (handle)) != GNOME_VFS_OK)
-			goto vfs_error;
-		handle = NULL;
+	if (stream != NULL) {
+		GFile *src;
+		GFile *dest;
+
+		if (g_output_stream_close (stream, NULL, &io_error) == FALSE) {
+			goto gio_error;
+		}
+		g_object_unref (stream);
+		stream = NULL;
 
 		/* check to ensure the file isn't corrupt */
 		if (!rb_metadata_file_valid (md->priv->uri, tmpname)) {
@@ -1341,17 +1397,21 @@ rb_metadata_save (RBMetaData *md, GError **error)
 			goto out_error;
 		}
 
-		if ((result = gnome_vfs_move (tmpname, md->priv->uri, TRUE)) != GNOME_VFS_OK)
-				goto vfs_error;
+		src = g_file_new_for_uri (tmpname);
+		dest = g_file_new_for_uri (md->priv->uri);
+		g_file_move (src, dest, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &io_error);
+		if (io_error != NULL) {
+			goto gio_error;
+		}
 	}
 
 	goto out;
-vfs_error:
+gio_error:
 	g_set_error (error,
 		     RB_METADATA_ERROR,
-		     RB_METADATA_ERROR_GNOMEVFS,
+		     RB_METADATA_ERROR_IO,
 		     "%s",
-		     gnome_vfs_result_to_string (result));
+		     io_error->message);
 	goto out_error;
 missing_plugin:
 	g_set_error (error,
@@ -1360,10 +1420,17 @@ missing_plugin:
 		     _("Failed to create %s element; check your installation"),
 		     plugin_name);
 out_error:
-	if (handle != NULL)
-		gnome_vfs_close (handle);
-	if (tmpname != NULL)
-		gnome_vfs_unlink (tmpname);
+	if (stream != NULL) {
+		g_output_stream_close (stream, NULL, NULL);
+		g_object_unref (stream);
+	}
+
+	if (tmpname != NULL) {
+		GFile *del;
+		del = g_file_new_for_uri (tmpname);
+		g_file_delete (del, NULL, NULL);
+		g_object_unref (del);
+	}
 
 out:
 	if (md->priv->tags)
@@ -1421,11 +1488,7 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 gboolean
 rb_metadata_has_missing_plugins (RBMetaData *md)
 {
-#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
 	return (g_slist_length (md->priv->missing_plugins) > 0);
-#else
-	return FALSE;
-#endif
 }
 
 gboolean
@@ -1433,7 +1496,6 @@ rb_metadata_get_missing_plugins (RBMetaData *md,
 				 char ***missing_plugins,
 				 char ***plugin_descriptions)
 {
-#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
 	char **mp;
 	char **pd;
 	int count;
@@ -1468,8 +1530,5 @@ rb_metadata_get_missing_plugins (RBMetaData *md,
 	*missing_plugins = mp;
 	*plugin_descriptions = pd;
 	return TRUE;
-#else
-	return FALSE;
-#endif
 }
 
