@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -26,8 +34,6 @@
 
 G_BEGIN_DECLS
 
-#define RB_METADATA_NUM_FIELDS 23
-
 typedef enum
 {
 	RB_METADATA_FIELD_TITLE,                   /* string */
@@ -60,14 +66,15 @@ typedef enum
 	RB_METADATA_FIELD_MUSICBRAINZ_ARTISTID,    /* string */
 	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID,     /* string */
 	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID,   /* string */
-	RB_METADATA_FIELD_MUSICBRAINZ_ARTISTSORTNAME,  /* string */
+	RB_METADATA_FIELD_ARTIST_SORTNAME,         /* string */
+	RB_METADATA_FIELD_ALBUM_SORTNAME,         /* string */
 
 	RB_METADATA_FIELD_LAST			   /* nothing */
 } RBMetaDataField;
 
 typedef enum
 {
-	RB_METADATA_ERROR_GNOMEVFS,
+	RB_METADATA_ERROR_IO,
 	RB_METADATA_ERROR_MISSING_PLUGIN,
 	RB_METADATA_ERROR_UNRECOGNIZED,
 	RB_METADATA_ERROR_UNSUPPORTED,
@@ -89,19 +96,22 @@ GQuark rb_metadata_error_quark (void);
 #define RB_IS_METADATA_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_METADATA))
 #define RB_METADATA_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_METADATA, RBMetaDataClass))
 
+typedef struct _RBMetaData RBMetaData;
+typedef struct _RBMetaDataClass RBMetaDataClass;
+
 typedef struct RBMetaDataPrivate RBMetaDataPrivate;
 
-typedef struct
+struct _RBMetaData
 {
 	GObject parent;
 
 	RBMetaDataPrivate *priv;
-} RBMetaData;
+};
 
-typedef struct
+struct _RBMetaDataClass
 {
 	GObjectClass parent_class;
-} RBMetaDataClass;
+};
 
 GType		rb_metadata_get_type	(void);
 
