@@ -11,6 +11,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -33,19 +41,10 @@
 #include <glib/gprintf.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
-#include <nautilus-burn-drive.h>
-#include <nautilus-burn-drive-selection.h>
+#include <gio/gio.h>
 
-#ifndef NAUTILUS_BURN_CHECK_VERSION
-#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE
-#endif
-
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
 #include <nautilus-burn.h>
-#endif
 
 #include "rb-file-helpers.h"
 #include "rb-glade-helpers.h"
@@ -65,18 +64,6 @@
 extern char *mkdtemp (char *template);
 #endif
 
-/* NAUTILUS_BURN_DRIVE_SIZE_TO_TIME was added in 2.12 */
-#ifndef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-#define nautilus_burn_drive_eject _nautilus_burn_drive_eject
-#define nautilus_burn_drive_new_from_path _nautilus_burn_drive_new_from_path
-#define nautilus_burn_drive_media_type_get_string _nautilus_burn_drive_media_type_get_string
-#endif
-/* NAUTILUS_BURN_DRIVE_SIZE_TO_TIME was added in 2.14 */
-#ifndef HAVE_BURN_DRIVE_UNREF
-#define nautilus_burn_drive_unref nautilus_burn_drive_free
-#define nautilus_burn_drive_ref nautilus_burn_drive_copy
-#endif
-
 #include "rb-recorder.h"
 
 #define CONF_STATE_BURN_SPEED CONF_PREFIX "/state/burn_speed"
@@ -129,7 +116,7 @@ struct RBPlaylistSourceRecorderPrivate
         GTimer      *timer;
         guint64      start_pos;
 
-        GtkWidget   *cd_icon;
+        GdkPixbuf   *cd_icon;
         GtkWidget   *vbox;
         GtkWidget   *multiple_copies_checkbutton;
         GtkWidget   *cancel_button;
@@ -292,30 +279,6 @@ get_speed_selection (GtkWidget *combobox)
         return speed;
 }
 
-#ifndef HAVE_BURN_DRIVE_GET_WRITE_SPEEDS
-
-#define nautilus_burn_drive_get_write_speeds get_write_speeds
-
-static const int *
-get_write_speeds (NautilusBurnDrive *drive)
-{
-	int  max_speed;
-	int  i;
-        static int *write_speeds = NULL;
-
-	if (write_speeds == NULL) {
-		max_speed = drive->max_speed_write;
-		write_speeds = g_new0 (int, max_speed + 1);
-
-		for (i = 0; i < max_speed; i++) {
-			write_speeds [i] = max_speed - i;
-		}
-	}
-
-        return (const int*)write_speeds;
-}
-#endif
-
 static void
 update_speed_combobox (RBPlaylistSourceRecorder *source)
 {
@@ -353,11 +316,7 @@ update_speed_combobox (RBPlaylistSourceRecorder *source)
 
                 for (i = 0; write_speeds [i] > 0; i++) {
 
-#ifdef NAUTILUS_BURN_DRIVE_CD_SPEED
                         name = g_strdup_printf ("%d \303\227", (int)NAUTILUS_BURN_DRIVE_CD_SPEED (write_speeds [i]));
-#else
-                        name = g_strdup_printf ("%d \303\227", write_speeds [i]);
-#endif
 
                         if (write_speeds [i] == default_speed) {
                                 default_speed_index = i + 1;
@@ -478,10 +437,10 @@ error_dialog (RBPlaylistSourceRecorder *source,
                                          GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_MESSAGE_ERROR,
                                          GTK_BUTTONS_CLOSE,
-                                         primary);
+                                         "%s", primary);
 
         gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                  text);
+                                                  "%s", text);
 
         gtk_window_set_title (GTK_WINDOW (dialog), "");
 
@@ -565,89 +524,6 @@ progress_set_fraction (GtkWidget *progress,
         gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), fraction);
 }
 
-#ifndef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-/* copied from nautilus-burn-drive 2.12 */
-static gboolean
-_nautilus_burn_drive_eject (NautilusBurnDrive *drive)
-{
-        char    *cmd;
-        gboolean res;
-
-        g_return_val_if_fail (drive != NULL, FALSE);
-
-        if (drive->device == NULL)
-                return FALSE;
-
-        cmd = g_strdup_printf ("eject %s", drive->device);
-        res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
-        g_free (cmd);
-
-        /* delay a bit to make sure eject finishes */
-        sleep (2);
-
-        return res;
-}
-/* copied from nautilus-burn-drive 2.12 */
-static NautilusBurnDrive *
-_nautilus_burn_drive_new_from_path (const char *device)
-{
-        GList             *drives, *l;
-        NautilusBurnDrive *drive;
-
-        drives = nautilus_burn_drive_get_list (FALSE, FALSE);
-
-        drive = NULL;
-
-        for (l = drives; l != NULL; l = l->next) {
-                NautilusBurnDrive *d = l->data;
-                if (g_str_equal (device, d->device)) {
-                        drive = nautilus_burn_drive_ref (d);
-                }
-        }
-
-        g_list_foreach (drives, (GFunc)nautilus_burn_drive_unref, NULL);
-        g_list_free (drives);
-
-        return drive;
-}
-
-/* copied from nautilus-burn-drive 2.12 */
-static const char *
-_nautilus_burn_drive_media_type_get_string (NautilusBurnMediaType type)
-{
-        switch (type) {
-        case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
-                return _("Could not determine media type because CD drive is busy");
-        case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
-                return _("Couldn't open media");
-        case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
-                return _("Unknown Media");
-        case NAUTILUS_BURN_MEDIA_TYPE_CD:
-                return _("Commercial CD or Audio CD");
-        case NAUTILUS_BURN_MEDIA_TYPE_CDR:
-                return _("CD-R");
-        case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-                return _("CD-RW");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD:
-                return _("DVD");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
-                return _("DVD-R, or DVD-RAM");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-                return _("DVD-RW");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
-                return _("DVD-RAM");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
-                return _("DVD+R");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-                return _("DVD+RW");
-        default:
-                break;
-        }
-
-        return _("Broken media type");
-}
-#endif /* NAUTILUS_BURN_DRIVE_SIZE_TO_TIME */
-
 static int
 burn_cd (RBPlaylistSourceRecorder *source,
          GError                  **error)
@@ -677,6 +553,12 @@ burn_cd (RBPlaylistSourceRecorder *source,
 
                 finished_msg = _("Finished creating audio CD.");
 
+		if (source->priv->cd_icon == NULL) {
+			source->priv->cd_icon = gtk_widget_render_icon (GTK_WIDGET (source),
+									GTK_STOCK_CDROM,
+									GTK_ICON_SIZE_BUTTON,
+									NULL);
+		}
                 rb_shell_hidden_notify (source->priv->shell, 0, finished_msg, source->priv->cd_icon, "", FALSE);
 
                 /* save the write speed that was used */
@@ -697,7 +579,7 @@ burn_cd (RBPlaylistSourceRecorder *source,
         } else if (res == RB_RECORDER_RESULT_ERROR) {
                 set_message_text (source, _("Writing failed.  Try again?"));
         } else {
-                set_message_text (source, _("Writing cancelled.  Try again?"));
+                set_message_text (source, _("Writing canceled.  Try again?"));
         }
 
         progress_set_fraction (source->priv->progress, 0);
@@ -1022,7 +904,7 @@ insert_media_request_cb (RBRecorder *recorder,
                                          GTK_BUTTONS_OK_CANCEL,
                                          "%s", title);
 
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), msg);
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", msg);
 
         gtk_window_set_title (GTK_WINDOW (dialog), "");
 
@@ -1097,12 +979,8 @@ ask_rewrite_disc (RBPlaylistSourceRecorder *source,
         char                 *msg;
         NautilusBurnDrive    *drive;
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         drive = nautilus_burn_drive_monitor_get_drive_for_device (nautilus_burn_get_drive_monitor (),
                                                                   device);
-#else
-        drive = nautilus_burn_drive_new_from_path (device);
-#endif
 
         type = nautilus_burn_drive_get_media_type (drive);
 
@@ -1115,7 +993,7 @@ ask_rewrite_disc (RBPlaylistSourceRecorder *source,
                                          GTK_BUTTONS_NONE,
                                          "%s", _("Erase information on this disc?"));
 
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), msg);
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", msg);
         g_free (msg);
 
         gtk_window_set_title (GTK_WINDOW (dialog), "");
@@ -1285,9 +1163,7 @@ rb_playlist_source_recorder_constructor (GType type,
         hbox = gtk_hbox_new (FALSE, 6);
         gtk_container_add (GTK_CONTAINER (widget), hbox);
         gtk_widget_show (hbox);
-        widget = gtk_image_new_from_stock (GTK_STOCK_CDROM, GTK_ICON_SIZE_BUTTON);
-        source->priv->cd_icon = widget;
-        g_object_ref (source->priv->cd_icon);
+
         gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
         gtk_widget_show (widget);
         widget = gtk_label_new_with_mnemonic (_("C_reate"));
@@ -1363,7 +1239,7 @@ rb_playlist_source_recorder_constructor (GType type,
                 dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                                  GTK_MESSAGE_ERROR,
                                                  GTK_BUTTONS_CLOSE,
-                                                 msg);
+                                                 "%s", msg);
                 gtk_dialog_run (GTK_DIALOG (dialog));
                 g_free (msg);
                 return G_OBJECT (source);
@@ -1372,7 +1248,7 @@ rb_playlist_source_recorder_constructor (GType type,
         update_speed_combobox (source);
         g_signal_connect (source->priv->device_menu, "device-changed",
                           G_CALLBACK (rb_playlist_source_recorder_device_changed_cb),
-                          xml);
+                          source);
 
         g_signal_connect_object (G_OBJECT (source->priv->recorder), "eos",
                                  G_CALLBACK (eos_cb), source, 0);
@@ -1646,35 +1522,19 @@ rb_playlist_source_recorder_estimate_total_size (RBPlaylistSourceRecorder *sourc
 }
 
 static gboolean
-check_dir_has_space (const char *path,
-                     guint64     bytes_needed)
+try_tmp_dir (const char *dir, guint64 bytes_needed)
 {
-        GnomeVFSResult   result     = GNOME_VFS_OK;
-        GnomeVFSURI     *dir_uri    = NULL;
-        GnomeVFSFileSize free_bytes = 0;
+	GFile *file;
+	gboolean r;
 
-        if (!g_file_test (path, G_FILE_TEST_IS_DIR))
-                return FALSE;
-
-        dir_uri = gnome_vfs_uri_new (path);
-	if (dir_uri == NULL) {
-		g_warning (_("Cannot get free space at %s"), path);
+	if (dir == NULL) {
 		return FALSE;
 	}
 
-        result = gnome_vfs_get_volume_free_space (dir_uri, &free_bytes);
-
-        gnome_vfs_uri_unref (dir_uri);
-
-        if (result != GNOME_VFS_OK) {
-                g_warning (_("Cannot get free space at %s"), path);
-                return FALSE;
-        }
-
-        if (bytes_needed >= free_bytes)
-                return FALSE;
-
-        return TRUE;
+	file = g_file_new_for_path (dir);
+	r = rb_check_dir_has_space (file, bytes_needed);
+	g_object_unref (file);
+	return r;
 }
 
 static char *
@@ -1682,24 +1542,17 @@ find_tmp_dir (RBPlaylistSourceRecorder *source,
               guint64                   bytes_needed,
               GError                  **error)
 {
-        char *path;
-
         g_return_val_if_fail (source != NULL, NULL);
         g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE_RECORDER (source), NULL);
 
         /* Use a configurable temporary directory? */
-        path = NULL;
-        if (path && strcmp (path, "")
-            && check_dir_has_space (path, bytes_needed))
-                return path;
-        else if (g_get_tmp_dir () &&
-                   check_dir_has_space (g_get_tmp_dir (), bytes_needed))
-                return g_strdup (g_get_tmp_dir ());
-        else if (g_get_home_dir () &&
-                   check_dir_has_space (g_get_home_dir (), bytes_needed))
-                return g_strdup (g_get_home_dir ());
-        else
-                return NULL;
+	if (try_tmp_dir (g_get_tmp_dir (), bytes_needed)) {
+		return g_strdup (g_get_tmp_dir ());
+	} else if (try_tmp_dir (g_get_home_dir (), bytes_needed)) {
+		return g_strdup (g_get_home_dir ());
+	} else {
+		return NULL;
+	}
 }
 
 static gboolean
@@ -1754,7 +1607,7 @@ check_media_length (RBPlaylistSourceRecorder *source,
         if ((media_duration < 0) && (duration > 4440)) {
                 message = g_strdup_printf (_("This playlist is %s minutes long.  "
                                              "This exceeds the length of a standard audio CD.  "
-                                             "If the destination media is larger than a standard audio CD "
+                                             "If the destination medium is larger than a standard audio CD "
                                              "please insert it in the drive and try again."),
                                            duration_string);
         }
@@ -1804,7 +1657,7 @@ rb_playlist_source_recorder_start (RBPlaylistSourceRecorder *source,
 
                         error_dialog (source,
                                       _("Could not find temporary space!"),
-                                      _("Could not find enough temporary space to convert audio tracks.  %s MiB required."),
+                                      _("Could not find enough temporary space to convert audio tracks.  %s MB required."),
                                       mib_needed_string);
                         g_free (mib_needed_string);
 
