@@ -9,6 +9,14 @@
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -33,36 +41,15 @@
 
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <gst/gst.h>
-#ifdef HAVE_GSTREAMER_0_8
-#include <gst/gconf/gconf.h>
-#include <gst/play/play.h>
-#endif
 
-#include <nautilus-burn-drive.h>
-#include <nautilus-burn-recorder.h>
-#ifndef NAUTILUS_BURN_CHECK_VERSION 	 
-#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE 	 
-#endif
-
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
 #include <nautilus-burn.h>
-#endif
 
 #include "rb-recorder.h"
 
 #include "rb-debug.h"
 #include "rb-marshal.h"
 
-#ifdef HAVE_GSTREAMER_0_8
-#define gst_caps_unref gst_caps_free
-#endif
-#ifndef HAVE_BURN_DRIVE_UNREF
-#define nautilus_burn_drive_unref nautilus_burn_drive_free
-#define nautilus_burn_drive_ref nautilus_burn_drive_copy
-#endif
-
 static void rb_recorder_class_init (RBRecorderClass *klass);
 static void rb_recorder_init       (RBRecorder      *recorder);
 static void rb_recorder_finalize   (GObject         *object);
@@ -71,7 +58,7 @@ struct _RBRecorderPrivate {
         char       *src_uri;
         char       *dest_file;
         char       *tmp_dir;
-        
+
         GstElement *pipeline;
         GstElement *decoder;
         GstElement *src;
@@ -81,18 +68,9 @@ struct _RBRecorderPrivate {
         GstElement *encoder;
         GstElement *sink;
 
-#ifdef HAVE_GSTREAMER_0_8
-        GstPad     *src_pad;
-        GError     *error;
-
-        guint       idle_id;
-        guint       error_signal_id;
-        guint       eos_signal_id;
-#elif HAVE_GSTREAMER_0_10
-	GstElement *capsfilter;
-	GstElement *audioconvert2;
-	gboolean    got_audio_pad;
-#endif
+        GstElement *capsfilter;
+        GstElement *audioconvert2;
+        gboolean    got_audio_pad;
 
         guint       tick_timeout_id;
 
@@ -119,14 +97,6 @@ typedef enum {
         LAST_SIGNAL
 } RBRecorderSignalType;
 
-#ifdef HAVE_GSTREAMER_0_8
-typedef struct {
-        RBRecorder *object;
-        GError     *error;
-        GValue     *info;
-} RBRecorderSignal;
-#endif
-
 static guint rb_recorder_signals [LAST_SIGNAL] = { 0 };
 
 #define RB_RECORDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_RECORDER, RBRecorderPrivate))
@@ -206,14 +176,14 @@ rb_recorder_class_init (RBRecorderClass *klass)
                               G_TYPE_BOOLEAN,
                               G_TYPE_BOOLEAN,
                               G_TYPE_BOOLEAN);
-	rb_recorder_signals [WARN_DATA_LOSS] =
-		g_signal_new ("warn-data-loss",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      rb_marshal_INT__VOID,
-			      G_TYPE_INT, 0);
+        rb_recorder_signals [WARN_DATA_LOSS] =
+                g_signal_new ("warn-data-loss",
+                              G_TYPE_FROM_CLASS (klass),
+                              G_SIGNAL_RUN_LAST,
+                              0,
+                              NULL, NULL,
+                              rb_marshal_INT__VOID,
+                              G_TYPE_INT, 0);
         rb_recorder_signals [ERROR] =
                 g_signal_new ("error",
                               G_TYPE_FROM_CLASS (klass),
@@ -234,13 +204,9 @@ rb_recorder_get_default_drive (void)
         NautilusBurnDrive *drive  = NULL;
         GList             *drives = NULL;
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         NautilusBurnDriveMonitor *monitor;
         monitor = nautilus_burn_get_drive_monitor ();
         drives = nautilus_burn_drive_monitor_get_recorder_drives (monitor);
-#else
-        drives = nautilus_burn_drive_get_list (TRUE, FALSE);
-#endif
 
         if (drives) {
                 drive = nautilus_burn_drive_ref ((NautilusBurnDrive*) drives->data);
@@ -296,31 +262,7 @@ rb_recorder_gst_free_pipeline (RBRecorder *recorder)
                         recorder->priv->start_timer = NULL;
                 }
         }
-
-#ifdef HAVE_GSTREAMER_0_8
-        if (recorder->priv->idle_id > 0) {
-                g_source_remove (recorder->priv->idle_id);
-                recorder->priv->idle_id = 0;
-        }
-
-        if (recorder->priv->error_signal_id > 0) {
-                if (g_signal_handler_is_connected (G_OBJECT (recorder->priv->pipeline),
-                                                   recorder->priv->error_signal_id))
-                        g_signal_handler_disconnect (G_OBJECT (recorder->priv->pipeline),
-                                                     recorder->priv->error_signal_id);
-                recorder->priv->error_signal_id = 0;
-        }
-
-        if (recorder->priv->eos_signal_id > 0) {
-                if (g_signal_handler_is_connected (G_OBJECT (recorder->priv->pipeline),
-                                                   recorder->priv->eos_signal_id))
-                        g_signal_handler_disconnect (G_OBJECT (recorder->priv->pipeline),
-                                                     recorder->priv->eos_signal_id);
-                recorder->priv->eos_signal_id = 0;
-        }
-#elif HAVE_GSTREAMER_0_10
-	recorder->priv->got_audio_pad = FALSE;
-#endif
+        recorder->priv->got_audio_pad = FALSE;
 
         gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
 
@@ -333,16 +275,16 @@ add_track (RBRecorder *recorder,
            const char *cdtext)
 {
         NautilusBurnRecorderTrack *track;
-        char                      *filename;
+        char                      *file;
 
         g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
 
-        filename = g_strdup (recorder->priv->dest_file);
+        file = g_strdup (recorder->priv->dest_file);
 
         track = g_new0 (NautilusBurnRecorderTrack, 1);
 
         track->type = NAUTILUS_BURN_RECORDER_TRACK_TYPE_AUDIO;
-        track->contents.audio.filename = filename;
+        track->contents.audio.filename = file;
         if (cdtext) {
                 track->contents.audio.cdtext = g_strdup (cdtext);
         }
@@ -352,172 +294,106 @@ add_track (RBRecorder *recorder,
         return TRUE;
 }
 
-#ifdef HAVE_GSTREAMER_0_8
-static void
-eos_cb (GstElement *element,
-        RBRecorder *recorder)
-{
-        rb_debug ("EOS");
-
-        if (recorder->priv->pipeline) {
-                gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
-        }
-
-        g_signal_emit (G_OBJECT (recorder), rb_recorder_signals [EOS], 0);
-}
-
-static gboolean
-error_signal_idle (RBRecorderSignal *signal)
-{
-        g_signal_emit (G_OBJECT (signal->object),
-                       rb_recorder_signals [ERROR],
-                       0,
-                       signal->error);
-
-        /* close if not already closing */
-        if (signal->object->priv->src_uri != NULL) {
-                rb_recorder_close (signal->object, NULL);
-        }
-
-        g_object_unref (signal->object);
-        g_error_free (signal->error);
-        g_free (signal);
-
-        return FALSE;
-}
-
 static void
 rb_recorder_gst_signal_error (RBRecorder *recorder,
                               const char *msg)
 {
-        RBRecorderSignal *signal;
+        GError *error;
 
-        signal = g_new0 (RBRecorderSignal, 1);
-        signal->object = recorder;
-        signal->error = g_error_new_literal (RB_RECORDER_ERROR,
-                                             RB_RECORDER_ERROR_GENERAL,
-                                             msg);
-	g_object_ref (recorder);
+        g_object_ref (recorder);
+        error = g_error_new_literal (RB_RECORDER_ERROR,
+                                     RB_RECORDER_ERROR_GENERAL,
+                                     msg);
+        g_signal_emit (G_OBJECT (recorder),
+                       rb_recorder_signals [ERROR],
+                       0,
+                       error);
 
-        g_idle_add ((GSourceFunc)error_signal_idle, signal);
-}
-
-static void
-error_cb (GstElement      *element,
-          GstElement      *source,
-          GError          *error,
-          gchar           *debug,
-          RBRecorder      *recorder)
-{
-        rb_debug ("Error");
-
-        rb_recorder_gst_signal_error (recorder, error->message);
-}
-#elif HAVE_GSTREAMER_0_10
-static void
-rb_recorder_gst_signal_error (RBRecorder *recorder,
-			      const char *msg)
-{
-	GError *error;
-
-	g_object_ref (recorder);
-	error = g_error_new_literal (RB_RECORDER_ERROR,
-				     RB_RECORDER_ERROR_GENERAL,
-				     msg);
-	g_signal_emit (G_OBJECT (recorder),
-		       rb_recorder_signals [ERROR],
-		       0,
-		       error);
-
-	/* close if not already closing */
-	if (recorder->priv->src_uri != NULL) {
-		rb_recorder_close (recorder, NULL);
+        /* close if not already closing */
+        if (recorder->priv->src_uri != NULL) {
+                rb_recorder_close (recorder, NULL);
         }
 
-	g_object_unref (recorder);
-	g_error_free (error);
+        g_object_unref (recorder);
+        g_error_free (error);
 }
 
 static gboolean
 pipe_message (GstBus *bus, GstMessage *message, RBRecorder *recorder)
 {
-	switch (message->type) {
-	case GST_MESSAGE_EOS: 
-		rb_debug ("EOS");
+        switch (message->type) {
+        case GST_MESSAGE_EOS:
+                rb_debug ("EOS");
 
-		if (recorder->priv->pipeline)
-			gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
+                if (recorder->priv->pipeline)
+                        gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
 
-		g_signal_emit (G_OBJECT (recorder), 
-			       rb_recorder_signals [EOS], 0);
-		break;
-	case GST_MESSAGE_ERROR:
-	{
-		GError *error;
-		gchar *debug;
+                g_signal_emit (G_OBJECT (recorder),
+                               rb_recorder_signals [EOS], 0);
+                break;
+        case GST_MESSAGE_ERROR:
+        {
+                GError *error;
+                gchar *debug;
 
-		rb_debug ("Error");
-		gst_message_parse_error (message, &error, &debug);
-		if (error) {
-			rb_recorder_gst_signal_error (recorder, error->message);
-			g_error_free (error);
-		} else {
-			rb_recorder_gst_signal_error (recorder, NULL);
-		}
+                rb_debug ("Error");
+                gst_message_parse_error (message, &error, &debug);
+                if (error) {
+                        rb_recorder_gst_signal_error (recorder, error->message);
+                        g_error_free (error);
+                } else {
+                        rb_recorder_gst_signal_error (recorder, NULL);
+                }
 
-		g_free (debug);
-		break;
-	}
-	default:
-		break;
-	}
+                g_free (debug);
+                break;
+        }
+        default:
+                break;
+        }
 
-	return TRUE;
+        return TRUE;
 }
-#endif
- 
+
 static void
 rb_recorder_new_pad_cb (GstElement *decodebin,
-			GstPad     *pad,
-			gboolean    last,
-			RBRecorder *recorder)
+                        GstPad     *pad,
+                        gboolean    last,
+                        RBRecorder *recorder)
 {
-	GstCaps      *caps;
-	GstStructure *str;
-	GstPad       *audio_pad;
+        GstCaps      *caps;
+        GstStructure *str;
+        GstPad       *audio_pad;
 
-	audio_pad = gst_element_get_pad (recorder->priv->audioconvert, "sink");
+        audio_pad = gst_element_get_pad (recorder->priv->audioconvert, "sink");
 
-	/* Only link once. */
-	if (GST_PAD_IS_LINKED (audio_pad))
-		return;
+        /* Only link once. */
+        if (GST_PAD_IS_LINKED (audio_pad))
+                return;
 
-	/* Only link audio. */
-	caps = gst_pad_get_caps (pad);
-	str = gst_caps_get_structure (caps, 0);
-	if (! g_strrstr (gst_structure_get_name (str), "audio")) {
-		gst_caps_unref (caps);
-		return;
-	}
+        /* Only link audio. */
+        caps = gst_pad_get_caps (pad);
+        str = gst_caps_get_structure (caps, 0);
+        if (! g_strrstr (gst_structure_get_name (str), "audio")) {
+                gst_caps_unref (caps);
+                return;
+        }
 
-	gst_caps_unref (caps);
-#ifdef HAVE_GSTREAMER_0_10
-	if (gst_pad_link (pad, audio_pad)) {
-		recorder->priv->got_audio_pad = TRUE;
+        gst_caps_unref (caps);
+        if (gst_pad_link (pad, audio_pad)) {
+                recorder->priv->got_audio_pad = TRUE;
         }
-#endif
 }
 
 static void
 rb_recorder_construct (RBRecorder *recorder,
-                       const char *uri,
+                       const char *src_uri,
+                       const char *dest_file,
                        GError    **error)
 {
         char    *element_name = NULL;
         GstCaps *filtercaps;
-#ifdef HAVE_GSTREAMER_0_10
-	GstBus  *bus;
-#endif
+        GstBus  *bus;
 
 #define MAKE_ELEMENT_OR_LOSE(NAME, NICE) G_STMT_START {                 \
                 element_name = #NAME ;                                  \
@@ -546,36 +422,24 @@ rb_recorder_construct (RBRecorder *recorder,
                 return;
         }
 
-#ifdef HAVE_GSTREAMER_0_8
-        recorder->priv->error_signal_id =
-                g_signal_connect_object (G_OBJECT (recorder->priv->pipeline),
-                                         "error",
-                                         G_CALLBACK (error_cb),
-                                         recorder, 0);
-#elif HAVE_GSTREAMER_0_10
-	bus = gst_element_get_bus (recorder->priv->pipeline);
-	gst_bus_add_watch (bus, (GstBusFunc) pipe_message, recorder);
-	gst_object_unref (bus);
-#endif
+        bus = gst_element_get_bus (recorder->priv->pipeline);
+        gst_bus_add_watch (bus, (GstBusFunc) pipe_message, recorder);
+        gst_object_unref (bus);
 
         /* Construct elements */
 
         /* The source */
+        recorder->priv->src = gst_element_make_from_uri (GST_URI_SRC, src_uri, NULL);
+        if (!recorder->priv->src) {
+                goto missing_element;
+        }
 
-        MAKE_ELEMENT_OR_LOSE(gnomevfssrc, src);
+        if (g_object_class_find_property (G_OBJECT_GET_CLASS (recorder->priv->src),
+                                          "iradio-mode")) {
+                g_object_set (recorder->priv->src, "iradio-mode", FALSE, NULL);
+        }
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->src);
 
-#ifdef HAVE_GSTREAMER_0_8
-        recorder->priv->src_pad = gst_element_get_pad (recorder->priv->src, "src");
-	if (recorder->priv->src_pad == NULL) {
-                g_set_error (error,
-                             RB_RECORDER_ERROR,
-                             RB_RECORDER_ERROR_INTERNAL,
-                             _("Could not access source pad"));
-		return;
-	}
-#endif
-
         /* The queue */
 
         MAKE_ELEMENT_OR_LOSE(typefind, typefind);
@@ -583,35 +447,32 @@ rb_recorder_construct (RBRecorder *recorder,
 
         MAKE_ELEMENT_OR_LOSE(decodebin, decoder);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->decoder);
-	
-	g_signal_connect_object (G_OBJECT (recorder->priv->decoder), 
-				 "new_decoded_pad", G_CALLBACK (rb_recorder_new_pad_cb), 
-				 recorder, 0);
+
+        g_signal_connect_object (G_OBJECT (recorder->priv->decoder),
+                                 "new_decoded_pad", G_CALLBACK (rb_recorder_new_pad_cb),
+                                 recorder, 0);
 
         MAKE_ELEMENT_OR_LOSE(audioconvert, audioconvert);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->audioconvert);
 
-#ifdef HAVE_GSTREAMER_0_8
-        MAKE_ELEMENT_OR_LOSE(audioscale, audioscale);
-#elif HAVE_GSTREAMER_0_10
         MAKE_ELEMENT_OR_LOSE(audioresample, audioscale);
-#endif
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->audioscale);
 
-#ifdef HAVE_GSTREAMER_0_10
         MAKE_ELEMENT_OR_LOSE(audioconvert, audioconvert2);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->audioconvert2);
 
-	MAKE_ELEMENT_OR_LOSE(capsfilter, capsfilter);
-	gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->capsfilter);
-#endif
-	
+        MAKE_ELEMENT_OR_LOSE(capsfilter, capsfilter);
+        gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->capsfilter);
+
         MAKE_ELEMENT_OR_LOSE(wavenc, encoder);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->encoder);
 
         /* Output sink */
-
-        MAKE_ELEMENT_OR_LOSE(gnomevfssink, sink);
+        recorder->priv->sink = gst_element_factory_make ("filesink", "sink");
+        if (recorder->priv->sink == NULL) {
+                goto missing_element;
+        }
+        g_object_set (recorder->priv->sink, "location", dest_file, NULL);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->sink);
 
         filtercaps = gst_caps_new_simple ("audio/x-raw-int",
@@ -619,43 +480,22 @@ rb_recorder_construct (RBRecorder *recorder,
                                           "rate",     G_TYPE_INT, 44100,
                                           "width",    G_TYPE_INT, 16,
                                           "depth",    G_TYPE_INT, 16,
-					  "endianness",	G_TYPE_INT, G_LITTLE_ENDIAN,
+                                          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
                                           NULL);
-#ifdef HAVE_GSTREAMER_0_8
+        g_object_set (recorder->priv->capsfilter, "caps",  filtercaps, NULL);
+
         gst_element_link_many (recorder->priv->src,
                                recorder->priv->typefind,
                                recorder->priv->decoder,
-                               recorder->priv->audioconvert,
-                               recorder->priv->audioscale,
                                NULL);
 
-        gst_element_link_filtered (recorder->priv->audioscale,
-                                   recorder->priv->encoder,
-                                   filtercaps);
-        gst_caps_free (filtercaps);
-
-        gst_element_link (recorder->priv->encoder,
-                          recorder->priv->sink);
-
-        recorder->priv->eos_signal_id =
-                g_signal_connect_object (G_OBJECT (recorder->priv->pipeline), "eos",
-                                         G_CALLBACK (eos_cb), recorder, 0);
-#elif HAVE_GSTREAMER_0_10
-	g_object_set (recorder->priv->capsfilter, "caps",  filtercaps, NULL);
-
-	gst_element_link_many (recorder->priv->src,
-			       recorder->priv->typefind,
-			       recorder->priv->decoder,
-			       NULL);
-
-	gst_element_link_many (recorder->priv->audioconvert,
-			       recorder->priv->audioscale,
-			       recorder->priv->audioconvert2,
-			       recorder->priv->capsfilter,
-			       recorder->priv->encoder,
-			       recorder->priv->sink,
-			       NULL);
-#endif
+        gst_element_link_many (recorder->priv->audioconvert,
+                               recorder->priv->audioscale,
+                               recorder->priv->audioconvert2,
+                               recorder->priv->capsfilter,
+                               recorder->priv->encoder,
+                               recorder->priv->sink,
+                               NULL);
 
         rb_debug ("Pipeline construction complete");
         return;
@@ -722,25 +562,11 @@ rb_recorder_new (GError **error)
 {
         RBRecorder *recorder;
 
-#ifdef HAVE_GSTREAMER_0_8
-        GstElement *dummy;
-
-        dummy = gst_element_factory_make ("fakesink", "fakesink");
-        if (!dummy
-            || !gst_scheduler_factory_make (NULL, GST_ELEMENT (dummy))) {
-                g_set_error (error, RB_RECORDER_ERROR,
-                             RB_RECORDER_ERROR_GENERAL,
-                             _("Couldn't initialize scheduler.  Did you run gst-register?"));
-                return NULL;
-        }
-#endif
-
         recorder = g_object_new (RB_TYPE_RECORDER, NULL, NULL);
 
         return recorder;
 }
 
-#ifdef HAVE_GSTREAMER_0_8
 static gboolean
 tick_timeout_cb (RBRecorder *recorder)
 {
@@ -750,13 +576,19 @@ tick_timeout_cb (RBRecorder *recorder)
         double elapsed;
         double secs;
         GstFormat format = GST_FORMAT_BYTES;
+        GstState  state;
 
         g_return_val_if_fail (recorder != NULL, FALSE);
         g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
         g_return_val_if_fail (recorder->priv != NULL, FALSE);
         g_return_val_if_fail (recorder->priv->pipeline != NULL, FALSE);
 
-        if (gst_element_get_state (recorder->priv->pipeline) != GST_STATE_PLAYING) {
+        if (!gst_element_get_state (recorder->priv->pipeline, &state,  NULL, 3 * GST_SECOND)) {
+                g_warning (_("Could not retrieve state from processing pipeline"));
+                return TRUE;
+        }
+
+        if (state != GST_STATE_PLAYING) {
                 recorder->priv->tick_timeout_id = 0;
                 if (recorder->priv->start_timer) {
                         g_timer_destroy (recorder->priv->start_timer);
@@ -765,78 +597,8 @@ tick_timeout_cb (RBRecorder *recorder)
                 return FALSE;
         }
 
-        if (!gst_pad_query (recorder->priv->src_pad, GST_QUERY_POSITION, &format, &position)) {
-                g_warning (_("Could not get current track position"));
-                return TRUE;
-        }
-
-        if (!gst_pad_query (recorder->priv->src_pad, GST_QUERY_TOTAL, &format, &total)) {
-                g_warning (_("Could not get current track position"));
-                return TRUE;
-        }
-
-        if (! recorder->priv->start_timer) {
-                recorder->priv->start_timer = g_timer_new ();
-                recorder->priv->start_pos = position;
-        }
-
-        fraction = (float)position / (float)total;
-
-        elapsed = g_timer_elapsed (recorder->priv->start_timer, NULL);
-
-        rate = (double)(position - recorder->priv->start_pos) / elapsed;
-
-        if (rate >= 1) {
-                secs = ceil ((total - position) / rate);
-        } else {
-                secs = -1;
-        }
-
-        if (fraction != recorder->priv->progress) {
-                recorder->priv->progress = fraction;
-                g_signal_emit (G_OBJECT (recorder),
-                               rb_recorder_signals [TRACK_PROGRESS_CHANGED],
-                               0,
-                               fraction, (long)secs);
-        }
-
-        /* Extra kick in the pants to keep things moving on a busy system */
-        gst_bin_iterate (GST_BIN (recorder->priv->pipeline));
-
-        return TRUE;
-}
-#elif HAVE_GSTREAMER_0_10
-static gboolean
-tick_timeout_cb (RBRecorder *recorder)
-{
-        gint64 position, total;
-        double fraction;
-        double rate;
-        double elapsed;
-        double secs;
-        GstFormat format = GST_FORMAT_BYTES;
-	GstState  state;
-
-        g_return_val_if_fail (recorder != NULL, FALSE);
-        g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
-        g_return_val_if_fail (recorder->priv != NULL, FALSE);
-        g_return_val_if_fail (recorder->priv->pipeline != NULL, FALSE);
-
-	if (!gst_element_get_state (recorder->priv->pipeline, &state,  NULL, 3 * GST_SECOND)) {
-		g_warning (_("Could not retrieve state from processing pipeline"));
-		return TRUE;
-	}
-
-	if (state != GST_STATE_PLAYING) {		
-                recorder->priv->tick_timeout_id = 0;
-                if (recorder->priv->start_timer) {
-                        g_timer_destroy (recorder->priv->start_timer);
-                        recorder->priv->start_timer = NULL;
-                }
-                return FALSE;
-        }
-
-        if (!gst_element_query_position (recorder->priv->src, &format, &position) || !gst_element_query_duration (recorder->priv->src, &format, &total)) {
+        if (!gst_element_query_position (recorder->priv->src, &format, &position) ||
+            !gst_element_query_duration (recorder->priv->src, &format, &total)) {
                 g_warning (_("Could not get current track position"));
                 return TRUE;
         }
@@ -868,61 +630,6 @@ tick_timeout_cb (RBRecorder *recorder)
 
         return TRUE;
 }
-#endif
-
-
-#ifdef HAVE_GSTREAMER_0_8
-static gboolean
-rb_recorder_sync_pipeline (RBRecorder *recorder,
-                           GError    **error)
-{
-        g_return_val_if_fail (recorder != NULL, FALSE);
-        g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
-        g_return_val_if_fail (recorder->priv != NULL, FALSE);
-        g_return_val_if_fail (recorder->priv->pipeline != NULL, FALSE);
-        
-        rb_debug ("Syncing pipeline");
-        if (recorder->priv->playing) {
-                rb_debug ("Playing pipeline");
-                if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PLAYING) == GST_STATE_FAILURE) {
-                        g_set_error (error,
-                                     RB_RECORDER_ERROR,
-                                     RB_RECORDER_ERROR_GENERAL,
-                                     _("Could not start pipeline playing"));
-                        return FALSE;
-                }
-
-                recorder->priv->idle_id = g_idle_add ((GSourceFunc)gst_bin_iterate,
-                                                      GST_BIN (recorder->priv->pipeline));
-                recorder->priv->tick_timeout_id = g_timeout_add (200,
-                                                                 (GSourceFunc)tick_timeout_cb,
-                                                                 recorder);
-        } else {
-                rb_debug ("Pausing pipeline");
-                if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PAUSED) == GST_STATE_FAILURE) {
-                        g_set_error (error,
-                                     RB_RECORDER_ERROR,
-                                     RB_RECORDER_ERROR_GENERAL,
-                                     _("Could not pause playback"));
-                        return FALSE;
-                }
-                if (recorder->priv->idle_id > 0) {
-                        g_source_remove (recorder->priv->idle_id);
-                        recorder->priv->idle_id = 0;
-                }
-                if (recorder->priv->tick_timeout_id > 0) {
-                        g_source_remove (recorder->priv->tick_timeout_id);
-                        recorder->priv->tick_timeout_id = 0;
-                        if (recorder->priv->start_timer) {
-                                g_timer_destroy (recorder->priv->start_timer);
-                                recorder->priv->start_timer = NULL;
-                        }
-                }
-        }
-        return TRUE;
-}
-
-#elif HAVE_GSTREAMER_0_10
 
 static gboolean
 rb_recorder_sync_pipeline (RBRecorder *recorder,
@@ -932,7 +639,7 @@ rb_recorder_sync_pipeline (RBRecorder *recorder,
         g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
         g_return_val_if_fail (recorder->priv != NULL, FALSE);
         g_return_val_if_fail (recorder->priv->pipeline != NULL, FALSE);
-        
+
         rb_debug ("Syncing pipeline");
         if (recorder->priv->playing) {
                 rb_debug ("Playing pipeline");
@@ -944,7 +651,7 @@ rb_recorder_sync_pipeline (RBRecorder *recorder,
                         return FALSE;
                 }
 
-		recorder->priv->tick_timeout_id = g_timeout_add (200, (GSourceFunc)tick_timeout_cb, recorder);
+                recorder->priv->tick_timeout_id = g_timeout_add (200, (GSourceFunc)tick_timeout_cb, recorder);
         } else {
                 rb_debug ("Pausing pipeline");
                 if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
@@ -965,7 +672,6 @@ rb_recorder_sync_pipeline (RBRecorder *recorder,
         }
         return TRUE;
 }
-#endif
 
 void
 rb_recorder_close (RBRecorder *recorder,
@@ -976,9 +682,7 @@ rb_recorder_close (RBRecorder *recorder,
 
         rb_debug ("Closing rb_recorder");
 
-#ifdef HAVE_GSTREAMER_0_10
-	recorder->priv->got_audio_pad = FALSE;
-#endif
+        recorder->priv->got_audio_pad = FALSE;
         recorder->priv->playing = FALSE;
         g_free (recorder->priv->src_uri);
         recorder->priv->src_uri = NULL;
@@ -1013,7 +717,7 @@ get_dest_from_uri (const char *tmp_dir,
                    const char *src_uri)
 {
         char *lock_filename;
-        char *filename;
+        char *wav_filename;
         int   fd;
 
         lock_filename = g_build_filename (tmp_dir, "rb-burn-tmp.XXXXXX", NULL);
@@ -1023,10 +727,10 @@ get_dest_from_uri (const char *tmp_dir,
         /* keep empty file around until finalize
            it will serve as a lock file to protect our new filename */
 
-        filename = g_strdup_printf ("%s.wav", lock_filename);
+        wav_filename = g_strconcat (lock_filename, ".wav", NULL);
         g_free (lock_filename);
 
-        return filename;
+        return wav_filename;
 }
 
 void
@@ -1037,7 +741,7 @@ rb_recorder_open (RBRecorder   *recorder,
 {
         char    *dest_file;
         gboolean audiocd_mode = src_uri && g_str_has_prefix (src_uri, "audiocd://");
-        
+
         g_return_if_fail (recorder != NULL);
         g_return_if_fail (RB_IS_RECORDER (recorder));
         g_return_if_fail (recorder->priv != NULL);
@@ -1050,34 +754,21 @@ rb_recorder_open (RBRecorder   *recorder,
                 return;
         }
 
-        rb_recorder_construct (recorder, src_uri, error);
+        dest_file = get_dest_from_uri (recorder->priv->tmp_dir, src_uri);
+
+        rb_recorder_construct (recorder, src_uri, dest_file, error);
         if (error && *error) {
+                g_free (dest_file);
                 return;
         }
-                
-#ifdef HAVE_GSTREAMER_0_8
-        if (recorder->priv->idle_id > 0) {
-                g_source_remove (recorder->priv->idle_id);
-                recorder->priv->idle_id = 0;
-        }
-#elif HAVE_GSTREAMER_0_10
-	recorder->priv->got_audio_pad = FALSE;
-#endif
 
-        g_object_set (G_OBJECT (recorder->priv->src), "iradio-mode", FALSE, NULL);
-        gst_element_set_state (recorder->priv->src, GST_STATE_NULL);
-        g_object_set (G_OBJECT (recorder->priv->src), "location", src_uri, NULL);
+        recorder->priv->got_audio_pad = FALSE;
 
         g_free (recorder->priv->src_uri);
         recorder->priv->src_uri = g_strdup (src_uri);
 
-        dest_file = get_dest_from_uri (recorder->priv->tmp_dir, src_uri);
-        gst_element_set_state (recorder->priv->sink, GST_STATE_NULL);
-        g_object_set (G_OBJECT (recorder->priv->sink), "location", dest_file, NULL);
-        
         g_free (recorder->priv->dest_file);
-        recorder->priv->dest_file = g_strdup (dest_file);
-        g_free (dest_file);
+        recorder->priv->dest_file = dest_file;
 
         recorder->priv->playing = FALSE;
 
@@ -1129,16 +820,6 @@ rb_recorder_pause (RBRecorder *recorder,
         rb_recorder_sync_pipeline (recorder, NULL);
 }
 
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-static const char *
-nautilus_burn_drive_get_device (NautilusBurnDrive *drive)
-{
-	g_return_val_if_fail (drive != NULL, NULL);
-
-	return drive->device;
-}
-#endif
-
 char *
 rb_recorder_get_device (RBRecorder  *recorder,
                         GError     **error)
@@ -1170,12 +851,7 @@ rb_recorder_set_device (RBRecorder  *recorder,
                         const char  *device,
                         GError     **error)
 {
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         NautilusBurnDriveMonitor *monitor;
-#else
-        GList *drives;
-        GList *tmp;
-#endif
         int type;
 
         g_return_val_if_fail (recorder != NULL, FALSE);
@@ -1193,29 +869,11 @@ rb_recorder_set_device (RBRecorder  *recorder,
 
         type = 0;
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         monitor = nautilus_burn_get_drive_monitor ();
         recorder->priv->drive = nautilus_burn_drive_monitor_get_drive_for_device (monitor, device);
         if (recorder->priv->drive != NULL) {
                 type = nautilus_burn_drive_get_drive_type (recorder->priv->drive);
         }
-#else
-        drives = nautilus_burn_drive_get_list (TRUE, FALSE);
-
-        for (tmp = drives; tmp != NULL; tmp = tmp->next) {
-                NautilusBurnDrive *drive = (NautilusBurnDrive *) tmp->data;
-
-                if (strcmp (drive->device, device) == 0) {
-                        recorder->priv->drive = drive;
-                        break;
-                }
-
-                nautilus_burn_drive_unref (drive);
-        }
-        g_list_free (drives);
-
-        type = recorder->priv->drive->type;
-#endif
 
         if (! recorder->priv->drive) {
                 g_set_error (error,
@@ -1270,8 +928,6 @@ rb_recorder_action_changed_cb (NautilusBurnRecorder        *cdrecorder,
                        action);
 }
 
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-/* If nautilus-cd-burner >= 2.12 */
 static void
 rb_recorder_burn_progress_cb (NautilusBurnRecorder *cdrecorder,
                               gdouble               fraction,
@@ -1286,23 +942,6 @@ rb_recorder_burn_progress_cb (NautilusBurnRecorder *cdrecorder,
                        fraction,
                        secs);
 }
-#else
-/* If nautilus-cd-burner == 2.10 */
-static void
-rb_recorder_burn_progress_cb (NautilusBurnRecorder *cdrecorder,
-                              gdouble               fraction,
-                              gpointer              data)
-{
-        RBRecorder *recorder = (RBRecorder*) data;
-        long        secs     = -1;
-
-        g_signal_emit (recorder,
-                       rb_recorder_signals [BURN_PROGRESS_CHANGED],
-                       0,
-                       fraction,
-                       secs);
-}
-#endif
 
 static gboolean
 rb_recorder_insert_cd_request_cb (NautilusBurnRecorder *cdrecorder,
@@ -1330,39 +969,20 @@ rb_recorder_warn_data_loss_cb (NautilusBurnRecorder *cdrecorder,
                                RBRecorder           *recorder)
 {
         int res = 0;
-        int ret = 0;
-        int retry_val;
-        int cancel_val;
-        int erase_val;
 
         g_signal_emit (G_OBJECT (recorder),
                        rb_recorder_signals [WARN_DATA_LOSS],
                        0, &res);
 
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-        retry_val  = NAUTILUS_BURN_RECORDER_RESPONSE_RETRY;
-        cancel_val = NAUTILUS_BURN_RECORDER_RESPONSE_CANCEL;
-        erase_val  = NAUTILUS_BURN_RECORDER_RESPONSE_ERASE;
-#else
-        retry_val  = TRUE;
-        cancel_val = TRUE;
-        erase_val  = FALSE;
-#endif
-
         switch (res) {
         case RB_RECORDER_RESPONSE_RETRY:
-                ret = retry_val;
-                break;
+                return NAUTILUS_BURN_RECORDER_RESPONSE_RETRY;
         case RB_RECORDER_RESPONSE_ERASE:
-                ret = erase_val;
-                break;
+                return NAUTILUS_BURN_RECORDER_RESPONSE_ERASE;
         case RB_RECORDER_RESPONSE_CANCEL:
         default:
-                ret = cancel_val;
-                break;
+                return NAUTILUS_BURN_RECORDER_RESPONSE_CANCEL;
         }
-
-        return ret;
 }
 
 char *
@@ -1401,18 +1021,10 @@ rb_recorder_get_media_length (RBRecorder *recorder,
                 return -1;
         }
 
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
         size = nautilus_burn_drive_get_media_capacity (recorder->priv->drive);
-#else
-        size = nautilus_burn_drive_get_media_size (recorder->priv->drive);
-#endif
 
         if (size > 0) {
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
                 secs = NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size);
-#else
-                secs = SIZE_TO_TIME (size);
-#endif
         } else {
                 secs = size;
         }
@@ -1430,23 +1042,23 @@ rb_recorder_get_media_length (RBRecorder *recorder,
 /* Copyright (C) Bastien Nocera */
 /* From xine-lib, whoop */
 typedef struct GNUC_PACKED {
-	gint16   wFormatTag;
-	gint16   nChannels;
-	gint32   nSamplesPerSec;
-	gint32   nAvgBytesPerSec;
-	gint16   nBlockAlign;
-	gint16   wBitsPerSample;
-	gint16   cbSize;
+        gint16   wFormatTag;
+        gint16   nChannels;
+        gint32   nSamplesPerSec;
+        gint32   nAvgBytesPerSec;
+        gint16   nBlockAlign;
+        gint16   wBitsPerSample;
+        gint16   cbSize;
 } waveformat;
 
 #ifndef GNUC_PACKED
 #pragma pack()
 #endif
 
-#define ACB_ERROR_OPEN				-1
-#define ACB_ERROR_NOT_WAVE_TOO_SMALL		-2
-#define ACB_ERROR_NOT_WAVE_FILE			-3
-#define ACB_ERROR_NOT_WAVE_FORMAT		-4
+#define ACB_ERROR_OPEN                          -1
+#define ACB_ERROR_NOT_WAVE_TOO_SMALL            -2
+#define ACB_ERROR_NOT_WAVE_FILE                 -3
+#define ACB_ERROR_NOT_WAVE_FORMAT               -4
 
 /* Copyright (C) Bastien Nocera */
 /* Data from
@@ -1487,7 +1099,7 @@ acb_wave_time (const char *filename)
                 return ACB_ERROR_NOT_WAVE_TOO_SMALL;
         }
 
-	len = GINT_FROM_LE (len);
+        len = GINT_FROM_LE (len);
         if (len != 16) {
                 close (fd);
                 g_print ("file len not defined\n");
@@ -1621,25 +1233,17 @@ rb_recorder_burn (RBRecorder *recorder,
                 flags |= NAUTILUS_BURN_RECORDER_WRITE_DEBUG;
         if (TRUE)
                 flags |= NAUTILUS_BURN_RECORDER_WRITE_DISC_AT_ONCE;
+        if (TRUE)
+                flags |= NAUTILUS_BURN_RECORDER_WRITE_BURNPROOF;
 
-	GDK_THREADS_LEAVE ();
-#ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
-        /* If nautilus-cd-burner >= 2.12 */
+        GDK_THREADS_LEAVE ();
         res = nautilus_burn_recorder_write_tracks (cdrecorder,
                                                    recorder->priv->drive,
                                                    recorder->priv->tracks,
                                                    speed,
                                                    flags,
                                                    &local_error);
-#else
-        /* If nautilus-cd-burner == 2.10 */
-        res = nautilus_burn_recorder_write_tracks (cdrecorder,
-                                                   recorder->priv->drive,
-                                                   recorder->priv->tracks,
-                                                   speed,
-                                                   flags);
-#endif
-	GDK_THREADS_ENTER ();
+        GDK_THREADS_ENTER ();
 
         if (res == NAUTILUS_BURN_RECORDER_RESULT_FINISHED) {
                 result = RB_RECORDER_RESULT_FINISHED;
@@ -1660,14 +1264,14 @@ rb_recorder_burn (RBRecorder *recorder,
                 g_set_error (error,
                              RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_GENERAL,
-                             msg);
+                             "%s", msg);
                 g_free (msg);
                 result = RB_RECORDER_RESULT_ERROR;
         } else {
                 /* cancelled */
                 result = RB_RECORDER_RESULT_CANCEL;
         }
-        
+
         g_object_unref (cdrecorder);
         recorder->priv->recorder = NULL;
 
