@@ -1,13 +1,22 @@
 /*
  * rb-daap-plugin.c
  *
- * Copyright (C) 2006 James Livingston <jrl@ids.org.au>
+ * Copyright (C) 2006 James Livingston <doclivingston@gmail.com>
+ * Copyright (C) 2008 Alban Crequy <alban.crequy@collabora.co.uk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  *
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Rhythmbox. This permission is above and beyond the permissions granted
+ * by the GPL license by which Rhythmbox is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -28,9 +37,9 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 #include <glib-object.h>
+#include <dbus/dbus-glib.h>
 
 #include <libsoup/soup.h>
-#include <libsoup/soup-address.h>
 
 #include "rb-daap-plugin.h"
 #include "rb-debug.h"
@@ -50,6 +59,8 @@
 #define CONF_DAAP_PREFIX  	CONF_PREFIX "/plugins/daap"
 #define CONF_ENABLE_BROWSING 	CONF_DAAP_PREFIX "/enable_browsing"
 
+#define DAAP_DBUS_PATH	"/org/gnome/Rhythmbox/DAAP"
+
 struct RBDaapPluginPrivate
 {
 	RBShell *shell;
@@ -58,6 +69,7 @@ struct RBDaapPluginPrivate
 	GtkWidget *preferences;
 	gboolean sharing;
 	gboolean shutdown;
+	gboolean dbus_intf_added;
 
 	GtkActionGroup *daap_action_group;
 	guint daap_ui_merge_id;
@@ -102,6 +114,10 @@ static void enable_browsing_changed_cb (GConfClient *client,
 					GConfEntry *entry,
 					RBDaapPlugin *plugin);
 
+gboolean rb_daap_add_source (RBDaapPlugin *plugin, gchar *service_name, gchar *host, unsigned int port, GError **error);
+gboolean rb_daap_remove_source (RBDaapPlugin *plugin, gchar *service_name, GError **error);
+#include "rb-daap-glue.h"
+
 RB_PLUGIN_REGISTER(RBDaapPlugin, rb_daap_plugin)
 
 static GtkActionEntry rb_daap_source_actions [] =
@@ -109,7 +125,7 @@ static GtkActionEntry rb_daap_source_actions [] =
 	{ "DaapSourceDisconnect", GTK_STOCK_DISCONNECT, N_("_Disconnect"), NULL,
 	  N_("Disconnect from DAAP share"),
 	  G_CALLBACK (rb_daap_plugin_cmd_disconnect) },
-	{ "MusicNewDAAPShare", GTK_STOCK_CONNECT, N_("Connect to DAAP share"), NULL,
+	{ "MusicNewDAAPShare", GTK_STOCK_CONNECT, N_("Connect to _DAAP share..."), NULL,
 	  N_("Connect to a new DAAP share"),
 	  G_CALLBACK (rb_daap_plugin_cmd_connect) },
 };
@@ -190,7 +206,10 @@ impl_activate (RBPlugin *bplugin,
 	GConfClient *client = eel_gconf_client_get_global ();
 	GtkUIManager *uimanager = NULL;
 	char *uifile;
+	DBusGConnection *conn;
+	GError *error = NULL;
 
+	plugin->priv->shutdown = FALSE;
 	plugin->priv->shell = g_object_ref (shell);
 
 	value = gconf_client_get_without_default (client,
@@ -248,6 +267,22 @@ impl_activate (RBPlugin *bplugin,
 	plugin->priv->sharing = !no_registration;
 	if (plugin->priv->sharing)
 		rb_daap_sharing_init (shell);
+
+	/*
+	 * Add dbus interface
+	 */
+	if (plugin->priv->dbus_intf_added == FALSE) {
+		conn = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+		if (conn != NULL) {
+			dbus_g_object_type_install_info (RB_TYPE_DAAP_PLUGIN,
+							 &dbus_glib_rb_daap_object_info);
+			dbus_g_connection_register_g_object (conn, DAAP_DBUS_PATH,
+							     G_OBJECT (bplugin));
+			plugin->priv->dbus_intf_added = TRUE;
+		}
+		else
+			rb_debug ("No session D-Bus. DAAP interface on D-Bus disabled.");
+	}
 }
 
 static void
@@ -586,6 +621,7 @@ rb_daap_plugin_cmd_disconnect (GtkAction *action,
 
 typedef struct {
 	RBDaapPlugin *plugin;
+	char *service_name;
 	char *location;
 } RBDaapShareResolveData;
 
@@ -599,7 +635,7 @@ new_daap_share_resolve_cb (SoupAddress *addr,
 	if (status == SOUP_STATUS_OK) {
 		rb_debug ("adding manually specified DAAP share at %s", data->location);
 		mdns_service_added (NULL,
-				    data->location,
+				    data->service_name,
 				    data->location,
 				    soup_address_get_physical (addr),
 				    soup_address_get_port (addr),
@@ -614,6 +650,7 @@ new_daap_share_resolve_cb (SoupAddress *addr,
 	}
 
 	g_object_unref (data->plugin);
+	g_free (data->service_name);
 	g_free (data->location);
 	g_free (data);
 	g_object_unref (addr);
@@ -634,6 +671,7 @@ new_daap_share_location_added_cb (RBURIDialog *dialog,
 
 	data = g_new0 (RBDaapShareResolveData, 1);
 	data->plugin = g_object_ref (plugin);
+	data->service_name = g_strdup (location);
 	data->location = g_strdup (location);
 
 	host = g_strdup (location);
@@ -647,6 +685,7 @@ new_daap_share_location_added_cb (RBURIDialog *dialog,
 	g_free (host);
 
 	soup_address_resolve_async (addr,
+				    NULL, NULL,
 				    (SoupAddressCallback) new_daap_share_resolve_cb,
 				    data);
 }
@@ -902,3 +941,39 @@ impl_create_configure_dialog (RBPlugin *bplugin)
 	return plugin->priv->preferences;
 }
 
+gboolean
+rb_daap_add_source (RBDaapPlugin *plugin, gchar *service_name, gchar *host, unsigned int port, GError **error)
+{
+	SoupAddress *addr;
+	RBDaapShareResolveData *data;
+
+	if (plugin->priv->shutdown)
+		return FALSE;
+
+	rb_debug ("Add DAAP source %s (%s:%d)", service_name, host, port);
+
+	data = g_new0 (RBDaapShareResolveData, 1);
+	data->plugin = g_object_ref (plugin);
+	data->service_name = g_strdup (service_name);
+	data->location = g_strdup (service_name);
+
+	addr = soup_address_new (host, port);
+
+	soup_address_resolve_async (addr,
+				    NULL, NULL,
+				    (SoupAddressCallback) new_daap_share_resolve_cb,
+				    data);
+
+	return TRUE;
+}
+
+gboolean
+rb_daap_remove_source (RBDaapPlugin *plugin, gchar *service_name, GError **error)
+{
+	if (plugin->priv->shutdown)
+		return FALSE;
+	rb_debug ("Remove DAAP source %s", service_name);
+	mdns_service_removed (plugin->priv->mdns_browser, service_name, plugin);
+	return TRUE;
+}
+
