@@ -9,6 +9,14 @@
 # the Free Software Foundation; either version 2, or (at your option)
 # any later version.
 #
+# The Rhythmbox authors hereby grant permission for non-GPL compatible
+# GStreamer plugins to be used and distributed together with GStreamer
+# and Rhythmbox. This permission is above and beyond the permissions granted
+# by the GPL license by which Rhythmbox is covered. If you modify this code
+# you may extend this exception to your version of the code, but you are not
+# obligated to do so. If you do not wish to do so, delete this exception
+# statement from your version.
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -39,7 +47,7 @@ ui_str = """
 </ui>
 """
 
-LYRIC_TITLE_STRIP=["\(live[^\)]*\)", "\(acoustic[^\)]*\)", "\([^\)]*mix\)", "\([^\)]*version\)", "\([^\)]*edit\)", "\(feat[^\)]*\)", "\([^\)]*\)"]
+LYRIC_TITLE_STRIP=["\(live[^\)]*\)", "\(acoustic[^\)]*\)", "\([^\)]*mix\)", "\([^\)]*version\)", "\([^\)]*edit\)", "\(feat[^\)]*\)"]
 LYRIC_TITLE_REPLACE=[("/", "-"), (" & ", " and ")]
 LYRIC_ARTIST_REPLACE=[("/", "-"), (" & ", " and ")]
 
@@ -52,7 +60,9 @@ def create_lyrics_view():
 	tview = gtk.TextView()
 	tview.set_wrap_mode(gtk.WRAP_WORD)
 	tview.set_editable(False)
+	tview.set_left_margin(6)
 
+	tview.set_size_request (0, 0)
 	sw = gtk.ScrolledWindow()
 	sw.add(tview)
 	sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
@@ -84,21 +94,20 @@ def parse_song_data(artist, title):
 def build_cache_path(artist, title):
 	folder = gconf.client_get_default().get_string(gconf_keys['folder'])
 	if folder is None:
-		folder = "~/.lyrics"
+		folder = os.path.join(rb.user_cache_dir(), "lyrics")
 
 	lyrics_folder = os.path.expanduser (folder)
 	if not os.path.exists (lyrics_folder):
 		os.mkdir (lyrics_folder)
 
-	artist_folder = lyrics_folder + '/' + artist[:128]
+	artist_folder = os.path.join(lyrics_folder, artist[:128])
 	if not os.path.exists (artist_folder):
 		os.mkdir (artist_folder)
 
-	return artist_folder + '/' + title[:128] + '.lyric'
+	return os.path.join(artist_folder, title[:128] + '.lyric')
 
 class LyricGrabber(object):
 	def __init__(self, db, entry):
-		self.loader = rb.Loader ()
 		self.db = db
 		self.entry = entry
 		
@@ -118,7 +127,8 @@ class LyricGrabber(object):
 		status = self.verify_lyric()
 		
 		if status:
-			self.loader.get_url(self.cache_path, callback)
+			l = rb.Loader()
+			l.get_url(self.cache_path, callback)
 		else:
 			if cache_only:
 				self.callback(_("No lyrics found"))
@@ -164,12 +174,12 @@ class LyricPane(object):
 		def edit_callback(widget):
 			if self.edit.get_active() == 1:
 				self.tview.set_editable(True)
-				self.edit.set_label("_Save")
+				self.edit.set_label(_("_Save"))
 			else:
 				if self.cache_path is not None:
 					save_callback()
 				self.tview.set_editable(False)
-				self.edit.set_label("_Edit")
+				self.edit.set_label(_("_Edit"))
 
 		def discard_callback(widget):
 			if self.cache_path is not None and os.path.exists(self.cache_path):
@@ -182,20 +192,23 @@ class LyricPane(object):
 			self.get_lyrics()
 	   
 
-		self.edit = gtk.ToggleButton("_Edit")
+		self.edit = gtk.ToggleButton(_("_Edit"))
 		self.edit.connect('toggled', edit_callback)
-		self.discard = gtk.Button("_Search again")
+		self.discard = gtk.Button(_("_Search again"))
 		self.discard.connect('clicked', discard_callback)
 		self.clear = gtk.Button(stock=gtk.STOCK_CLEAR)
 		self.clear.connect('clicked', clear_callback)
-		self.hbox = gtk.HBox()
-		self.hbox.pack_end(self.edit, expand=False)
-		self.hbox.pack_end(self.clear, expand=False)
-		self.hbox.pack_start(self.discard, expand=False)
-		
+		self.hbox = gtk.HButtonBox()
+		self.hbox.set_spacing (6)
+		self.hbox.set_layout(gtk.BUTTONBOX_END)
+		self.hbox.add(self.edit)
+		self.hbox.add(self.clear)
+		self.hbox.add(self.discard)
+		self.hbox.set_child_secondary (self.clear, is_secondary=True)
+
 		(self.view, self.buffer, self.tview) = create_lyrics_view()
 
-		self.view.pack_start(self.hbox, expand=False, fill=False)
+		self.view.pack_start(self.hbox, expand=False, fill=False, padding=6)
 		self.view.set_spacing(2)
 	
 		self.view.show_all()
@@ -235,13 +248,16 @@ class LyricPane(object):
 		self.visible = 1
 		self.get_lyrics()
 
+	def __got_lyrics(self, text):
+		self.buffer.set_text(text)
+
 	def get_lyrics(self):
 		if self.entry is None:
 			return
 
 		self.buffer.set_text(_("Searching for lyrics..."));
 		lyrics_grabber = LyricGrabber(self.db, self.entry)
-		lyrics_grabber.search_lyrics(self.buffer.set_text)
+		lyrics_grabber.search_lyrics(self.__got_lyrics)
 			
 
 class LyricWindow (gtk.Window):
@@ -265,7 +281,7 @@ class LyricWindow (gtk.Window):
 		self.show_all()
 
 	def s_title(self, title, artist):
-		self.set_title(title + " - " + artist + " - Lyrics")
+		self.set_title(title + " - " + artist + " - " + _("Lyrics"))
 
 class LyricsDisplayPlugin(rb.Plugin):
 
@@ -326,6 +342,11 @@ class LyricsDisplayPlugin(rb.Plugin):
 		else:
 			self.action.set_sensitive (False)
 
+	def __got_lyrics(self, text):
+		# don't do anything if the window has already been destroyed
+		if self.window is not None:
+			self.window.buffer.set_text (text)
+
 	def update_song_lyrics(self, entry):
 		if entry == self.current_entry:
 			return
@@ -340,7 +361,7 @@ class LyricsDisplayPlugin(rb.Plugin):
 
 		self.window.s_title(title, artist)
 		lyrics_grabber = LyricGrabber(db, entry)
-		lyrics_grabber.search_lyrics(self.window.buffer.set_text)
+		lyrics_grabber.search_lyrics(self.__got_lyrics)
 
 	def show_song_lyrics (self, action, shell):
 
@@ -362,7 +383,7 @@ class LyricsDisplayPlugin(rb.Plugin):
 		self.window.s_title(title, artist)
 		self.window.connect("destroy", self.window_deleted)
 		lyrics_grabber = LyricGrabber(db, entry)
-		lyrics_grabber.search_lyrics(self.window.buffer.set_text)
+		lyrics_grabber.search_lyrics(self.__got_lyrics)
 
 	def window_deleted (self, window):
 		print "lyrics window destroyed"
