@@ -8,7 +8,7 @@
 # the Free Software Foundation; either version 2, or (at your option)
 # any later version.
 #
-# The Rhythmbox authors hereby grants permission for non-GPL compatible
+# The Rhythmbox authors hereby grant permission for non-GPL compatible
 # GStreamer plugins to be used and distributed together with GStreamer
 # and Rhythmbox. This permission is above and beyond the permissions granted
 # by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -29,11 +29,14 @@ import rb, rhythmdb
 from TrackListHandler import TrackListHandler
 from BuyAlbumHandler import BuyAlbumHandler, MagnatunePurchaseError
 
+import os
 import gobject
 import gtk.glade
-import gnomevfs, gnome, gconf
+import gnome, gconf
 import xml
-import urllib, zipfile
+import urllib
+import urlparse
+import zipfile
 
 has_gnome_keyring = False
 
@@ -47,10 +50,15 @@ has_gnome_keyring = False
 magnatune_partner_id = "rhythmbox"
 
 # URIs
-magnatune_dir = gnome.user_dir_get() + "rhythmbox/magnatune/"
-magnatune_song_info_uri = gnomevfs.URI("http://magnatune.com/info/song_info_xml.zip")
-local_song_info_uri = gnomevfs.URI(magnatune_dir + "song_info.xml")
-local_song_info_temp_uri = gnomevfs.URI(magnatune_dir + "song_info.xml.zip.tmp")
+magnatune_song_info_uri = "http://magnatune.com/info/song_info_xml.zip"
+
+magnatune_in_progress_dir = os.path.join(rb.user_data_dir(), 'magnatune')
+magnatune_cache_dir = os.path.join(rb.user_cache_dir(), 'magnatune')
+
+magnatune_song_info = os.path.join(magnatune_cache_dir, 'song_info.xml')
+magnatune_song_info_temp = os.path.join(magnatune_cache_dir, 'song_info.zip.tmp')
+
+
 ALBUM_ART_URL = 'http://www.magnatune.com/music/%s/%s/cover.jpg'
 
 class MagnatuneSource(rb.BrowserSource):
@@ -76,11 +84,11 @@ class MagnatuneSource(rb.BrowserSource):
 		self.__activated = False
 		self.__notify_id = 0
 		self.__update_id = 0
-		self.__xfer_handle = None
+		self.__catalogue_loader = None
+		self.__catalogue_check = None
 		self.__info_screen = None
 		self.__updating = True
 		self.__has_loaded = False
-		self.__load_handle = None
 		self.__load_current_size = 0
 		self.__load_total_size = 0
 
@@ -90,6 +98,7 @@ class MagnatuneSource(rb.BrowserSource):
 		self.purchase_filesize = 0 # total amount of bytes to download
 
 
+
 	def do_set_property(self, property, value):
 		if property.name == 'plugin':
 			self.__plugin = value
@@ -109,7 +118,7 @@ class MagnatuneSource(rb.BrowserSource):
 				progress = min (float(self.__load_current_size) / self.__load_total_size, 1.0)
 			else:
 				progress = -1.0
-			return (_("Loading Magnatune catalogue"), None, progress)
+			return (_("Loading Magnatune catalog"), None, progress)
 		elif self.__downloading:
 			progress = min (self.__download_progress, 1.0)
 			return (_("Downloading Magnatune Album(s)"), None, progress)
@@ -129,9 +138,12 @@ class MagnatuneSource(rb.BrowserSource):
 			self.__db = shell.get_property('db')
 			self.__entry_type = self.get_property('entry-type')
 
+			# move files from old ~/.gnome2 paths
+			if os.path.exists(magnatune_in_progress_dir) is False:
+				self.__move_data_files()
+
 			self.__activated = True
 			self.__show_loading_screen (True)
-			self.__load_catalogue()
 
 			# start our catalogue updates
 			self.__update_id = gobject.timeout_add(6 * 60 * 60 * 1000, self.__update_catalogue)
@@ -162,9 +174,13 @@ class MagnatuneSource(rb.BrowserSource):
 			gobject.source_remove (self.__notify_id)
 			self.__notify_id = 0
 
-		if self.__xfer_handle is not None:
-			self.__xfer_handle.cancel()
-			self.__xfer_handle = None
+		if self.__catalogue_loader is not None:
+			self.__catalogue_loader.cancel()
+			self.__catalogue_loader = None
+
+		if self.__catalogue_check is not None:
+			self.__catalogue_check.cancel()
+			self.__catalogue_check = None
 
 		self.__client.set_string("/apps/rhythmbox/plugins/magnatune/sorting", self.get_entry_view().get_sorting_type())
 
@@ -182,7 +198,7 @@ class MagnatuneSource(rb.BrowserSource):
 			sku = self.__sku_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
 			url = self.__home_dict[sku]
 			if url not in urls:
-				gnomevfs.url_show(url)
+				rb.show_uri(url)
 				urls.add(url)
 
 	def buy_cd(self):
@@ -193,7 +209,7 @@ class MagnatuneSource(rb.BrowserSource):
 			sku = self.__sku_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
 			url = self.__buy_dict[sku]
 			if url not in urls:
-				gnomevfs.url_show(url)
+				rb.show_uri(url)
 				urls.add(url)
 
 	def radio_toggled(self, gladexml):
@@ -281,62 +297,59 @@ class MagnatuneSource(rb.BrowserSource):
 	#
 	# internal catalogue downloading and loading
 	#
-	def __load_catalogue_read_cb (self, handle, data, exc_type, bytes_requested, parser):
-		if exc_type:
-			if issubclass (exc_type, gnomevfs.EOFError):
-				def finish_loadscreen():
-					# successfully loaded
-					gtk.gdk.threads_enter()
-					self.__show_loading_screen (False)
 
-					in_progress_dir = gnomevfs.DirectoryHandle(gnomevfs.URI(magnatune_dir))
-					in_progress = in_progress_dir.next()
-					while True:
-						if in_progress.name[0:12] == "in_progress_":
-							in_progress = gnomevfs.read_entire_file(magnatune_dir + in_progress.name)
-							for uri in in_progress.split("\n"):
-								if uri == '':
-									continue
-								self.__download_album(gnomevfs.URI(uri))
-						try:
-							in_progress = in_progress_dir.next()
-						except:
-							break
-					gtk.gdk.threads_leave()
-				gobject.idle_add (finish_loadscreen)
-			else:
-				# error reading file
-				raise exc_type
+	def __catalogue_chunk_cb(self, result, total, parser):
+		if not result or isinstance (result, Exception):
+			if result:
+				# report error somehow?
+				print "error loading catalogue: %s" % result
 
-			parser.close()
-			handle.close(lambda handle, exc: None) # FIXME: report it?
-			self.__load_handle = None
+			try:
+				parser.close ()
+			except xml.sax.SAXParseException, e:
+				# there isn't much we can do here
+				print "error parsing catalogue: %s" % e
+
+			self.__show_loading_screen (False)
 			self.__updating = False
-			self.__notify_status_changed()
- 		else:
+			self.__catalogue_loader = None
 
-			parser.feed(data)
-			handle.read(64 * 1024, self.__load_catalogue_read_cb, parser)
+			# restart in-progress downloads
+			# (doesn't really belong here)
+			inprogress = os.listdir(magnatune_in_progress_dir)
+			inprogress = filter(lambda x: x.startswith("in_progress_"), inprogress)
+			for ip in inprogress:
+				for uri in open(ip).readlines():
+					print "restarting download from %s" % uri
+					self.__download_album(uri)
+
+		else:
+			# hack around some weird chars that show up in the catalogue for some reason
+			result = result.replace("\x19", "'")
+			result = result.replace("\x13", "-")
+
+			try:
+				parser.feed(result)
+			except xml.sax.SAXParseException, e:
+				print "error parsing catalogue: %s" % e
+
+			self.__load_current_size += len(result)
+			self.__load_total_size = total
 
 		self.__notify_status_changed()
 
-	def __load_catalogue_open_cb (self, handle, exc_type):
-		if exc_type:
-			self.__load_handle = None
-			self.__notify_status_changed()
-
-			if gnomevfs.exists(local_song_info_uri):
-				raise exc_type
-			else:
-				return
-
-		parser = xml.sax.make_parser()
-		parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__buy_dict, self.__art_dict))
-		handle.read (64 * 1024, self.__load_catalogue_read_cb, parser)
 
 	def __load_catalogue(self):
 		self.__notify_status_changed()
-		self.__load_handle = gnomevfs.async.open (local_song_info_uri, self.__load_catalogue_open_cb)
+		self.__has_loaded = True
+
+		parser = xml.sax.make_parser()
+		parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__buy_dict, self.__art_dict))
+		
+		self.__catalogue_loader = rb.ChunkLoader()
+		self.__catalogue_loader.get_url_chunks(magnatune_song_info, 64*1024, True, self.__catalogue_chunk_cb, parser)
+
+
 
 	def __find_song_info(self, catalogue):
 		for info in catalogue.infolist():
@@ -344,76 +357,59 @@ class MagnatuneSource(rb.BrowserSource):
 				return info.filename;
 		return None
 
-	def __download_update_cb (self, _reserved, info, moving):
-		self.__load_current_size = info.bytes_copied
-		self.__load_total_size = info.bytes_total
-		self.__notify_status_changed()
 
-		if info.phase == gnomevfs.XFER_PHASE_COMPLETED:
+	def __download_catalogue_chunk_cb (self, result, total, out):
+		if not result:
 			# done downloading, unzip to real location
-			catalog = zipfile.ZipFile(local_song_info_temp_uri.path)
-			out = create_if_needed(local_song_info_uri, gnomevfs.OPEN_WRITE)
+			out.close()
+
+			catalog = zipfile.ZipFile(magnatune_song_info_temp)
+			out = open(magnatune_song_info, 'w')
 			filename = self.__find_song_info(catalog)
 			if filename is None:
-				rb.error_dialog(title="Unable to load catalogue", message=_("Rhythmbox could not understand the Magnatune catalogue, please file a bug."))
+				rb.error_dialog(title=_("Unable to load catalog"),
+						message=_("Rhythmbox could not understand the Magnatune catalog, please file a bug."))
 				return
 			out.write(catalog.read(filename))
 			out.close()
 			catalog.close()
-			gnomevfs.unlink(local_song_info_temp_uri)
-			self.__updating = False
-			self.__load_catalogue()
-			self.__xfer_handle = None
-		else:
-			#print info
-			pass
 
-		return 1
+			os.unlink(magnatune_song_info_temp)
+			self.__updating = False
+			self.__catalogue_loader = None
+			self.__load_catalogue()
+
+		elif isinstance(result, Exception):
+			# complain
+			pass
+		else:
+			out.write(result)
+			self.__load_current_size += len(result)
+			self.__load_total_size = total
+
+		self.__notify_status_changed()
+
 
 	def __download_catalogue(self):
 		self.__updating = True
-		create_if_needed(local_song_info_temp_uri, gnomevfs.OPEN_WRITE).close()
-		self.__xfer_handle = gnomevfs.async.xfer (source_uri_list = [magnatune_song_info_uri],
-							  target_uri_list = [local_song_info_temp_uri],
-							  xfer_options = gnomevfs.XFER_FOLLOW_LINKS_RECURSIVE,
-							  error_mode = gnomevfs.XFER_ERROR_MODE_ABORT,
-							  overwrite_mode = gnomevfs.XFER_OVERWRITE_MODE_REPLACE,
-							  progress_update_callback = self.__download_update_cb,
-							  update_callback_data = False)
-							  
+
+		out = open(magnatune_song_info_temp, 'w')
+
+		self.__catalogue_loader = rb.ChunkLoader()
+		self.__catalogue_loader.get_url_chunks(magnatune_song_info_uri, 4*1024, True, self.__download_catalogue_chunk_cb, out)
+
 
 	def __update_catalogue(self):
-		def info_cb (handle, results):
-			(remote_uri, remote_exc, remote_info) = results[0]
-			(local_uri, local_exc, local_info) = results[1]
+		def update_cb (result):
+			self.__catalogue_check = None
+			if result is True:
+				self.__download_catalogue()
+			elif self.__has_loaded is False:
+				self.__load_catalogue()
 
-			if remote_exc:
-				# error locating remote file
-				print "error locating remote catalogue", remote_exc
-			elif local_exc:
-				if issubclass (local_exc, gnomevfs.NotFoundError):
-					# we haven't got it yet
-					print "no local copy of catalogue"
-					self.__download_catalogue()
-				else:
-					# error locating local file
-					print "error locating local catalogue", local_exc
-					self.__download_catalogue()
-			else:
-				try:
-					if remote_info.mtime > local_info.mtime:
-						# newer version available
-						self.__download_catalogue()
-					else:
-						# up to date
-						pass
-				except ValueError, e:
-					# couldn't get the mtimes. download?
-					print "error checking times", e
-					self.__download_catalogue()
-			return
+		self.__catalogue_check = rb.UpdateCheck()
+		self.__catalogue_check.check_for_update(magnatune_song_info, magnatune_song_info_uri, update_cb)
 
-		gnomevfs.async.get_file_info ((magnatune_song_info_uri, local_song_info_uri), info_cb)
 
 	def __show_loading_screen(self, show):
 		if self.__info_screen is None:
@@ -457,137 +453,164 @@ class MagnatuneSource(rb.BrowserSource):
 		url = "https://magnatune.com/buy/buy_dl_cc_xml?"
 		url = url + urllib.urlencode(url_dict)
 
-		buy_album_handler = BuyAlbumHandler(format) # so we can get the url and auth info
-		auth_parser = xml.sax.make_parser()
-		auth_parser.setContentHandler(buy_album_handler)
-
-		self.__wait_dlg = gtk.Dialog(title="Authorizing Purchase", flags=gtk.DIALOG_NO_SEPARATOR|gtk.DIALOG_DESTROY_WITH_PARENT)
-		lbl = gtk.Label("Authorizing purchase with the Magnatune server. Please wait...")
+		self.__wait_dlg = gtk.Dialog(title=_("Authorizing Purchase"), flags=gtk.DIALOG_NO_SEPARATOR|gtk.DIALOG_DESTROY_WITH_PARENT)
+		lbl = gtk.Label(_("Authorizing purchase with the Magnatune server. Please wait..."))
 		self.__wait_dlg.vbox.pack_start(lbl)
 		lbl.show()
 		self.__wait_dlg.show()
-		gnomevfs.async.open(gnomevfs.URI(url), self.__auth_open_cb, data=(buy_album_handler, auth_parser))
 
-	def __auth_open_cb(self, handle, exc_type, data):
-		if exc_type:
-			raise exc_type
-
-		handle.read(64 * 1024, self.__auth_read_cb, data)
+		l = rb.Loader()
+		l.get_url (url, self.__auth_data_cb, format)
 
 
-	def __auth_read_cb (self, handle, data, exc_type, bytes_requested, parser):
-		buy_album_handler = parser[0]
-		auth_parser = parser[1]
-		data = data.replace("<br>", "") # get rid of any stray <br> tags that will mess up the parser
-		if exc_type:
-			if issubclass (exc_type, gnomevfs.EOFError):
-				def start_download ():
-					# successfully loaded
-					gtk.gdk.threads_enter()
-					audio_dl_uri = gnomevfs.URI(buy_album_handler.url)
-					audio_dl_uri = gnomevfs.URI(buy_album_handler.url[0:buy_album_handler.url.rfind("/") + 1] + urllib.quote(audio_dl_uri.short_name))
-					audio_dl_uri.user_name = str(buy_album_handler.username) # URI objects don't like unicode strings
-					audio_dl_uri.password = str(buy_album_handler.password)
 
-					in_progress = create_if_needed(gnomevfs.URI(magnatune_dir + "in_progress_" + audio_dl_uri.short_name), gnomevfs.OPEN_WRITE)
-					in_progress.write(str(audio_dl_uri))
-					in_progress.close()
-					self.__download_album(audio_dl_uri)
-					self.__wait_dlg.destroy()
-					gtk.gdk.threads_leave()
-				gobject.idle_add (start_download)
-			else:
-				# error reading file
-				raise exc_type
+	def __auth_data_cb (self, data, format):
 
+		buy_album_handler = BuyAlbumHandler(format)
+		auth_parser = xml.sax.make_parser()
+		auth_parser.setContentHandler(buy_album_handler)
+
+		if data is None:
+			# hmm.
+			return
+
+		self.__wait_dlg.destroy()
+		try:
+			data = data.replace("<br>", "") # get rid of any stray <br> tags that will mess up the parser
+			# print data
+			auth_parser.feed(data)
 			auth_parser.close()
-			handle.close(lambda handle, exc: None) # FIXME: report it?
 
- 		else:
-			try :
-				print data
-				auth_parser.feed(data)
-				handle.read(64 * 1024, self.__auth_read_cb, parser)
-			except MagnatunePurchaseError, e:
-				self.__wait_dlg.destroy()
-				rb.error_dialog(title = _("Purchase Error"),
-						message = _("An error occurred while trying to purchase the album.\nThe Magnatune server returned:\n%s") % str(e))
-			except Exception, e:
-				self.__wait_dlg.destroy()
-				rb.error_dialog(title = _("Error"),
-						message = _("An error occurred while trying to purchase the album.\nThe error text is:\n%s") % str(e))
+			# process the URI: add authentication info, quote the filename component for some reason
+
+			parsed = urlparse.urlparse(buy_album_handler.url)
+			netloc = "%s:%s@%s" % (str(buy_album_handler.username), str(buy_album_handler.password), parsed.hostname)
+
+			spath = os.path.split(urllib.url2pathname(parsed.path))
+			basename = spath[1]
+			path = urllib.pathname2url(os.path.join(spath[0], urllib.quote(basename)))
+
+			authed = (parsed[0], netloc, path) + parsed[3:]
+			audio_dl_uri = urlparse.urlunparse(authed)
+
+			in_progress = open(os.path.join(magnatune_in_progress_dir, "in_progress_" + basename), 'w')
+			in_progress.write(str(audio_dl_uri))
+			in_progress.close()
+
+			self.__download_album(audio_dl_uri)
+
+		except MagnatunePurchaseError, e:
+			rb.error_dialog(title = _("Purchase Error"),
+					message = _("An error occurred while trying to purchase the album.\nThe Magnatune server returned:\n%s") % str(e))
+
+		except Exception, e:
+			rb.error_dialog(title = _("Error"),
+					message = _("An error occurred while trying to purchase the album.\nThe error text is:\n%s") % str(e))
+
+
+
 
 	def __download_album(self, audio_dl_uri):
-			library_location = self.__client.get_list("/apps/rhythmbox/library_locations", gconf.VALUE_STRING)[0] # Just use the first library location
-			to_file_uri = gnomevfs.URI(magnatune_dir + audio_dl_uri.short_name)
 
-			shell = self.get_property('shell')
-			manager = shell.get_player().get_property('ui-manager')
-			manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(True)
-			self.__downloading = True
-			self.cancelled = False
-			gtk.gdk.threads_leave()
-			self.purchase_filesize += gnomevfs.get_file_info(audio_dl_uri).size
-			gtk.gdk.threads_enter()
-			create_if_needed(to_file_uri, gnomevfs.OPEN_WRITE).close()
-			gnomevfs.async.xfer (source_uri_list = [audio_dl_uri],
-							target_uri_list = [to_file_uri],
-							xfer_options = gnomevfs.XFER_FOLLOW_LINKS_RECURSIVE,
-							error_mode = gnomevfs.XFER_ERROR_MODE_ABORT,
-							overwrite_mode = gnomevfs.XFER_OVERWRITE_MODE_REPLACE,
-							progress_update_callback = self.__purchase_download_update_cb,
-							update_callback_data = (to_file_uri, library_location, audio_dl_uri),
-							progress_sync_callback = self.__purchase_download_progress_cb,
-							sync_callback_data = (to_file_uri, audio_dl_uri))
+		fullpath = urlparse.urlparse(audio_dl_uri).path
+		basename = os.split(fullpath)[1]
+		destpath = os.path.join(magnatune_in_progress_dir, basename)
 
-	def __purchase_download_update_cb(self, _reserved, info, data):
-		if (info.phase == gnomevfs.XFER_PHASE_COMPLETED):
-			to_file_uri = data[0]
-			library_location = data[1]
-			audio_dl_uri = data[2]
+		shell = self.get_property('shell')
+		manager = shell.get_player().get_property('ui-manager')
+		manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(True)
+		self.__downloading = True
+		self.cancelled = False
 
-			try:
-				del self.__downloads[str(audio_dl_uri)]
-			except:
-				return 0
-			self.purchase_filesize -= gnomevfs.get_file_info(audio_dl_uri).size
-			album = zipfile.ZipFile(to_file_uri.path)
-			for track in album.namelist():
-				track_uri = gnomevfs.URI(library_location + "/" + track)
-				out = create_if_needed(track_uri, gnomevfs.OPEN_WRITE)
-				out.write(album.read(track))
-				out.close()
-			album.close()
-			gnomevfs.unlink(gnomevfs.URI(magnatune_dir + "in_progress_" + to_file_uri.short_name))
-			gnomevfs.unlink(to_file_uri)
-			if self.purchase_filesize == 0:
-				self.__downloading = False
-			self.__db.add_uri("file://" + urllib.quote(track_uri.dirname))
-		return 1
+		self.__downloads[audio_dl_uri] = 0
 
-	def __purchase_download_progress_cb(self, info, data):
-		to_file_uri = data[0]
-		audio_dl_uri = data[1]
+		# no way to resume downloads, sadly
+		out = open(destpath, 'w')
 
-		if self.cancelled:
-			try:
-				del self.__downloads[str(audio_dl_uri)]
-				self.purchase_filesize -= gnomevfs.get_file_info(audio_dl_uri).size
-				gnomevfs.unlink(gnomevfs.URI(magnatune_dir + "in_progress_" + to_file_uri.short_name))
-				gnomevfs.unlink(to_file_uri)
-			except: # this may get run more than once
-				pass
-			if self.purchase_filesize == 0:
-				self.__downloading = False
+		dl = rb.ChunkLoader()
+		dl.get_url_chunks(audio_dl_uri, 4*1024, True, self.__download_album_chunk, (audio_dl_uri, destpath, out))
+
+
+	def __remove_download_files (self, dest):
+		sp = os.path.split(dest)
+		inprogress = os.path.join(sp[0], "in_progress_" + sp[1])
+		os.unlink(inprogress)
+		os.unlink(dest)
+
+
+	def __download_finished (self, total, audio_dl_uri, dest, out):
+		try:
+			del self.__downloads[audio_dl_uri]
+		except:
 			return 0
 
-		self.__downloads[str(audio_dl_uri)] = info.bytes_copied
-		purchase_downloaded = 0
-		for i in self.__downloads.values():
-			purchase_downloaded += i
-		self.__download_progress = purchase_downloaded / float(self.purchase_filesize)
-		self.__notify_status_changed()
-		return 1
+		out.close()
+		self.purchase_filesize -= total
+
+		# just use the first library location
+		# not quite prepared to use gio here directly yet, so we can only deal with
+		# local libraries here.
+		library_location = self.__client.get_list("/apps/rhythmbox/library_locations", gconf.VALUE_STRING)[0]
+		if library_location.startswith("file://"):
+			urlpath = urlparse.urlparse(library_location).path
+			library_dir = urllib.url2pathname(urlpath)
+		else:
+			library_dir = rb.music_dir ()
+
+		album = zipfile.ZipFile(dest)
+		for track in album.namelist():
+			track_uri = "file://" + urllib.pathname2url(os.path.join(library_dir, track))
+
+			track_uri = rb.sanitize_uri_for_filesystem(track_uri)
+			rb.uri_create_parent_dirs(track_uri)
+
+			track_path = urllib.url2pathname(urlparse.urlparse(track_uri).path)
+
+			track_out = open(track_path, 'w')
+			track_out.write(album.read(track))
+			track_out.close()
+
+		album.close()
+
+		self.__remove_download_files (dest)
+
+		if self.purchase_filesize == 0:
+			self.__downloading = False
+
+		self.__db.add_uri(os.path.split(track_path)[0])
+
+
+	def __download_album_chunk(self, result, total, (audio_dl_uri, dest, out)):
+
+		if not result:
+			self.__download_finished (total, audio_dl_uri, dest, out)
+		elif isinstance(result, Exception):
+			# probably report this somehow?
+			pass
+		elif self.cancelled:
+			try:
+				del self.__downloads[audio_dl_uri]
+				self.purchase_filesize -= total
+
+				self.__remove_download_files (dest)
+
+			except:
+				pass
+
+			if self.purchase_filesize == 0:
+				self.__downloading = False
+
+			return False
+		else:
+			if self.__downloads[audio_dl_uri] == 0:
+				self.purchase_filesize += total
+
+			out.write(result)
+			self.__downloads[audio_dl_uri] += len(result)
+
+			self.__download_progress = sum(self.__downloads.values()) / float(self.purchase_filesize)
+			self.__notify_status_changed()
+
 
 	def cancel_downloads(self):
 		self.cancelled = True
@@ -610,30 +633,33 @@ class MagnatuneSource(rb.BrowserSource):
 		self.__db.emit_entry_extra_metadata_notify (entry, 'rb:coverArt-uri', url)
 		return False
 
+	def __move_data_files (self):
+		# create cache and data directories
+		# (we know they don't already exist, and we know the parent dirs do)
+		os.mkdir(magnatune_in_progress_dir, 0700)
+		if os.path.exists(magnatune_cache_dir) is False:
+			os.mkdir(magnatune_cache_dir, 0700)
+
+		# move song info to cache dir
+		old_magnatune_dir = os.path.join(rb.dot_dir(), 'magnatune')
+		if os.path.exists(old_magnatune_dir) is False:
+			print "old magnatune directory does not exist"
+			return
+
+		old_song_info = os.path.join(old_magnatune_dir, 'song_info.xml')
+		if os.path.exists(old_song_info):
+			print "moving existing song_info.xml to cache dir"
+			os.rename(old_song_info, magnatune_song_info)
+		else:
+			print "no song_info.xml found (%s)" % old_song_info
+
+		# move in progress downloads to data dir
+		otherfiles = os.listdir(old_magnatune_dir)
+		for f in otherfiles:
+			print "moving file %s to new in-progress dir" % f
+			os.rename(os.path.join(old_magnatune_dir, f),
+				  os.path.join(magnatune_in_progress_dir, f))
+
+
 gobject.type_register(MagnatuneSource)
 
-
-def create_if_needed(uri, mode):
-	if not gnomevfs.exists(uri):
-		for directory in URIIterator(uri):
-			if not gnomevfs.exists(directory):
-				gnomevfs.make_directory(directory, 0755)
-		out = gnomevfs.create(uri, open_mode=mode)
-	else:
-		out = gnomevfs.open(uri, open_mode=mode)
-	return out
-
-class URIIterator:
-	def __init__(self, uri):
-		self.uri_list = uri.dirname.split("/")[1:] # dirname starts with /
-		self.counter = 0
-	def __iter__(self):
-		return self
-	def next(self):
-		if self.counter == len(self.uri_list) + 1:
-			raise StopIteration
-		value = "file://"
-		for i in range(self.counter):
-			value += "/" + self.uri_list[i]
-		self.counter += 1
-		return gnomevfs.URI(value)
