@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -143,13 +143,13 @@ gboolean	rb_shell_quit (RBShell *shell,
 void            rb_shell_hidden_notify  (RBShell *shell,
 					 guint timeout,
 					 const char *primary,
-					 GtkWidget *icon,
+					 GdkPixbuf *pixbuf,
 					 const char *secondary,
 					 gboolean requested);
 void            rb_shell_hidden_notify_markup  (RBShell *shell,
 						guint timeout,
 						const char *primary_markup,
-						GtkWidget *icon,
+						GdkPixbuf *pixbuf,
 						const char *secondary_markup,
 						gboolean requested);
 void		rb_shell_notify_playing_entry (RBShell *shell,
