@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -19,8 +27,7 @@
  *
  */
 
-#include <gtk/gtkactiongroup.h>
-#include <gtk/gtkuimanager.h>
+#include <gtk/gtk.h>
 #include "eggtrayicon.h"
 #include "rhythmdb.h"
 #include "rb-shell.h"
@@ -37,19 +44,22 @@ G_BEGIN_DECLS
 #define RB_IS_TRAY_ICON_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_TRAY_ICON))
 #define RB_TRAY_ICON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_TRAY_ICON, RBTrayIconClass))
 
+typedef struct _RBTrayIcon RBTrayIcon;
+typedef struct _RBTrayIconClass RBTrayIconClass;
+
 typedef struct RBTrayIconPrivate RBTrayIconPrivate;
 
-typedef struct
+struct _RBTrayIcon
 {
 	EggTrayIcon parent;
 
 	RBTrayIconPrivate *priv;
-} RBTrayIcon;
+};
 
-typedef struct
+struct _RBTrayIconClass
 {
 	EggTrayIconClass parent_class;
-} RBTrayIconClass;
+};
 
 GType			rb_tray_icon_get_type	(void);
 
@@ -60,14 +70,14 @@ void                    rb_tray_icon_get_geom   (RBTrayIcon *icon, int *x, int *
 void                    rb_tray_icon_notify     (RBTrayIcon *icon,
 						 guint timeout,
 						 const char *primary_markup,
-						 GtkWidget *msgicon,
+						 GdkPixbuf *pixbuf,
 						 const char *secondary_markup,
 						 gboolean requested);
 
 void                    rb_tray_icon_cancel_notify (RBTrayIcon *icon);
 
-void rb_tray_icon_set_tooltip_primary_markup (RBTrayIcon *icon, const char *primary_markup);
-void rb_tray_icon_set_tooltip_icon (RBTrayIcon *icon, GtkWidget *msgicon);
+void rb_tray_icon_set_tooltip_primary_text (RBTrayIcon *icon, const char *primary_text);
+void rb_tray_icon_set_tooltip_icon (RBTrayIcon *icon, GdkPixbuf *pixbuf);
 void rb_tray_icon_set_tooltip_secondary_markup (RBTrayIcon *icon, const char *secondary_markup);
 
 G_END_DECLS
