@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -21,6 +29,23 @@
  *
  */
 
+/**
+ * SECTION:rb-library-source
+ * @short_description: main library source, containing all local songs
+ *
+ * The library source contains all local songs that have been imported
+ * into the database.
+ *
+ * It provides a preferences page for configuring the library location,
+ * the directory structure to use when transferring new files into
+ * the library from another source, and the preferred audio encoding
+ * to use.
+ *
+ * If multiple library locations are set in GConf, the library source
+ * creates a child source for each location, which will only show
+ * files found under that location.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -29,10 +54,8 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 
-#ifdef ENABLE_TRACK_TRANSFER
 #include <profiles/gnome-media-profiles.h>
 #include <profiles/audio-profile-choose.h>
-#endif
 
 #include "rhythmdb.h"
 #include "rb-debug.h"
@@ -60,9 +83,7 @@ static char *impl_get_browser_key (RBSource *source);
 static char *impl_get_paned_key (RBBrowserSource *source);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_can_paste (RBSource *asource);
-#ifdef ENABLE_TRACK_TRANSFER
 static void impl_paste (RBSource *source, GList *entries);
-#endif
 static guint impl_want_uri (RBSource *source, const char *uri);
 static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
 
@@ -73,7 +94,6 @@ static void rb_library_source_library_location_changed (GConfClient *client,
 						    guint cnxn_id,
 						    GConfEntry *entry,
 						    RBLibrarySource *source);
-#ifdef ENABLE_TRACK_TRANSFER
 static void rb_library_source_layout_path_changed (GConfClient *client,
 						   guint cnxn_id,
 						   GConfEntry *entry,
@@ -84,7 +104,6 @@ static void rb_library_source_layout_filename_changed (GConfClient *client,
 						       RBLibrarySource *source);
 static void rb_library_source_edit_profile_clicked_cb (GtkButton *button,
 						       RBLibrarySource *source);
-#endif
 static void rb_library_source_ui_pref_changed (GConfClient *client,
 					       guint cnxn_id,
 					       GConfEntry *entry,
@@ -95,14 +114,13 @@ static gboolean rb_library_source_library_location_cb (GtkEntry *entry,
 static void rb_library_source_watch_toggled_cb (GtkToggleButton *button,
 						RBLibrarySource *source);
 static void rb_library_source_sync_child_sources (RBLibrarySource *source);
-#ifdef ENABLE_TRACK_TRANSFER
 static void rb_library_source_path_changed_cb (GtkComboBox *box,
 						RBLibrarySource *source);
 static void rb_library_source_filename_changed_cb (GtkComboBox *box,
 						   RBLibrarySource *source);
 static void rb_library_source_format_changed_cb (GtkWidget *widget,
 						 RBLibrarySource *source);
-#endif
+static void layout_example_label_update (RBLibrarySource *source);
 
 #define CONF_UI_LIBRARY_DIR CONF_PREFIX "/ui/library"
 #define CONF_STATE_LIBRARY_DIR CONF_PREFIX "/state/library"
@@ -110,7 +128,6 @@ static void rb_library_source_format_changed_cb (GtkWidget *widget,
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/library/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/library/show_browser"
 
-#ifdef ENABLE_TRACK_TRANSFER
 typedef struct {
 	char *title;
 	char *path;
@@ -134,7 +151,6 @@ const LibraryPathElement library_layout_filenames[] = {
 	{N_("Number. Artist - Title"), "%tN. %ta - %tt"},
 };
 const int num_library_layout_filenames = G_N_ELEMENTS (library_layout_filenames);
-#endif
 
 struct RBLibrarySourcePrivate
 {
@@ -149,19 +165,15 @@ struct RBLibrarySourcePrivate
 
 	GtkWidget *library_location_entry;
 	GtkWidget *watch_library_check;
-#ifdef ENABLE_TRACK_TRANSFER
 	GtkWidget *layout_path_menu;
 	GtkWidget *layout_filename_menu;
 	GtkWidget *preferred_format_menu;
 	GtkWidget *layout_example_label;
-#endif
 
 	guint library_location_notify_id;
 	guint ui_dir_notify_id;
-#ifdef ENABLE_TRACK_TRANSFER
 	guint layout_path_notify_id;
 	guint layout_filename_notify_id;
-#endif
 };
 
 #define RB_LIBRARY_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LIBRARY_SOURCE, RBLibrarySourcePrivate))
@@ -184,9 +196,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) impl_can_paste;
-#ifdef ENABLE_TRACK_TRANSFER
 	source_class->impl_paste = impl_paste;
-#endif
 	source_class->impl_want_uri = impl_want_uri;
 	source_class->impl_add_uri = impl_add_uri;
 
@@ -195,9 +205,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	g_type_class_add_private (klass, sizeof (RBLibrarySourcePrivate));
 
-#ifdef ENABLE_TRACK_TRANSFER
 	gnome_media_profiles_init (eel_gconf_client_get_global ());
-#endif
 }
 
 static void
@@ -231,7 +239,7 @@ rb_library_source_dispose (GObject *object)
 		eel_gconf_notification_remove (source->priv->library_location_notify_id);
 		source->priv->library_location_notify_id = 0;
 	}
-#ifdef ENABLE_TRACK_TRANSFER
+
 	if (source->priv->layout_path_notify_id != 0) {
 		eel_gconf_notification_remove (source->priv->layout_path_notify_id);
 		source->priv->layout_path_notify_id = 0;
@@ -241,7 +249,6 @@ rb_library_source_dispose (GObject *object)
 		eel_gconf_notification_remove (source->priv->layout_filename_notify_id);
 		source->priv->layout_filename_notify_id = 0;
 	}
-#endif
 
 	G_OBJECT_CLASS (rb_library_source_parent_class)->dispose (object);
 }
@@ -273,6 +280,13 @@ add_child_sources_idle (RBLibrarySource *source)
 	return FALSE;
 }
 
+static void
+db_load_complete_cb (RhythmDB *db, RBLibrarySource *source)
+{
+	/* once the database is loaded, we can run the query to populate the library source */
+	g_object_set (source, "populate", TRUE, NULL);
+}
+
 static GObject *
 rb_library_source_constructor (GType type,
 			       guint n_construct_properties,
@@ -289,13 +303,42 @@ rb_library_source_constructor (GType type,
 	g_object_get (source, "shell", &shell, NULL);
 	g_object_get (shell, "db", &source->priv->db, NULL);
 
+	g_signal_connect_object (source->priv->db, "load-complete", G_CALLBACK (db_load_complete_cb), source, 0);
+
 	rb_library_source_ui_prefs_sync (source);
 
 	/* Set up the default library location if there's no library location set */
 	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
 	if (g_slist_length (list) == 0) {
-		list = g_slist_prepend (list, g_strdup (rb_music_dir ()));
-		eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+		char *music_dir_uri;
+
+		music_dir_uri = g_filename_to_uri (rb_music_dir (), NULL, NULL);
+		if (music_dir_uri != NULL) {
+			list = g_slist_prepend (list, music_dir_uri);
+			eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+		}
+	} else {
+		/* ensure all library locations are URIs and not file paths */
+		GSList *t;
+		gboolean update = FALSE;
+		for (t = list; t != NULL; t = t->next) {
+			char *location;
+
+			location = (char *)t->data;
+			if (location[0] == '/') {
+				char *uri = g_filename_to_uri (location, NULL, NULL);
+				if (uri != NULL) {
+					rb_debug ("converting library location path %s to URI %s", location, uri);
+					g_free (location);
+					t->data = uri;
+					update = TRUE;
+				}
+			}
+		}
+
+		if (update) {
+			eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+		}
 	}
 	rb_slist_deep_free (list);
 
@@ -320,6 +363,14 @@ rb_library_source_constructor (GType type,
 	return G_OBJECT (source);
 }
 
+/**
+ * rb_library_source_new:
+ * @shell: the #RBShell
+ *
+ * Creates and returns the #RBLibrarySource instance
+ *
+ * Return value: the #RBLibrarySource
+ */
 RBSource *
 rb_library_source_new (RBShell *shell)
 {
@@ -342,6 +393,7 @@ rb_library_source_new (RBShell *shell)
 					  "sorting-key", CONF_STATE_LIBRARY_SORTING,
 					  "shell", shell,
 					  "icon", icon,
+					  "populate", FALSE,		/* wait until the database is loaded */
 					  NULL));
 	if (icon != NULL) {
 		g_object_unref (icon);
@@ -352,7 +404,6 @@ rb_library_source_new (RBShell *shell)
 	return source;
 }
 
-#ifdef ENABLE_TRACK_TRANSFER
 static void
 rb_library_source_edit_profile_clicked_cb (GtkButton *button, RBLibrarySource *source)
 {
@@ -364,7 +415,6 @@ rb_library_source_edit_profile_clicked_cb (GtkButton *button, RBLibrarySource *s
 	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 }
-#endif
 
 static void
 rb_library_source_location_button_clicked_cb (GtkButton *button, RBLibrarySource *source)
@@ -382,7 +432,7 @@ rb_library_source_location_button_clicked_cb (GtkButton *button, RBLibrarySource
 			uri = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dialog));
 		}
 
-		path = gnome_vfs_format_uri_for_display (uri);
+		path = g_uri_unescape_string (uri, NULL);
 
 		gtk_entry_set_text (GTK_ENTRY (source->priv->library_location_entry), path);
 		rb_library_source_library_location_cb (GTK_ENTRY (source->priv->library_location_entry),
@@ -400,10 +450,8 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
 	GtkWidget *tmp;
 	GladeXML *xml;
-#ifdef ENABLE_TRACK_TRANSFER
 	GtkWidget *label;
 	int i;
-#endif
 
 	if (source->priv->config_widget)
 		return source->priv->config_widget;
@@ -434,13 +482,12 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 			  G_CALLBACK (rb_library_source_watch_toggled_cb),
 			  asource);
 
-#ifdef ENABLE_TRACK_TRANSFER
 	rb_glade_boldify_label (xml, "library_structure_label");
 
 	tmp = glade_xml_get_widget (xml, "layout_path_menu_box");
 	label = glade_xml_get_widget (xml, "layout_path_menu_label");
 	source->priv->layout_path_menu = gtk_combo_box_new_text ();
-	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->layout_path_menu);
+	gtk_box_pack_start (GTK_BOX (tmp), source->priv->layout_path_menu, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->layout_path_menu);
 	g_signal_connect (G_OBJECT (source->priv->layout_path_menu),
 			  "changed",
@@ -454,7 +501,7 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	tmp = glade_xml_get_widget (xml, "layout_filename_menu_box");
 	label = glade_xml_get_widget (xml, "layout_filename_menu_label");
 	source->priv->layout_filename_menu = gtk_combo_box_new_text ();
-	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->layout_filename_menu);
+	gtk_box_pack_start (GTK_BOX (tmp), source->priv->layout_filename_menu, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->layout_filename_menu);
 	g_signal_connect (G_OBJECT (source->priv->layout_filename_menu),
 			  "changed",
@@ -474,7 +521,7 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	tmp = glade_xml_get_widget (xml, "preferred_format_menu_box");
 	label = glade_xml_get_widget (xml, "preferred_format_menu_label");
 	source->priv->preferred_format_menu = gm_audio_profile_choose_new ();
-	gtk_box_pack_start_defaults (GTK_BOX (tmp), source->priv->preferred_format_menu);
+	gtk_box_pack_start (GTK_BOX (tmp), source->priv->preferred_format_menu, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), source->priv->preferred_format_menu);
 	g_signal_connect (G_OBJECT (source->priv->preferred_format_menu),
 			  "changed",
@@ -482,11 +529,6 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 			  asource);
 
 	source->priv->layout_example_label = glade_xml_get_widget (xml, "layout_example_label");
-#else
-	tmp = glade_xml_get_widget (xml, "library_structure_vbox");
-	gtk_widget_set_no_show_all (tmp, TRUE);
-	gtk_widget_hide (tmp);
-#endif
 
 	g_object_unref (G_OBJECT (xml));
 
@@ -528,10 +570,8 @@ static void
 rb_library_source_preferences_sync (RBLibrarySource *source)
 {
 	GSList *list;
-#ifdef ENABLE_TRACK_TRANSFER
 	char *str;
 	GConfClient *gconf_client;
-#endif
 
 	rb_debug ("syncing pref dialog state");
 
@@ -548,7 +588,7 @@ rb_library_source_preferences_sync (RBLibrarySource *source)
 
 		gtk_widget_set_sensitive (source->priv->library_location_entry, TRUE);
 
-		path = gnome_vfs_format_uri_for_display (list->data);
+		path = g_uri_unescape_string (list->data, NULL);
 		gtk_entry_set_text (GTK_ENTRY (source->priv->library_location_entry), path);
 		g_free (path);
 	} else if (g_slist_length (list) == 0) {
@@ -571,7 +611,6 @@ rb_library_source_preferences_sync (RBLibrarySource *source)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (source->priv->watch_library_check),
 				      eel_gconf_get_boolean (CONF_MONITOR_LIBRARY));
 
-#ifdef ENABLE_TRACK_TRANSFER
 	/* preferred format */
 	str = eel_gconf_get_string (CONF_LIBRARY_PREFERRED_FORMAT);
 	if (str) {
@@ -595,7 +634,6 @@ rb_library_source_preferences_sync (RBLibrarySource *source)
 	rb_library_source_layout_filename_changed (gconf_client, -1,
 						   gconf_client_get_entry (gconf_client, CONF_LIBRARY_LAYOUT_FILENAME, NULL, TRUE, NULL),
 						   source);
-#endif
 }
 
 static gboolean
@@ -605,19 +643,20 @@ rb_library_source_library_location_cb (GtkEntry *entry,
 {
 	GSList *list = NULL;
 	const char *path;
+	GFile *file;
 	char *uri;
 
 	path = gtk_entry_get_text (entry);
-	uri = gnome_vfs_make_uri_from_input (path);
+	file = g_file_parse_name (path);
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
 
 	if (uri && uri[0])
 		list = g_slist_prepend (NULL, (gpointer)uri);
 
 	eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
 
-	g_free (uri);
-	if (list)
-		rb_slist_deep_free (list);
+	rb_slist_deep_free (list);
 
 	return FALSE;
 }
@@ -643,75 +682,6 @@ impl_get_paned_key (RBBrowserSource *status)
 	return g_strdup (CONF_STATE_PANED_POSITION);
 }
 
-static void
-rb_library_source_add_location_entry_changed_cb (GtkEntry *entry,
-						 GtkWidget *target)
-{
-	gtk_widget_set_sensitive (target, g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY (entry)), -1) > 0);
-}
-
-void
-rb_library_source_add_location (RBLibrarySource *source, GtkWindow *win)
-{
-	GladeXML *xml = rb_glade_xml_new ("uri.glade",
-					  "open_uri_dialog_content",
-					  source);
-	GtkWidget *content, *uri_widget, *open_button;
-	GtkWidget *dialog = gtk_dialog_new_with_buttons (_("Add Location"),
-							 win,
-							 GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
-							 GTK_STOCK_CANCEL,
-							 GTK_RESPONSE_CANCEL,
-							 NULL);
-
-	g_return_if_fail (dialog != NULL);
-
-	open_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
-					     GTK_STOCK_OPEN,
-					     GTK_RESPONSE_OK);
-	gtk_widget_set_sensitive (open_button, FALSE);
-
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-					 GTK_RESPONSE_OK);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
-
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-
-	content = glade_xml_get_widget (xml, "open_uri_dialog_content");
-
-	g_return_if_fail (content != NULL);
-
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-			    content, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (content), 5);
-
-	uri_widget = glade_xml_get_widget (xml, "uri");
-
-	g_return_if_fail (uri_widget != NULL);
-
-	g_signal_connect_object (G_OBJECT (uri_widget),
-				 "changed",
-				 G_CALLBACK (rb_library_source_add_location_entry_changed_cb),
-				 open_button, 0);
-
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
-		char *uri = gtk_editable_get_chars (GTK_EDITABLE (uri_widget), 0, -1);
-		if (uri != NULL) {
-			GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
-			if (vfsuri != NULL) {
-				rhythmdb_add_uri (source->priv->db, uri);
-				gnome_vfs_uri_unref (vfsuri);
-			} else {
-				rb_debug ("invalid uri: \"%s\"", uri);
-			}
-
-		}
-	}
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-}
-
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 {
@@ -761,7 +731,6 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-#ifdef ENABLE_TRACK_TRANSFER
 static void
 rb_library_source_path_changed_cb (GtkComboBox *box, RBLibrarySource *source)
 {
@@ -791,19 +760,21 @@ rb_library_source_format_changed_cb (GtkWidget *widget, RBLibrarySource *source)
 
 	profile = gm_audio_profile_choose_get_active (widget);
 	eel_gconf_set_string (CONF_LIBRARY_PREFERRED_FORMAT, gm_audio_profile_get_id (profile));
+	
+	layout_example_label_update (source);
 }
 
 /**
  * Perform magic on a path to make it safe.
  *
- * This will always replace '/' with ' ', and optionally make the file name
- * shell-friendly. This involves removing [?*\ ] and replacing with '_'.  Also
- * any leading periods are removed so that the files don't end up being hidden.
+ * This will always replace '/' with '-', and optionally make the file name
+ * shell-friendly. This involves removing replacing shell metacharacters and all
+ * whitespace with '_'. Also any leading periods are removed so that the files
+ * don't end up being hidden.
  */
 static char *
 sanitize_path (const char *str)
 {
-	gchar *res = NULL;
 	gchar *s;
 
 	/* Skip leading periods, otherwise files disappear... */
@@ -822,9 +793,8 @@ sanitize_path (const char *str)
 		/* TODO: I'd like this to compress whitespace aswell */
 		g_strdelimit (s, "\t ", '_');
 	}
-	res = g_filename_from_utf8(s, -1, NULL, NULL, NULL);
-	g_free(s);
-	return res ? res : g_strdup(str);
+
+	return s;
 }
 
 static char *
@@ -841,9 +811,8 @@ sanitize_pattern (const char *pat)
 	}
 }
 
-/**
- * Parse a filename pattern and replace markers with values from a TrackDetails
- * structure.
+/*
+ * Parse a filename pattern and replace markers with values from a RhythmDBEntry
  *
  * Valid markers so far are:
  * %at -- album title
@@ -876,6 +845,7 @@ filepath_parse_pattern (const char *pattern,
 	p = pattern;
 	while (*p) {
 		char *string = NULL;
+		char *t;
 
 		/* If not a % marker, copy and continue */
 		if (*p != '%') {
@@ -901,7 +871,7 @@ filepath_parse_pattern (const char *pattern,
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM));
 				break;
 			case 'T':
-				temp = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_FOLDED));
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM_FOLDED));
 				break;
 			case 'a':
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST));
@@ -909,14 +879,14 @@ filepath_parse_pattern (const char *pattern,
 			case 'A':
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_FOLDED));
 				break;
-			/*case 's':
-				string = sanitize_path (album sort name);
+			case 's':
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
 				break;
 			case 'S':
-				char *t = g_utf8_strdown (album sort name)
+				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
 				string = sanitize_path (t);
 				g_free (t);
-				break;*/
+				break;
 			default:
 				string = g_strdup_printf ("%%a%c", *p);
 			}
@@ -940,14 +910,14 @@ filepath_parse_pattern (const char *pattern,
 			case 'A':
 				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_FOLDED));
 				break;
-			/*case 's':
-				string = sanitize_path (artist sort name);
+			case 's':
+				string = sanitize_path (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME));
 				break;
 			case 'S':
-				char *t = g_utf8_strdown (artist sort name)
+				t = g_utf8_strdown (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST_SORTNAME), -1);
 				string = sanitize_path (t);
 				g_free (t);
-				break;*/
+				break;
 			case 'n':
 				/* Track number */
 				string = g_strdup_printf ("%u", (guint)rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER));
@@ -1109,8 +1079,9 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 static char*
 build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 {
-	GnomeVFSURI *uri;
-	GnomeVFSURI *new;
+	GFile *library_location;
+	GFile *dir;
+	GFile *dest;
 	char *realfile;
 	char *realpath;
 	char *filename;
@@ -1137,15 +1108,11 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 	g_free (layout_filename);
 	layout_filename = tmp;
 
-	uri = gnome_vfs_uri_new ((const char *)list->data);
-	if (uri == NULL) {
-		goto out;
-	}
-
 	realpath = filepath_parse_pattern (layout_path, entry);
-	new = gnome_vfs_uri_append_path (uri, realpath);
-	gnome_vfs_uri_unref (uri);
-	uri = new;
+
+	library_location = g_file_new_for_uri ((const char *)list->data);
+	dir = g_file_resolve_relative_path (library_location, realpath);
+	g_object_unref (library_location);
 	g_free (realpath);
 
 	if (g_str_has_prefix (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE), "audio/x-raw")) {
@@ -1183,14 +1150,13 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 		filename = realfile;
 	}
 
-	new = gnome_vfs_uri_append_file_name (uri, filename);
-	gnome_vfs_uri_unref (uri);
-	uri = new;
+	dest = g_file_resolve_relative_path (dir, filename);
+	g_object_unref (dir);
 	g_free (extension);
 	g_free (filename);
 
-	string = gnome_vfs_uri_to_string (uri, 0);
-	gnome_vfs_uri_unref (uri);
+	string = g_file_get_uri (dest);
+	g_object_unref (dest);
  out:
 	rb_slist_deep_free (list);
 	g_free (layout_path);
@@ -1199,12 +1165,10 @@ build_filename (RBLibrarySource *source, RhythmDBEntry *entry)
 
 	return string;
 }
-#endif
 
 static gboolean
 impl_can_paste (RBSource *asource)
 {
-#ifdef ENABLE_TRACK_TRANSFER
 	GSList *list;
 	gboolean can_paste = TRUE;
 	char *str;
@@ -1225,14 +1189,10 @@ impl_can_paste (RBSource *asource)
 	can_paste &= (str != NULL);
 	g_free (str);
 	return can_paste;
-#else
-	return FALSE;
-#endif
 }
 
-#ifdef ENABLE_TRACK_TRANSFER
 static void
-completed_cb (RhythmDBEntry *entry, const char *dest, RBLibrarySource *source)
+completed_cb (RhythmDBEntry *entry, const char *dest, guint64 dest_size, RBLibrarySource *source)
 {
 	rhythmdb_add_uri (source->priv->db, dest);
 }
@@ -1266,18 +1226,22 @@ impl_paste (RBSource *asource, GList *entries)
 		RhythmDBEntryType entry_type;
 		RBSource *source_source;
 		char *dest;
+		char *sane_dest;
 
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
 		entry_type = rhythmdb_entry_get_entry_type (entry);
-		if (entry_type == source_entry_type)
-			/* copying to ourselves would be silly */
+		if (entry_type == source_entry_type) {
+			rb_debug ("can't copy an entry from the library to itself");
 			continue;
+		}
 
 		/* see if the responsible source lets us copy */
 		source_source = rb_shell_get_source_by_entry_type (shell, entry_type);
-		if ((source_source != NULL) && !rb_source_can_copy (source_source))
+		if ((source_source != NULL) && !rb_source_can_copy (source_source)) {
+			rb_debug ("source for the entry doesn't want us to copy it");
 			continue;
+		}
 
 		dest = build_filename (source, entry);
 		if (dest == NULL) {
@@ -1285,15 +1249,17 @@ impl_paste (RBSource *asource, GList *entries)
 			continue;
 		}
 
+		sane_dest = rb_sanitize_uri_for_filesystem (dest);
+		g_free (dest);
+
 		rb_removable_media_manager_queue_transfer (rm_mgr, entry,
-							  dest, NULL,
-							  (RBTranferCompleteCallback)completed_cb, source);
+							  sane_dest, NULL,
+							  (RBTransferCompleteCallback)completed_cb, source);
 	}
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
 
 	g_object_unref (rm_mgr);
 }
-#endif
 
 static guint
 impl_want_uri (RBSource *source, const char *uri)
@@ -1324,26 +1290,21 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	RBSource *source;
 	GPtrArray *query;
 	RBShell *shell;
-	GnomeVFSURI *uri;
 	char *name;
 	GdkPixbuf *icon;
 	RhythmDBEntryType entry_type;
 	char *sort_column;
 	int sort_order;
+	GFile *file;
 
 	g_object_get (library_source,
 		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
-	uri = gnome_vfs_uri_new (path);
-	if (uri == NULL) {
-		g_object_unref (shell);
-		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
-		return;
-	}
 
-	name = gnome_vfs_uri_extract_short_name (uri);
-	gnome_vfs_uri_unref (uri);
+	file = g_file_new_for_uri (path);		/* ? */
+	name = g_file_get_basename (file);
+	g_object_unref (file);
 
 	rb_entry_view_get_sorting_order (rb_source_get_entry_view (RB_SOURCE (library_source)),
 					 &sort_column, &sort_order);
