@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -29,7 +37,6 @@
 #include <libxml/tree.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
 #include <totem-pl-parser.h>
 
 #include "rb-entry-view.h"
@@ -47,6 +54,24 @@
 #include "rb-static-playlist-source.h"
 #include "rb-auto-playlist-source.h"
 
+#include "rb-playlist-manager.h"
+
+/**
+ * SECTION:rb-playlist-source
+ * @short_description: Base class for playlist sources
+ *
+ * This class provides some common infrastructure for playlist
+ * sources.  A playlist, in this context, is a persistent user-defined
+ * selection from a set of songs.  Playlists (static and auto) based 
+ * on the main library are saved to the playlists.xml file stored
+ * alongside the rhythmdb.xml file.  Playlists on portable music players
+ * are saved on the device in the format the player itself supports.
+ *
+ * This class provides most of the source UI (excluding the search bar),
+ * holds some of the framework for loading and saving the playlists.xml
+ * file, and records which playlists need to be saved.
+ */
+
 static void rb_playlist_source_class_init (RBPlaylistSourceClass *klass);
 static void rb_playlist_source_init (RBPlaylistSource *source);
 static GObject *rb_playlist_source_constructor (GType type,
@@ -92,7 +117,12 @@ static void rb_playlist_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entr
 static void rb_playlist_source_track_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 						     GtkTreeModel *tree_model, GtkTreeIter *iter,
 						     RBPlaylistSource *source);
+static void default_mark_dirty (RBPlaylistSource *source);
+static void rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view,
+						RBStaticPlaylistSource *source);
+static char *rb_playlist_source_make_sorting_key (RBPlaylistSource *source);
 
+#define CONF_STATE_SORTING_PREFIX CONF_PREFIX "/state/sorting/"
 #define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/PlaylistViewPopup"
 #define PLAYLIST_SOURCE_POPUP_PATH "/PlaylistSourcePopup"
 
@@ -111,6 +141,7 @@ struct RBPlaylistSourcePrivate
 	gboolean dispose_has_run;
 
 	char *title;
+	char *sorting_name;
 };
 
 #define RB_PLAYLIST_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PLAYLIST_SOURCE, RBPlaylistSourcePrivate))
@@ -121,6 +152,7 @@ enum
 	PROP_DB,
 	PROP_DIRTY,
 	PROP_LOCAL,
+	PROP_SORTING_NAME
 };
 
 static const GtkTargetEntry target_uri [] = { { "text/uri-list", 0, 0 } };
@@ -143,7 +175,6 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_get_browser_key = impl_get_browser_key;
 	source_class->impl_get_entry_view = impl_get_entry_view;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
@@ -154,7 +185,13 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_show_popup = impl_show_popup;
 
 	klass->impl_show_entry_view_popup = default_show_entry_view_popup;
+	klass->impl_mark_dirty = default_mark_dirty;
 
+	/**
+	 * RBPlaylistSource:db:
+	 *
+	 * The #RhythmDB instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_DB,
 					 g_param_spec_object ("db",
@@ -162,6 +199,12 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							      "rhythmdb instance",
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READABLE));
+	/**
+	 * RBPlaylistSource:dirty:
+	 *
+	 * Whether the playlist has been changed since it was last saved
+	 * to disk.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_DIRTY,
 					 g_param_spec_boolean ("dirty",
@@ -169,6 +212,12 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       "whether this playlist should be saved",
 							       FALSE,
 							       G_PARAM_READABLE));
+	/**
+	 * RBPlaylistSource:is-local:
+	 *
+	 * Whether the playlist is attached to the local library.
+	 * Remote DAAP playlists, for example, are not local.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_LOCAL,
 					 g_param_spec_boolean ("is-local",
@@ -176,6 +225,19 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       "whether this playlist is attached to the local library",
 							       TRUE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBPlaylistSource:sorting-name:
+	 *
+	 * A unique-ish name for the playlist, used to construct gconf keys
+	 * to store information relating to the playlist.
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_SORTING_NAME,
+					 g_param_spec_string ("sorting-name",
+					 		      "sorting-name",
+							      "globally unique name for storing sort-order",
+					 		      NULL,
+							      G_PARAM_READWRITE));
 
 	g_type_class_add_private (klass, sizeof (RBPlaylistSourcePrivate));
 }
@@ -220,6 +282,7 @@ rb_playlist_source_constructor (GType type,
 	RBShell *shell;
 	RhythmDB *db;
 	RhythmDBQueryModel *query_model;
+	char *sorting_key;
 
 	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
 
@@ -236,9 +299,18 @@ rb_playlist_source_constructor (GType type,
 	source->priv->entries = g_hash_table_new_full (rb_refstring_hash, rb_refstring_equal,
 						       (GDestroyNotify)rb_refstring_unref, NULL);
 
+	/* If a sorting key is available at construction time, use it. This may be
+	 * NULL in which case a key may be assigned later through a set property. */
+	sorting_key = rb_playlist_source_make_sorting_key (source);
 	source->priv->songs = rb_entry_view_new (source->priv->db,
 						 shell_player,
-					 	 NULL, TRUE, TRUE);
+					 	 sorting_key, TRUE, TRUE);
+	g_free (sorting_key);
+
+	g_signal_connect_object (source->priv->songs,
+				 "sort-order-changed",
+				 G_CALLBACK (rb_playlist_source_songs_sort_order_changed_cb),
+				 source, 0);
 
 	query_model = rhythmdb_query_model_new_empty (source->priv->db);
 	rb_playlist_source_set_query_model (source, query_model);
@@ -353,11 +425,21 @@ rb_playlist_source_set_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (object);
+	char *sorting_key;
 
 	switch (prop_id) {
 	case PROP_LOCAL:
 		source->priv->is_local = g_value_get_boolean (value);
 		break;
+	case PROP_SORTING_NAME:
+		g_free (source->priv->sorting_name);
+		source->priv->sorting_name = g_value_dup_string (value);
+
+		/* propagate sorting key to the entry view */
+		sorting_key = rb_playlist_source_make_sorting_key (source);
+		g_object_set (source->priv->songs, "sort-key", sorting_key, NULL);
+		g_free (sorting_key);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -382,6 +464,9 @@ rb_playlist_source_get_property (GObject *object,
 	case PROP_LOCAL:
 		g_value_set_boolean (value, source->priv->is_local);
 		break;
+	case PROP_SORTING_NAME:
+		g_value_set_string (value, source->priv->sorting_name);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -469,30 +554,6 @@ rb_playlist_source_drop_cb (GtkWidget *widget,
 	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
-#ifndef TOTEM_PL_PARSER_CHECK_VERSION
-static void
-playlist_iter_func (GtkTreeModel *model,
-		    GtkTreeIter *iter,
-		    char **uri,
-		    char **title,
-		    gpointer user_data)
-{
-	RhythmDBEntry *entry;
-
-	gtk_tree_model_get (model, iter, 0, &entry, -1);
-
-	if (uri != NULL) {
-		*uri = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_LOCATION);
-	}
-	if (title != NULL) {
-		*title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
-	}
-
-	if (entry != NULL) {
-		rhythmdb_entry_unref (entry);
-	}
-}
-#else
 static void
 playlist_iter_func (GtkTreeModel *model,
 		    GtkTreeIter *iter,
@@ -519,16 +580,25 @@ playlist_iter_func (GtkTreeModel *model,
 		rhythmdb_entry_unref (entry);
 	}
 }
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
+/**
+ * rb_playlist_source_save_playlist:
+ * @source: a #RBPlaylistSource
+ * @uri: destination URI
+ * @export_type: format to save in
+ *
+ * Saves the playlist to an external file in a standard
+ * format (M3U, PLS, or XSPF).
+ */
 void
 rb_playlist_source_save_playlist (RBPlaylistSource *source,
 				  const char *uri,
-				  gboolean m3u_format)
+				  RBPlaylistExportType export_type)
 {
 	TotemPlParser *playlist;
 	GError *error = NULL;
 	char *name;
+	gint totem_format;
 
 	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
 
@@ -537,9 +607,22 @@ rb_playlist_source_save_playlist (RBPlaylistSource *source,
 
 	g_object_get (source, "name", &name, NULL);
 
+	switch (export_type) {
+	case RB_PLAYLIST_EXPORT_TYPE_XSPF:
+		totem_format = TOTEM_PL_PARSER_XSPF;
+		break;
+	case RB_PLAYLIST_EXPORT_TYPE_M3U:
+		totem_format = TOTEM_PL_PARSER_M3U;
+		break;
+	case RB_PLAYLIST_EXPORT_TYPE_PLS:
+	default:
+		totem_format = TOTEM_PL_PARSER_PLS;
+		break;
+	}
+
 	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
 					  playlist_iter_func, uri, name,
-					  m3u_format ? TOTEM_PL_PARSER_M3U : TOTEM_PL_PARSER_PLS,
+					  totem_format,
 					  NULL, &error);
 	g_object_unref (playlist);
 	g_free (name);
@@ -631,6 +714,19 @@ get_playlist_name_from_xml (xmlNodePtr node)
 	return name;
 }
 
+/**
+ * rb_playlist_source_new_from_xml:
+ * @shell: the #RBShell instance
+ * @node: libxml node containing the playlist
+ *
+ * Constructs a playlist source instance from the XML serialized
+ * format.  This function knows about all the playlist types that
+ * can be saved to disk, and it hands off the XML node to the
+ * appropriate constructor based on the 'type' attribute of
+ * the root node of the playlist.
+ *
+ * Return value: the playlist
+ */
 RBSource *
 rb_playlist_source_new_from_xml	(RBShell *shell,
 				 xmlNodePtr node)
@@ -670,6 +766,14 @@ rb_playlist_source_new_from_xml	(RBShell *shell,
 	return source;
 }
 
+/**
+ * rb_playlist_source_save_to_xml:
+ * @source: the playlist source to save
+ * @parent_node: libxml node below which to save the playlist
+ *
+ * Converts the playlist to XML format, below the specified
+ * parent node.
+ */
 void
 rb_playlist_source_save_to_xml (RBPlaylistSource *source,
 				xmlNodePtr parent_node)
@@ -719,8 +823,12 @@ rb_playlist_source_entry_added_cb (RhythmDB *db,
 	location = rhythmdb_entry_get_refstring (entry, RHYTHMDB_PROP_LOCATION);
 
 	if (g_hash_table_lookup (source->priv->entries, location)) {
-		rhythmdb_query_model_add_entry (source->priv->model, entry, -1);
-		source->priv->dirty = TRUE;
+		if (_rb_source_check_entry_type (RB_SOURCE (source), entry)) {
+			rhythmdb_query_model_add_entry (source->priv->model, entry, -1);
+			source->priv->dirty = TRUE;
+		} else {
+			g_hash_table_remove (source->priv->entries, location);
+		}
 	}
 
 	rb_refstring_unref (location);
@@ -747,6 +855,15 @@ rb_playlist_source_track_cell_data_func (GtkTreeViewColumn *column,
 	g_free (str);
 }
 
+/**
+ * rb_playlist_source_setup_entry_view:
+ * @source: the #RBPlaylistSource
+ * @entry_view: the new #RBEntryView to set up
+ *
+ * Connects signal handlers and sets up drag and drop support for
+ * an entry view to be used by a playlist source.  This only needs
+ * to be called if the playlist subclass is creating a new entry view.
+ */
 void
 rb_playlist_source_setup_entry_view (RBPlaylistSource *source,
 				     RBEntryView *entry_view)
@@ -764,6 +881,18 @@ rb_playlist_source_setup_entry_view (RBPlaylistSource *source,
 			   GDK_ACTION_COPY);
 }
 
+/**
+ * rb_playlist_source_set_query_model:
+ * @source: the #RBPlaylistSource
+ * @model: the new #RhythmDBQueryModel
+ *
+ * Sets a new query model for the playlist.  This updates the
+ * entry view to use the new query model and also updates the
+ * source query-model property.
+ *
+ * This needs to be called when the playlist subclass
+ * creates a new query model.
+ */
 void
 rb_playlist_source_set_query_model (RBPlaylistSource *source,
 				    RhythmDBQueryModel *model)
@@ -794,6 +923,15 @@ rb_playlist_source_set_query_model (RBPlaylistSource *source,
 	g_object_set (source, "query-model", source->priv->model, NULL);
 }
 
+/**
+ * rb_playlist_source_get_db:
+ * @source: a #RBPlaylistSource
+ *
+ * Returns the #RhythmDB instance.  The caller must not
+ * unref the object once finished with it.
+ *
+ * Return value: the #RhythmDB instance
+ */
 RhythmDB *
 rb_playlist_source_get_db (RBPlaylistSource *source)
 {
@@ -802,6 +940,15 @@ rb_playlist_source_get_db (RBPlaylistSource *source)
 	return source->priv->db;
 }
 
+/**
+ * rb_playlist_source_get_query_model:
+ * @source: a #RBPlaylistSource
+ *
+ * Returns the current #RhythmDBQueryModel for the playlist.
+ * The caller must not unref the object once finished with it.
+ *
+ * Return value: the current #RhythmDBQueryModel
+ */
 RhythmDBQueryModel *
 rb_playlist_source_get_query_model (RBPlaylistSource *source)
 {
@@ -810,14 +957,37 @@ rb_playlist_source_get_query_model (RBPlaylistSource *source)
 	return source->priv->model;
 }
 
-void
-rb_playlist_source_mark_dirty (RBPlaylistSource *source)
+static void
+default_mark_dirty (RBPlaylistSource *source)
 {
-	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
-
 	source->priv->dirty = TRUE;
 }
 
+/**
+ * rb_playlist_source_mark_dirty:
+ * @source: a #RBPlaylistSource
+ *
+ * Marks the playlist dirty.  This generally means that the playlist
+ * will be saved to disk fairly soon, but the exact meaning can vary
+ * between playlist types.
+ */
+void
+rb_playlist_source_mark_dirty (RBPlaylistSource *source)
+{
+	RBPlaylistSourceClass *klass;
+	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (source));
+
+	klass = RB_PLAYLIST_SOURCE_GET_CLASS (source);
+	klass->impl_mark_dirty (source);
+}
+
+/**
+ * rb_playlist_source_location_in_map:
+ * @source: a #RBPlaylistSource
+ * @location: a URI to check
+ *
+ * Return value: TRUE if the specified URI is in the playlist entry map
+ */
 gboolean
 rb_playlist_source_location_in_map (RBPlaylistSource *source,
 				    const char *location)
@@ -838,6 +1008,18 @@ rb_playlist_source_location_in_map (RBPlaylistSource *source,
 	return found;
 }
 
+/**
+ * rb_playlist_source_add_to_map:
+ * @source: a #RBPlaylistSource
+ * @location: a URI to add
+ *
+ * Adds a URI to the playlist's entry map.  This is useful when the
+ * URI is being added to the database, but no entry exists for it yet.
+ * When the entry is created, it will be added to the query model.
+ *
+ * Return value: TRUE if the URI was added to the entry map,
+ *  FALSE if it was already there.
+ */
 gboolean
 rb_playlist_source_add_to_map (RBPlaylistSource *source,
 			       const char *location)
@@ -857,3 +1039,40 @@ rb_playlist_source_add_to_map (RBPlaylistSource *source,
 
 	return TRUE;
 }
+
+static void
+rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view,
+						RBStaticPlaylistSource *source)
+{
+	rb_debug ("sort order changed");
+	rb_entry_view_resort_model (view);
+}
+
+/* takes the 'sorting-name' property and produces a full GConf key for storing
+ * the playlist's column sort order
+ */
+static char *
+rb_playlist_source_make_sorting_key (RBPlaylistSource *source)
+{
+	char *sorting_name;
+	char *sorting_key;
+	char *sorting_key_tail;
+	g_object_get (source, "sorting-name", &sorting_name, NULL);
+
+	if (sorting_name && sorting_name[0] != '\0') {
+		/* escape invalid characters in the key name */
+		sorting_key_tail = gconf_escape_key (sorting_name, -1);
+
+		/* bind column sort order to a full gconf key */
+		sorting_key = g_strjoin (NULL, CONF_STATE_SORTING_PREFIX, 
+					sorting_key_tail, NULL);
+
+		g_free (sorting_key_tail);
+	} else {
+		sorting_key = NULL;
+	}
+	g_free (sorting_name);
+
+	return sorting_key;
+}
+
