@@ -4,7 +4,7 @@
  * arch-tag: Implementation of star rating GtkTreeView cell renderer
  *
  * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
- * Copyright (C) 2002  Olivier Martin <oleevye@wanadoo.fr>
+ * Copyright (C) 2002  Olivier Martin <olive.martin@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -59,7 +59,7 @@ static void rb_cell_renderer_rating_render (GtkCellRenderer *cell,
 					    GdkRectangle *expose_area,
 					    GtkCellRendererState flags);
 static gboolean rb_cell_renderer_rating_activate (GtkCellRenderer *cell,
-					          GdkEvent *event, 
+					          GdkEvent *event,
 					          GtkWidget *widget,
 					          const gchar *path,
 					          GdkRectangle *background_area,
@@ -74,7 +74,7 @@ struct RBCellRendererRatingPrivate
 
 struct RBCellRendererRatingClassPrivate
 {
-	RBRatingPixbufs *pixbufs;	
+	RBRatingPixbufs *pixbufs;
 };
 
 G_DEFINE_TYPE (RBCellRendererRating, rb_cell_renderer_rating, GTK_TYPE_CELL_RENDERER)
@@ -83,7 +83,15 @@ G_DEFINE_TYPE (RBCellRendererRating, rb_cell_renderer_rating, GTK_TYPE_CELL_REND
 						RBCellRendererRatingPrivate))
 
 
-enum 
+/**
+ * SECTION:rb-cell-renderer-rating
+ * @short_description: a #GtkCellRenderer for displaying song ratings
+ *
+ * This cell renderer is used to display song ratings in the #RBEntryView,
+ * and allows the user to modify ratings directly in the track listing.
+ */
+
+enum
 {
 	PROP_0,
 	PROP_RATING
@@ -97,7 +105,6 @@ enum
 
 static guint rb_cell_renderer_rating_signals[LAST_SIGNAL] = { 0 };
 
-
 static void
 rb_cell_renderer_rating_init (RBCellRendererRating *cellrating)
 {
@@ -128,9 +135,23 @@ rb_cell_renderer_rating_class_init (RBCellRendererRatingClass *class)
 	class->priv = g_new0 (RBCellRendererRatingClassPrivate, 1);
 	class->priv->pixbufs = rb_rating_pixbufs_new ();
 
+	/**
+	 * RBCellRendererRating:rating:
+	 *
+	 * The rating displayed by the renderer, as a floating point value
+	 * between 0.0 and 5.0.
+	 */
 	rb_rating_install_rating_property (object_class, PROP_RATING);
 
-	rb_cell_renderer_rating_signals[RATED] = 
+	/**
+	 * RBCellRendererRating::rated:
+	 * @renderer: the #RBCellRendererRating
+	 * @score: the new rating
+	 * @path: string form of the #GtkTreePath to the row that was changed
+	 *
+	 * Emitted when the user changes the rating.
+	 */
+	rb_cell_renderer_rating_signals[RATED] =
 		g_signal_new ("rated",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -162,7 +183,7 @@ rb_cell_renderer_rating_get_property (GObject *object,
 				      GParamSpec *pspec)
 {
 	RBCellRendererRating *cellrating = RB_CELL_RENDERER_RATING (object);
-  
+
 	switch (param_id) {
 	case PROP_RATING:
 		g_value_set_double (value, cellrating->priv->rating);
@@ -173,7 +194,6 @@ rb_cell_renderer_rating_get_property (GObject *object,
 	}
 }
 
-
 static void
 rb_cell_renderer_rating_set_property (GObject *object,
 				      guint param_id,
@@ -181,7 +201,7 @@ rb_cell_renderer_rating_set_property (GObject *object,
 				      GParamSpec *pspec)
 {
 	RBCellRendererRating *cellrating= RB_CELL_RENDERER_RATING (object);
-  
+
 	switch (param_id) {
 	case PROP_RATING:
 		cellrating->priv->rating = g_value_get_double (value);
@@ -195,17 +215,18 @@ rb_cell_renderer_rating_set_property (GObject *object,
 }
 
 /**
- * rb_cell_renderer_rating_new: create a cell renderer that will 
+ * rb_cell_renderer_rating_new:
+ *
+ * create a cell renderer that will
  * display some pixbufs for representing the rating of a song.
  * It is also able to update the rating.
- *  
+ *
  * Return value: the new cell renderer
  **/
-
 GtkCellRenderer *
 rb_cell_renderer_rating_new ()
 {
-	return GTK_CELL_RENDERER (gtk_type_new (rb_cell_renderer_rating_get_type ()));
+	return GTK_CELL_RENDERER (g_object_new (rb_cell_renderer_rating_get_type (), NULL, NULL));
 }
 
 static void
@@ -224,7 +245,7 @@ rb_cell_renderer_rating_get_size (GtkCellRenderer *cell,
 
 	if (x_offset)
 		*x_offset = 0;
-	
+
 	if (y_offset)
 		*y_offset = 0;
 
@@ -257,30 +278,27 @@ rb_cell_renderer_rating_render (GtkCellRenderer  *cell,
 					  &pix_rect.y,
 					  &pix_rect.width,
 					  &pix_rect.height);
-  
+
 	pix_rect.x += cell_area->x;
 	pix_rect.y += cell_area->y;
 	pix_rect.width -= cell->xpad * 2;
 	pix_rect.height -= cell->ypad * 2;
-	
-	
+
 	if (gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect) == FALSE)
 		return;
 
-
-
 	selected = (flags & GTK_CELL_RENDERER_SELECTED);
 
-	rb_rating_render_stars (widget, window, cell_class->priv->pixbufs, 
-				draw_rect.x - pix_rect.x, 
-				draw_rect.y - pix_rect.y, 
+	rb_rating_render_stars (widget, window, cell_class->priv->pixbufs,
+				draw_rect.x - pix_rect.x,
+				draw_rect.y - pix_rect.y,
 				draw_rect.x, draw_rect.y,
-				cellrating->priv->rating, selected);	
+				cellrating->priv->rating, selected);
 }
 
 static gboolean
 rb_cell_renderer_rating_activate (GtkCellRenderer *cell,
-				  GdkEvent *event, 
+				  GdkEvent *event,
 				  GtkWidget *widget,
 				  const gchar *path,
 				  GdkRectangle *background_area,
@@ -295,19 +313,17 @@ rb_cell_renderer_rating_activate (GtkCellRenderer *cell,
 	g_return_val_if_fail (RB_IS_CELL_RENDERER_RATING (cellrating), FALSE);
 
 	gtk_widget_get_pointer (widget, &mouse_x, &mouse_y);
-	gtk_tree_view_widget_to_tree_coords (GTK_TREE_VIEW (widget),
-					     mouse_x,
-					     mouse_y,
-					     &mouse_x,
-					     &mouse_y);
+	gtk_tree_view_convert_widget_to_bin_window_coords (GTK_TREE_VIEW (widget),
+							   mouse_x, mouse_y,
+							   &mouse_x, &mouse_y);
 
-	rating = rb_rating_get_rating_from_widget (widget, 
+	rating = rb_rating_get_rating_from_widget (widget,
 						   mouse_x - cell_area->x,
 						   cell_area->width,
 						   cellrating->priv->rating);
 
 	if (rating != -1.0) {
-		g_signal_emit (G_OBJECT (cellrating), 
+		g_signal_emit (G_OBJECT (cellrating),
 			       rb_cell_renderer_rating_signals[RATED],
 			       0, path, rating);
 	}
