@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -23,10 +31,7 @@
 #ifndef __RB_ENTRY_VIEW_H
 #define __RB_ENTRY_VIEW_H
 
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtkdnd.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
+#include <gtk/gtk.h>
 
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
@@ -70,16 +75,19 @@ typedef enum {
 GType rb_entry_view_state_get_type (void);
 #define RB_TYPE_ENTRY_VIEW_STATE	(rb_entry_view_state_get_type())
 
+typedef struct _RBEntryView RBEntryView;
+typedef struct _RBEntryViewClass RBEntryViewClass;
+
 typedef struct RBEntryViewPrivate RBEntryViewPrivate;
 
-typedef struct
+struct _RBEntryView
 {
 	GtkScrolledWindow parent;
 
 	RBEntryViewPrivate *priv;
-} RBEntryView;
+};
 
-typedef struct
+struct _RBEntryViewClass
 {
 	GtkScrolledWindowClass parent;
 
@@ -94,15 +102,18 @@ typedef struct
 	void (*sort_order_changed)     (RBEntryView *view);
 
 	void (*show_popup)             (RBEntryView *view, gboolean over_entry);
-} RBEntryViewClass;
+};
 
 GType		rb_entry_view_get_type			(void);
 
 RBEntryView *	rb_entry_view_new			(RhythmDB *db,
                                                          GObject *shell_player,
 							 const char *sort_key,
-                                                         gboolean drag_source,
-							 gboolean drag_dest);
+                                                         gboolean is_drag_source,
+							 gboolean is_drag_dest);
+
+GtkTreeViewColumn *rb_entry_view_get_column		(RBEntryView *view,
+							 RBEntryViewColumn coltype);
 
 void		rb_entry_view_append_column		(RBEntryView *view,
 							 RBEntryViewColumn coltype,
@@ -163,8 +174,8 @@ void		rb_entry_view_set_sorting_order		(RBEntryView *view,
                                                          const char  *column_name,
                                                          gint         sort_order);
 /* deal with the sorting order as a composite string */
-const char *	rb_entry_view_get_sorting_type		(RBEntryView *view);
- void		rb_entry_view_set_sorting_type		(RBEntryView *view,
+char *		rb_entry_view_get_sorting_type		(RBEntryView *view);
+void		rb_entry_view_set_sorting_type		(RBEntryView *view,
                                                          const char  *sorttype);
 
 void		rb_entry_view_set_fixed_column_width	(RBEntryView *view,
