@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -69,7 +77,7 @@ static void limit_toggled_cb (GtkWidget *limit, RBQueryCreator *creator);
 
 static int get_property_index_from_proptype (const RBQueryCreatorPropertyOption *options,
 			    	  int length, RhythmDBPropType prop);
-static void sort_option_menu_changed (GtkOptionMenu *propmenu, RBQueryCreator *creator);
+static void sort_option_menu_changed (GtkComboBox *propmenu, RBQueryCreator *creator);
 
 typedef struct
 {
@@ -98,6 +106,24 @@ typedef struct
 G_DEFINE_TYPE (RBQueryCreator, rb_query_creator, GTK_TYPE_DIALOG)
 #define QUERY_CREATOR_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), rb_query_creator_get_type(), RBQueryCreatorPrivate))
 
+/**
+ * SECTION:rb-query-creator
+ * @short_description: database query creator widget
+ *
+ * The query creator is used to create and edit automatic playlists.
+ * It is only capable of constructing queries that consist of a flat
+ * list of criteria.  It cannot nested criteria or represent full 
+ * boolean logic expressions.
+ *
+ * In addition to query criteria, the query creator also allows the user
+ * to specify limits on the size of the result set, in terms of the number
+ * of entries, the total duration, or the total file size; and also the
+ * order in which the results are to be sorted.
+ *
+ * The structure of the query creator is defined in the glade file 
+ * create-playlist.xml.
+ */
+
 enum
 {
 	PROP_0,
@@ -115,6 +141,11 @@ rb_query_creator_class_init (RBQueryCreatorClass *klass)
 	object_class->set_property = rb_query_creator_set_property;
 	object_class->get_property = rb_query_creator_get_property;
 
+	/**
+	 * RBQueryCreator:db:
+	 *
+	 * The #RhythmDB instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_DB,
 					 g_param_spec_object ("db",
@@ -123,6 +154,11 @@ rb_query_creator_class_init (RBQueryCreatorClass *klass)
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * RBQueryCreator:creating:
+	 *
+	 * TRUE if a new playlist is being created.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_CREATING,
 					 g_param_spec_boolean ("creating",
@@ -196,6 +232,8 @@ rb_query_creator_constructor (GType type,
 	priv->sort_menu = GTK_WIDGET (glade_xml_get_widget (xml, "sortMenu"));
 	priv->sort_desc = GTK_WIDGET (glade_xml_get_widget (xml, "sortDesc"));
 
+	gtk_combo_box_set_active (GTK_COMBO_BOX (priv->limit_option), 0);
+
 	g_signal_connect_object (G_OBJECT (priv->limit_check), "toggled", G_CALLBACK (limit_toggled_cb),
 				 creator, 0);
 	limit_toggled_cb (priv->limit_check, creator);
@@ -295,6 +333,12 @@ rb_query_creator_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_query_creator_new:
+ * @db: the #RhythmDB instance
+ *
+ * Return value: new query creator widget
+ */
 GtkWidget *
 rb_query_creator_new (RhythmDB *db)
 {
@@ -313,6 +357,7 @@ rb_query_creator_load_query (RBQueryCreator *creator,
 	gboolean disjunction = FALSE;
 	RhythmDBQueryData *qdata;
 	GPtrArray *subquery;
+	guint64 limit;
 
 	g_return_val_if_fail (query->len == 2, FALSE);
 
@@ -321,18 +366,19 @@ rb_query_creator_load_query (RBQueryCreator *creator,
 
 	subquery = qdata->subquery;
 
-	if (subquery->len > 0)
+	if (subquery->len > 0) {
 		for (i = 0; i < subquery->len; i++) {
 			RhythmDBQueryData *data = g_ptr_array_index (subquery, i);
 			if (data->type != RHYTHMDB_QUERY_DISJUNCTION)
 				append_row (creator);
 		}
+	}
 
 	rows = priv->rows;
 
 	for (i = 0; i < subquery->len; i++) {
 		RhythmDBQueryData *data = g_ptr_array_index (subquery, i);
-		GtkOptionMenu *propmenu;
+		GtkComboBox *propmenu;
 		GtkWidget *criteria_menu;
 		int index;
 		const RBQueryCreatorPropertyType *property_type;
@@ -342,9 +388,9 @@ rb_query_creator_load_query (RBQueryCreator *creator,
 			continue;
 		}
 
-		propmenu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (rows->data), 0));
+		propmenu = GTK_COMBO_BOX (get_box_widget_at_pos (GTK_BOX (rows->data), 0));
 		index = get_property_index_from_proptype (property_options, num_property_options, data->propid);
-		gtk_option_menu_set_history (propmenu, index);
+		gtk_combo_box_set_active (propmenu, index);
 
 		criteria_menu = get_box_widget_at_pos (GTK_BOX (rows->data), 1);
 		select_criteria_from_value (creator, criteria_menu, data->propid, data->type);
@@ -358,48 +404,44 @@ rb_query_creator_load_query (RBQueryCreator *creator,
 	}
 
 	/* setup the limits */
-	{
-		guint64 limit;
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->disjunction_check),
+				      disjunction);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->limit_check),
+				      limit_type != RHYTHMDB_QUERY_MODEL_LIMIT_NONE);
 
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->disjunction_check),
-					      disjunction);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->limit_check),
-					      limit_type != RHYTHMDB_QUERY_MODEL_LIMIT_NONE);
+	switch (limit_type) {
+	case RHYTHMDB_QUERY_MODEL_LIMIT_NONE:
+		limit = 0;
+		break;
 
-		switch (limit_type) {
-		case RHYTHMDB_QUERY_MODEL_LIMIT_NONE:
-			limit = 0;
-			break;
+	case RHYTHMDB_QUERY_MODEL_LIMIT_COUNT:
+		gtk_combo_box_set_active (GTK_COMBO_BOX (priv->limit_option), 0);
+		limit = g_value_get_ulong (g_value_array_get_nth (limit_value, 0));
+		break;
 
-		case RHYTHMDB_QUERY_MODEL_LIMIT_COUNT:
-			gtk_option_menu_set_history (GTK_OPTION_MENU (priv->limit_option), 0);
-			limit = g_value_get_ulong (g_value_array_get_nth (limit_value, 0));
-			break;
+	case RHYTHMDB_QUERY_MODEL_LIMIT_TIME:
+		gtk_combo_box_set_active (GTK_COMBO_BOX (priv->limit_option), 3);
+		/* convert to minutes */
+		limit = g_value_get_ulong (g_value_array_get_nth (limit_value, 0)) / 60;
+		break;
 
-		case RHYTHMDB_QUERY_MODEL_LIMIT_TIME:
-			gtk_option_menu_set_history (GTK_OPTION_MENU (priv->limit_option), 3);
-			/* convert to minutes */
-			limit = g_value_get_ulong (g_value_array_get_nth (limit_value, 0)) / 60;
-			break;
+	case RHYTHMDB_QUERY_MODEL_LIMIT_SIZE:
+		limit = g_value_get_uint64 (g_value_array_get_nth (limit_value, 0));
 
-		case RHYTHMDB_QUERY_MODEL_LIMIT_SIZE:
-			limit = g_value_get_uint64 (g_value_array_get_nth (limit_value, 0));
-
-			if (limit % 1000 == 0) {
-				gtk_option_menu_set_history (GTK_OPTION_MENU (priv->limit_option), 2);
-				limit /= 1000;
-			} else {
-				gtk_option_menu_set_history (GTK_OPTION_MENU (priv->limit_option), 1);
-			}
-
-			break;
-		default:
-			g_assert_not_reached ();
+		if (limit % 1000 == 0) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (priv->limit_option), 2);
+			limit /= 1000;
+		} else {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (priv->limit_option), 1);
 		}
 
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (priv->limit_entry), limit);
+		break;
+	default:
+		g_assert_not_reached ();
 	}
 
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (priv->limit_entry), limit);
+
 	return TRUE;
 }
 
@@ -427,12 +469,26 @@ rb_query_creator_set_sorting (RBQueryCreator *creator,
 	/* check that it is a valid sort option */
 	g_return_val_if_fail (i < num_property_options, FALSE);
 
-	gtk_option_menu_set_history (GTK_OPTION_MENU (priv->sort_menu), i);
-	sort_option_menu_changed (GTK_OPTION_MENU (priv->sort_menu), creator); /* force the checkbox to change label */
+	gtk_combo_box_set_active (GTK_COMBO_BOX (priv->sort_menu), i);
+	sort_option_menu_changed (GTK_COMBO_BOX (priv->sort_menu), creator); /* force the checkbox to change label */
 
 	return TRUE;
 }
 
+/**
+ * rb_query_creator_new_from_query:
+ * @db: the #RhythmDB instance
+ * @query: an existing query to start from
+ * @limit_type: the type of result set limit
+ * @limit_value: the result set limit value
+ * @sort_column: the column on which to sort query results
+ * @sort_direction: the direction in which to sort query results
+ *
+ * Constructs a new query creator with an existing query and limit and sort
+ * settings.
+ *
+ * Return value: new query creator widget
+ */
 GtkWidget *
 rb_query_creator_new_from_query (RhythmDB *db,
                                  GPtrArray *query,
@@ -455,6 +511,15 @@ rb_query_creator_new_from_query (RhythmDB *db,
 	return GTK_WIDGET (creator);
 }
 
+/**
+ * get_box_widget_at_pos:
+ * @box: #GtkBox to extract child from
+ * @pos: index of the child to retrieve
+ *
+ * Extracts a child widget from a #GtkBox.
+ *
+ * Return value: child widget from the box
+ */
 GtkWidget *
 get_box_widget_at_pos (GtkBox *box, guint pos)
 {
@@ -491,6 +556,14 @@ get_entry_for_property (RBQueryCreator *creator,
 	return property_type->criteria_create_widget (constrain);
 }
 
+/**
+ * rb_query_creator_get_query:
+ * @creator: #RBQueryCreator instance
+ *
+ * Constructs a database query that represents the criteria in the query creator.
+ *
+ * Return value: database query array
+ */
 GPtrArray *
 rb_query_creator_get_query (RBQueryCreator *creator)
 {
@@ -509,14 +582,14 @@ rb_query_creator_get_query (RBQueryCreator *creator)
 	rows = priv->rows;
 
 	for (row = rows; row; row = row->next) {
-		GtkOptionMenu *propmenu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (row->data),
+		GtkComboBox *propmenu = GTK_COMBO_BOX (get_box_widget_at_pos (GTK_BOX (row->data),
 										  0));
-		GtkOptionMenu *criteria_menu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (row->data),
+		GtkComboBox *criteria_menu = GTK_COMBO_BOX (get_box_widget_at_pos (GTK_BOX (row->data),
 										       1));
-		guint prop_position = gtk_option_menu_get_history (propmenu);
+		guint prop_position = gtk_combo_box_get_active (propmenu);
 		const RBQueryCreatorPropertyOption *prop_option = &property_options[prop_position];
 		const RBQueryCreatorCriteriaOption *criteria_options = prop_option->property_type->criteria_options;
-		const RBQueryCreatorCriteriaOption *criteria_option = &criteria_options[gtk_option_menu_get_history (criteria_menu)];
+		const RBQueryCreatorCriteriaOption *criteria_option = &criteria_options[gtk_combo_box_get_active (criteria_menu)];
 
 		g_assert (prop_option->property_type->criteria_get_widget_data != NULL);
 		{
@@ -550,6 +623,16 @@ rb_query_creator_get_query (RBQueryCreator *creator)
 	return query;
 }
 
+/**
+ * rb_query_creator_get_limit:
+ * @creator: #RBQueryCreator instance
+ * @type: used to return the limit type
+ * @limit: used to return the limit value
+ *
+ * Retrieves the limit type and value from the query creator.
+ * The limit value is returned as the first element in a
+ * #GValueArray.
+ */
 void
 rb_query_creator_get_limit (RBQueryCreator *creator,
 			    RhythmDBQueryModelLimitType *type,
@@ -567,7 +650,7 @@ rb_query_creator_get_limit (RBQueryCreator *creator,
 		l = gtk_spin_button_get_value(GTK_SPIN_BUTTON (priv->limit_entry));
 		*limit = g_value_array_new (0);
 
-		switch (gtk_option_menu_get_history (GTK_OPTION_MENU (priv->limit_option))) {
+		switch (gtk_combo_box_get_active (GTK_COMBO_BOX (priv->limit_option))) {
 		case 0:
 			*type = RHYTHMDB_QUERY_MODEL_LIMIT_COUNT;
 			rb_value_array_append_data (*limit, G_TYPE_ULONG, (gulong)l);
@@ -579,7 +662,7 @@ rb_query_creator_get_limit (RBQueryCreator *creator,
 
 		case 2:
 			*type = RHYTHMDB_QUERY_MODEL_LIMIT_SIZE;
-			rb_value_array_append_data (*limit, G_TYPE_ULONG, (gulong)l);
+			rb_value_array_append_data (*limit, G_TYPE_UINT64, l * 1000);
 			break;
 
 		case 3:
@@ -596,9 +679,18 @@ rb_query_creator_get_limit (RBQueryCreator *creator,
 	}
 }
 
+/**
+ * rb_query_creator_get_sort_order:
+ * @creator: #RBQueryCreator instance
+ * @sort_key: returns the sort key name
+ * @sort_direction: returns the sort direction
+ *
+ * Retrieves the sort settings from the query creator.
+ * The sort direction is returned as a #GtkSortType value.
+ */
 void
 rb_query_creator_get_sort_order (RBQueryCreator *creator,
-                                 const char **sort_column,
+                                 const char **sort_key,
                                  gint *sort_direction)
 {
 	RBQueryCreatorPrivate *priv;
@@ -614,10 +706,10 @@ rb_query_creator_get_sort_order (RBQueryCreator *creator,
 			*sort_direction = GTK_SORT_ASCENDING;
 	}
 
-	if (sort_column != NULL) {
+	if (sort_key != NULL) {
 		int i;
-		i = gtk_option_menu_get_history (GTK_OPTION_MENU (priv->sort_menu));
-		*sort_column = sort_options[i].sort_key;
+		i = gtk_combo_box_get_active (GTK_COMBO_BOX (priv->sort_menu));
+		*sort_key = sort_options[i].sort_key;
 	}
 }
 
@@ -695,30 +787,30 @@ append_row (RBQueryCreator *creator)
 	len = g_list_length (rows);
 
 	hbox = GTK_BOX (gtk_hbox_new (FALSE, 5));
-	gtk_box_pack_start_defaults (GTK_BOX (priv->vbox), GTK_WIDGET (hbox));
+	gtk_box_pack_start (GTK_BOX (priv->vbox), GTK_WIDGET (hbox), TRUE, TRUE, 0);
 	priv->rows = g_list_prepend (priv->rows, hbox);
 	gtk_box_reorder_child (priv->vbox, GTK_WIDGET (hbox), -1);
 
-	/* This is the main (leftmost) GtkOptionMenu, for types. */
+	/* This is the main (leftmost) GtkComboBox, for types. */
 	option = create_property_option_menu (creator, property_options, num_property_options);
 	gtk_size_group_add_widget (priv->property_size_group, option);
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (option));
-	gtk_option_menu_set_history (GTK_OPTION_MENU (option), 0);
+	gtk_box_pack_start (hbox, GTK_WIDGET (option), TRUE, TRUE, 0);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (option), 0);
 	criteria = create_criteria_option_menu (property_options[0].property_type->criteria_options,
 						property_options[0].property_type->num_criteria_options);
 	gtk_size_group_add_widget (priv->criteria_size_group, criteria);
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (criteria));
+	gtk_box_pack_start (hbox, GTK_WIDGET (criteria), TRUE, TRUE, 0);
 
 	entry = get_entry_for_property (creator, property_options[0].strict_val, &constrain);
 	if (constrain)
 		gtk_size_group_add_widget (priv->entry_size_group, entry);
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (entry));
+	gtk_box_pack_start (hbox, GTK_WIDGET (entry), TRUE, TRUE, 0);
 
 	remove_button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
 	g_signal_connect_object (G_OBJECT (remove_button), "clicked", G_CALLBACK (remove_button_click_cb),
 				 creator, 0);
 	gtk_size_group_add_widget (priv->button_size_group, remove_button);
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (remove_button));
+	gtk_box_pack_start (hbox, GTK_WIDGET (remove_button), TRUE, TRUE, 0);
 
 	gtk_widget_show_all (GTK_WIDGET (priv->vbox));
 	return GTK_WIDGET (hbox);
@@ -754,7 +846,7 @@ select_criteria_from_value (RBQueryCreator *creator,
 
 	for (i = 0; i < length; i++) {
 		if (qtype == options[i].val) {
-			gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), i);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (option_menu), i);
 			return;
 		}
 	}
@@ -762,7 +854,7 @@ select_criteria_from_value (RBQueryCreator *creator,
 }
 
 static void
-property_option_menu_changed (GtkOptionMenu *propmenu,
+property_option_menu_changed (GtkComboBox *propmenu,
 			      RBQueryCreator *creator)
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
@@ -775,7 +867,7 @@ property_option_menu_changed (GtkOptionMenu *propmenu,
 	guint old_value;
 	gboolean constrain;
 
-	prop_option = &property_options[gtk_option_menu_get_history (propmenu)];
+	prop_option = &property_options[gtk_combo_box_get_active (propmenu)];
 	old_value = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (propmenu), "prop-menu old-value"));
 
 	/* don't recreate the criteria menu and entry if they will be the same*/
@@ -783,7 +875,7 @@ property_option_menu_changed (GtkOptionMenu *propmenu,
 		return;
 
 	g_object_set_data (G_OBJECT (propmenu), "prop-menu old-value",
-			   GINT_TO_POINTER (gtk_option_menu_get_history (propmenu)));
+			   GINT_TO_POINTER (gtk_combo_box_get_active (propmenu)));
 
 	row = lookup_row_by_widget (creator, GTK_WIDGET (propmenu));
 
@@ -796,7 +888,7 @@ property_option_menu_changed (GtkOptionMenu *propmenu,
 	criteria = create_criteria_option_menu (criteria_options, length);
 	gtk_widget_show (criteria);
 	gtk_size_group_add_widget (priv->criteria_size_group, criteria);
-	gtk_box_pack_start_defaults (GTK_BOX (row), GTK_WIDGET (criteria));
+	gtk_box_pack_start (GTK_BOX (row), GTK_WIDGET (criteria), TRUE, TRUE, 0);
 	gtk_box_reorder_child (GTK_BOX (row), criteria, 1);
 
 	entry = get_box_widget_at_pos (GTK_BOX (row), 2);
@@ -807,7 +899,7 @@ property_option_menu_changed (GtkOptionMenu *propmenu,
 
 	if (constrain)
 		gtk_size_group_add_widget (priv->entry_size_group, entry);
-	gtk_box_pack_start_defaults (GTK_BOX (row), GTK_WIDGET (entry));
+	gtk_box_pack_start (GTK_BOX (row), GTK_WIDGET (entry), TRUE, TRUE, 0);
 	gtk_box_reorder_child (GTK_BOX (row), entry, 2);
 }
 
@@ -816,63 +908,46 @@ create_property_option_menu (RBQueryCreator *creator,
                              const RBQueryCreatorPropertyOption *options,
 			     int length)
 {
-	GtkWidget *option_menu;
-	GtkWidget *menu;
-	GtkWidget *menu_item;
+	GtkWidget *combo;
 	int i;
 
-	option_menu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-
-	/* add the property options */
+	combo = gtk_combo_box_new_text ();
 	for (i = 0; i < length; i++) {
-		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-		gtk_widget_show (menu_item);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(options[i].name));
 	}
 
-		gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
-	g_object_set_data (G_OBJECT (option_menu), "prop-menu old-value", GINT_TO_POINTER (0));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
+	
+	g_object_set_data (G_OBJECT (combo), "prop-menu old value", GINT_TO_POINTER (0));
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-	gtk_widget_show (menu);
-
-	g_signal_connect_object (G_OBJECT (option_menu), "changed",
+	g_signal_connect_object (G_OBJECT (combo), "changed",
 				 G_CALLBACK (property_option_menu_changed), creator, 0);
 
-	return option_menu;
+	return combo;
 }
 
 static GtkWidget*
 create_criteria_option_menu (const RBQueryCreatorCriteriaOption *options,
 			     int length)
 {
-	GtkWidget *option_menu;
-	GtkWidget *menu;
-	GtkWidget *menu_item;
+	GtkWidget *combo;
 	int i;
 
-	option_menu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-
+	combo = gtk_combo_box_new_text ();
 	for (i = 0; i < length; i++) {
-		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-		gtk_widget_show (menu_item);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(options[i].name));
 	}
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-	gtk_widget_show (menu);
-
-	return option_menu;
+	return combo;
 }
 
 static void
-sort_option_menu_changed (GtkOptionMenu *propmenu,
+sort_option_menu_changed (GtkComboBox *propmenu,
 			  RBQueryCreator *creator)
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
-	int index = gtk_option_menu_get_history (propmenu);
+	int index = gtk_combo_box_get_active (propmenu);
 
 	gtk_button_set_label (GTK_BUTTON (priv->sort_desc), _(sort_options[index].sort_descending_name));
 	rb_debug("changing descending label to %s[%d]", sort_options[index].sort_descending_name, index);
@@ -884,21 +959,21 @@ setup_sort_option_menu (RBQueryCreator *creator,
 			const RBQueryCreatorSortOption *options,
 			int length)
 {
-	GtkWidget *menu;
-	GtkWidget *menu_item;
+	GtkListStore *store;
 	int i;
 
-	menu = gtk_menu_new ();
-	gtk_widget_show (menu);
+	store = gtk_list_store_new (1, G_TYPE_STRING);
 
 	for (i = 0; i < length; i++) {
-		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-		gtk_widget_show (menu_item);
+		GtkTreeIter iter;
+
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, _(options[i].name), -1);
 	}
 
+	gtk_combo_box_set_model (GTK_COMBO_BOX (option_menu), GTK_TREE_MODEL (store));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (option_menu), 0);
+
 	g_signal_connect_object (G_OBJECT (option_menu), "changed",
 				 G_CALLBACK (sort_option_menu_changed), creator, 0);
-
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
 }
