@@ -541,6 +541,7 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 				 G_OBJECT (song_info), 0);
 	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "song_info_rating_container")),
 			   song_info->priv->rating);
+	g_object_set (glade_xml_get_widget (xml, "rating_label"), "mnemonic-widget", song_info->priv->rating, NULL);
 
 	/* add relationship between the rating label and the rating widget */
 	lobj = gtk_widget_get_accessible (glade_xml_get_widget (xml, "rating_label"));
@@ -1044,14 +1045,19 @@ rb_song_info_update_playback_error (RBSongInfo *song_info)
 static void
 rb_song_info_update_bitrate (RBSongInfo *song_info)
 {
-	char *tmp = NULL;
-	gulong bitrate = 0;
+	char *tmp;
+	gulong bitrate;
+
 	bitrate = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_BITRATE);
 
-	if (bitrate > 0)
-		tmp = g_strdup_printf (_("%lu kbps"), bitrate);
-	else
+	if (rhythmdb_entry_is_lossless (song_info->priv->current_entry)) {
+		tmp = g_strdup (_("Lossless"));
+	} else if (bitrate == 0) {
 		tmp = g_strdup (_("Unknown"));
+	} else {
+		tmp = g_strdup_printf (_("%lu kbps"), bitrate);
+	}
+
 	gtk_label_set_text (GTK_LABEL (song_info->priv->bitrate),
 			    tmp);
 	g_free (tmp);
