@@ -9,6 +9,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -28,13 +36,20 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include "rb-uri-dialog.h"
 #include "rb-glade-helpers.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
 
+/**
+ * SECTION:rb-uri-dialog
+ * @short_description: simple URI entry dialog
+ * @include: rb-uri-dialog.h
+ *
+ * A simple dialog used to request a single URI from the user.
+ */
+
 static void rb_uri_dialog_class_init (RBURIDialogClass *klass);
 static void rb_uri_dialog_init (RBURIDialog *dialog);
 static void rb_uri_dialog_finalize (GObject *object);
@@ -87,6 +102,11 @@ rb_uri_dialog_class_init (RBURIDialogClass *klass)
 	object_class->set_property = rb_uri_dialog_set_property;
 	object_class->get_property = rb_uri_dialog_get_property;
 
+	/**
+	 * RBURIDialog:label:
+	 *
+	 * The label displayed in the dialog.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_LABEL,
 					 g_param_spec_string ("label",
@@ -95,6 +115,13 @@ rb_uri_dialog_class_init (RBURIDialogClass *klass)
 							      "",
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * RBURIDialog::location-added:
+	 * @dialog: the #RBURIDialog
+	 * @uri: URI entered
+	 *
+	 * Emitted when the user has entered a URI into the dialog.
+	 */
 	rb_uri_dialog_signals [LOCATION_ADDED] =
 		g_signal_new ("location-added",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -211,6 +238,15 @@ rb_uri_dialog_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_uri_dialog_new:
+ * @title: Window title for the dialog
+ * @label: Label to display in the dialog
+ *
+ * Creates a URI entry dialog.
+ *
+ * Returns: URI dialog instance.
+ */
 GtkWidget *
 rb_uri_dialog_new (const char *title, const char *label)
 {
